{ FoxClone
  Copyright (C) 2021 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}
{-------------------------------------------------------------------------------
UNIT - ABOUT

This unit defines the panel used to display the about messages (credits and licence) on the about tab.

V1.0 - 11 Jun 2021
-------------------------------------------------------------------------------}
unit about;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, StdCtrls, Controls, ExtCtrls, Graphics, Dialogs, LCLtype, GTKlabel,
  utilities, il8nstrings;
type
  TAboutPanel = class(TPanel)
    const
      phBorder = 10; //default border unit for gaps vertically in the panel
      pwBorder = 15; //default border unit for gaps horizontally in the panel
      RightMargin = 150;
    private
      FoxImg : TImage; //image top right in the panel
      FoxText : TMemo; //text of the about screen - what foxclone is, copyright, usage
    public
      TitleLbl : TLabelGTK; //title for the top of the tab public so that changing font size works
      constructor Create(TheOwner: TComponent);
      procedure PanelResize;
  end;

var
  AboutTab : TAboutPanel;

implementation
{-------------------------------------------------------------------------------
CONSTRUCTOR TABOUTPANEL.CREATE
Constructor for the about tab.

Initially tried a TLabel for the text but found with wordwrap on, line spacing was cramped compared
to earlier versions of foxclone where line breaks had been inserted manually - line spacing, or leading,
looked nice with manual line breaks. There is no control to set line height in a tlabel when the
content is multi-line.

Then switched to a TMemo, line spacing is slightly better than the TLabel. Initial problem with this
was that default background is white. After messing around found that simply setting enabled = false
solved the problem. Odd - was expecting the text to be greyed with the object disabled, but, no, it
stays black. Not the same behaviour as a TLabel (or TCheckList or ListBox) where the text is greyed
when the object is disabled.

Used by:

V1.0 - 11 Jun 2021
V1.1 - 01 Jan 2022, set colour on panel for visual appearance in iso.
V1.2 - 10 Jan 2022, set colour on tmemo and readonly for appearance also enabled true.
-------------------------------------------------------------------------------}
constructor TAboutPanel.Create(TheOwner: TComponent);
const
  phDefault = 486; //client height on a 600h main window - now wrong as wGap increased   was 526
  pwDefault = 794; //client width on a 800w main window
begin
  inherited Create(TheOwner);
  Color:= clForm; //required for visual appearance in the iso
  FoxImg:=TImage.Create(Self);
  with FoxImg do begin
       Parent:=self;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= self;
       BorderSpacing.top:= phBorder * 3;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= self;
       BorderSpacing.left:= pwborder;
       Picture.LoadFromFile(AboutImgFile);
       AutoSize:= true;
  end;
  TitleLbl:= TLabelGTK.Create(Self);
  with TitleLbl do begin
       Parent:=self;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= FoxImg;
       BorderSpacing.top:= phBorder;
       AnchorSide[akLeft].Side:= asrRight;
       AnchorSide[akLeft].Control:= FoxImg;
       BorderSpacing.left:= pwborder;
       Font.Size:= DefaultFontSize + 4;
       setCaption('Foxclone');
  end;
  FoxText:= TMemo.Create(Self);
  with FoxText do begin
       Parent:=self;
       Color:= clForm;
       Enabled:= true;
       ReadOnly:= true;
       BorderStyle:= bsNone;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= TitleLbl;
       BorderSpacing.top:= phBorder;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= TitleLbl;
       BorderSpacing.left:= 0;
       Width:= pwDefault - FoxImg.Width - 2 * pwborder - RightMargin;
       WordWrap:= true;
       Lines.Add(rAboutStr);
       Lines.Add('');
       Lines.Add(rCopyRightStr);
       Lines.Add('');
       Lines.Add(rImageStr);
       Lines.Add('');
       Lines.Add('');
       Lines.Add(rLicenceStr1);
       Lines.Add('');
       Lines.Add(rLicenceStr2);
       Lines.Add('');
       Lines.Add(rLicenceStr3);
       Height:= phDefault - phborder * 5 - TitleLbl.Height;
  end;
end; //end constructor TAboutPanel.Create
{-------------------------------------------------------------------------------
PROCEDURE TABOUTPANEL.PANELRESIZE
Simple, it just has to recalculate the width of the TMemo. The height is fixed and the main window
cannot be reduced in height below the initial default.

Used by:

V1.0 - 11 Jun 2021
-------------------------------------------------------------------------------}
procedure TAboutPanel.PanelResize;
begin
  FoxText.Width:= Self.Width - FoxImg.Width - 2 * pwborder - RightMargin;
end; //end procedure TAboutPanel.PanelResize

end.

