{ FoxClone
  Copyright (C) 2021 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}
{-------------------------------------------------------------------------------
UNIT - BASEPANEL

Each tab on the mainform has a panel specific to the function, backup, restore, etc. This unit defines
the basepanel, all other panels are children, modified as appropriate. The backup panel is largely
unchanged from basepanel.

It contains the methods and type definitions common to all panels.

//https://wiki.freepascal.org/Autosize_/_Layout
//https://lazarus-ccr.sourceforge.io/docs/lcl/controls/anchoring%20controls.html

V1.0 - 10 Dec 2021
-------------------------------------------------------------------------------}
unit basepanel;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, StdCtrls, Controls, CheckLst, ExtCtrls, Graphics, Dialogs, LCLtype, GTKlabel,
  utilities, il8nstrings, drives;

type
  //used in the hint functions to determine the hintstr to be used
  TBoxType = (bkBox1,bkBox2,bkBox3,bkBox4,reBox1,reBox2,reBox3,reBox4,clBox1,clBox1File,
              clBox2,clBox2File,clBox3,clBox4,veBox1,veBox2,veBox3);

  //modified TCheckListBox to include TBoxType
  TFoxCheckBox = class(TCheckListBox)
    public //tried protected but could not set boxtype in the restore unit
      BoxType : TBoxType;
      //having found out how the onclick hooks work (or not) -
      //https://forum.lazarus.freepascal.org/index.php?topic=54622.msg405980#msg405980
      //this boolean array is used to mirror the state of items in a TCheckListBox so that the
      //onclick handler can work with different behaviour from clicking an item and clicking a checkbox.
      //Each is populated by the procedure that populates the TCheckListbox.
      CheckState : array of boolean;
  end;

  TFoxListBox = class(TListBox)
    public
      BoxType : TBoxType;
  end;

  TBasePanel = class(TPanel)
  const
    phBottom = 68; //space at bottom for Tbuttons, declared here because used in resize
    phBorder = 10; //default border unit for gaps vertically in the panel
    pwBorder = 15; //default border unit for gaps horizontally in the panel
    ItemHeightAdjust = 4; //adjustment to ItemHeight to match appearance without a custom ondrawitem
                          //ALL checklist and listboxes have itemheight set.
  protected
    BoxItemHeight : integer; //all items in boxes have the same height
    ActionImg : TImage; //image top right in the panel
    TitleLbl : TLabelGTK; //title for the top of the tab
    CloneComboBox : TComboBox; //clone direct or clone from backup file

    Box1Lbl, Box2Lbl, Box3Lbl, Box4Lbl : TLabelGTK; //titles for the checklist boxes
    RExisting : TLabelGTK; //label below box4 on a restore
    Box1, Box2, Box3, Box4: TFoxCheckBox;  //the checklist boxes
    LBox4 : TFoxListBox; //LBox4 used on the restore tab

    NextBtn : TButton; //save to file in backup

    procedure clearBox(index : byte); //index is the box number
    procedure setScrollBarTBox(aBox : TFoxCheckBox);
    procedure setScrollBarLBox(aBox : TListBox);
    procedure setScrollBars;
    procedure DriveHint(Sender : TObject; Shift : TShiftState; X,Y : Integer);
    procedure PartHint(Sender : TObject; Shift : TShiftState; X,Y : Integer);
    procedure BoldOS(Control : TWinControl; Index : Integer; ARect : TRect; State : TOwnerDrawState);
    procedure LBoxHint(Sender : TObject; Shift : TShiftState; X,Y : Integer);

    procedure BoxMouseLeave(Sender: TObject);

  public
    constructor Create(TheOwner: TComponent);
    procedure PanelResize;
    procedure PopulateBox1;
    procedure FontResize;
    property PartBox : TFoxCheckBox read Box3; //added so confform can access parts selected for restore

end;


implementation
uses mainform, restore, clone, verify;  //restore & clone needed for font resize
{-------------------------------------------------------------------------------
TBASEPANEL.CONSTRUCTOR
Constructor for the template panel used by backup/restore/clone and verify.

Creates all the objects for the tabs with the exception of the listboxes on the restore tab.

CheckBoxes/ListBoxes 2 - 4 are disabled.

It does not populate any boxes with information. At the time of creation no drive/partition information
exists.

Used by:

V1.3 - 21 Feb 2021 - added resourcestrings to message dialog
V1.4 - 02 Mar 2021 - added drives to box1 and set up scrollbars on all boxes
V1.5 - 27 Mar 2021 - nothing is populated and scrollbars are set separately
V1.6 - 29 Mar 2021 - lbOwnerDraw set on box2 so that OS can be shown in bold
V1.7 - 28 Apr 2021 - using custom draw to get the font bold on parts with an OS results in the item
                     height of boxes with partitions having a different item height to the default item
                     height for a drive box. Also the item height of the list box on restore appears
                     different. Easiest solution was determine the text height of the default font,
                     and set item height a bit higher (ItemHeightAdjust) so that it matches a box with
                     no custom draw. Then fix the item height for ALL boxes.
V1.8 - 25 Jun 2021 - const ItemHeightAdjust moved into basepanel type definition
V1.9 - 24 Jul 2021 - modded to check width of text strings for nextbutton and set to largest. Height set
                     to text height plus 14, was set at 30. Changes so that appearance can be
                     consistent when changing font size.
V1.10 - 28 Jul 2021 - InfoImg2 object removed.
V1.11 - 09 Dec 2021 - Problems with visual appearance in the iso, finally fixed by setting the colour
                      of the panel to clForm (after trying many of the other colours. See
                      https://wiki.freepascal.org/Colors#System_colors.
V1.12 - 12 Jan 2022 - event handler added for onmouseleave to checklistboxes. To improve visual
                      consistency across drive and partition checklistboxes. Removes highlighting on
                      the box when you leave it.
-------------------------------------------------------------------------------}
constructor TBasePanel.Create(TheOwner: TComponent);
const
  phMainTitle = 20; //default 14pt ubuntu
  phBoxTitle = 10; //default 10pt ubuntu
  phDefault = 486; //client height on a 600h main window - now wrong as wGap increased   was 526
  pwDefault = 794; //client width on a 800w main window
var
  BoxHeight, BoxWidth : integer;
begin
  BoxHeight:= (phDefault - phMainTitle - 2 * phBoxTitle - 10 * phBorder - phBottom) div 2;
  //Box width, 3 border units, what's left is two boxes
  BoxWidth:= (pwDefault - 3 * pwBorder) div 2;
  BoxItemHeight:= GetTextHeight('AHgp',MainFm.Font) + ItemHeightAdjust;

  inherited Create(TheOwner);
  //Enabled:= true; makes no difference
  Color:= clForm; //fix for appearence in the iso
  // Image objects on the top right - changed from earlier versions, now just a single image per tab
  ActionImg:=TImage.Create(Self);
  with ActionImg do begin
       Parent:=self;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= self;
       BorderSpacing.top:= phBorder;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= self;
       BorderSpacing.left:= wWidth - BackupImgWidth;
       BorderSpacing.right:= 0;
       Picture.LoadFromFile(BackupImgFile);
       AutoSize:= true;
  end;
  // Title label
  TitleLbl:= TLabelGTK.Create(Self);
  with TitleLbl do begin
       Parent:= self;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= self;
       BorderSpacing.top:= phBorder * 2;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= self;
       BorderSpacing.left:= pwBorder;
       Font.Size:= DefaultFontSize + 4; //presumably because it is set it doesnt change when font size changed
       setCaption(rBackTitleStr);
  end;
  // Box1 Title
  Box1Lbl:= TLabelGTK.Create(Self);
  with Box1Lbl do begin
       Parent:= self;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= TitleLbl;
       BorderSpacing.top:= phBorder * 4;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= TitleLbl;
       BorderSpacing.left:= 0;
       setCaption(rBackBox1Str);
  end;
  // check box 1
  Box1:= TFoxCheckBox.Create(Self);
  with Box1 do begin
       Parent:= self;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= Box1Lbl;
       BorderSpacing.top:= phBorder;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Box1Lbl;
       BorderSpacing.left:= 0;
       Width:= BoxWidth;
       Height:= BoxHeight;
       ItemHeight:= BoxItemHeight; //added in v1.7
       ExtendedSelect:= false; //stops shift plus click
       MultiSelect:= false;//stops ctrl plus click
       ShowHint:= true;
       ScrollWidth:= width;
       OnMouseMove:= @DriveHint;
       OnMouseLeave:= @BoxMouseLeave;
  end;
  // Box2 Title
  Box2Lbl:= TLabelGTK.Create(Self);
  with Box2Lbl do begin
       Parent:= self;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= Box1Lbl;
       BorderSpacing.top:= 0;
       AnchorSide[akLeft].Side:= asrRight;
       AnchorSide[akLeft].Control:= Box1;
       BorderSpacing.left:= pwBorder;
       setCaption(rBackBox2Str);
       Enabled:= false;
  end;
  // check box 2
  Box2:= TFoxCheckBox.Create(Self);
  with Box2 do begin
       Parent:= self;
       ItemHeight:= BoxItemHeight;
       Style:= lbOwnerDrawFixed;//lbOwnerDrawFixed; what is the difference - the item height
       OnDrawItem:= @BoldOS;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= Box2Lbl;
       BorderSpacing.top:= phBorder;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Box2Lbl;
       BorderSpacing.left:= 0;
       Width := Box1.Width;
       Height := Box1.Height;
       ItemHeight:= BoxItemHeight; //added in v1.7
       ExtendedSelect:= false; //stops shift plus click
       MultiSelect:= false;//stops ctrl plus click
       ShowHint:= true;
       OnMouseMove:= @PartHint; //to get custom hints
       OnMouseLeave:= @BoxMouseLeave;
       Enabled:= false;
  end;
  // Box3 Title
  Box3Lbl:= TLabelGTK.Create(Self);
  with Box3Lbl do begin
       Parent:= self;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= Box1;
       BorderSpacing.top:= phBorder * 2;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= TitleLbl;
       BorderSpacing.left:= 0;
       setCaption(rBackBox3Str);
       Enabled:= false;
  end;
  // check box 3
  Box3:= TFoxCheckBox.Create(Self);
  with Box3 do begin
       Parent:= self;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= Box3Lbl; //anchor the top of box3 to the bottom of box3lbl
       BorderSpacing.top:= phBorder;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Box1Lbl; //anchor the left of box3 to the left side of box3lbl
       BorderSpacing.left:= 0;
       Width := Box1.Width;
       Height := Box1.Height;
       ItemHeight:= BoxItemHeight; //added in v1.7
       ExtendedSelect:= false; //stops shift plus click
       MultiSelect:= false;//stops ctrl plus click
       Enabled:= false;
       ShowHint:= true;
       ScrollWidth:= width;
       OnMouseMove:= @DriveHint;
       OnMouseLeave:= @BoxMouseLeave;
  end;
  // Box4 Title
  Box4Lbl:= TLabelGTK.Create(Self);
  with Box4Lbl do begin
       Parent:= self;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= Box3Lbl;
       BorderSpacing.top:= 0;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Box2;
       BorderSpacing.left:= 0;
       setCaption(rBackBox4Str);
       Enabled:= false;
  end;
  // check box 4
  Box4:= TFoxCheckBox.Create(Self);
  with Box4 do begin
       Parent:= self;
       Style:= lbOwnerDrawFixed;//lbOwnerDrawFixed; what is the difference - the item height
       OnDrawItem:= @BoldOS;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= Box4Lbl;
       BorderSpacing.top:= phBorder;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Box2;
       BorderSpacing.left:= 0;
       Width:= Box1.Width;
       Height:= Box1.Height;
       ItemHeight:= BoxItemHeight; //added in v1.7
       ExtendedSelect:= false; //stops shift plus click
       MultiSelect:= false;//stops ctrl plus click
       ShowHint:= true;
       OnMouseMove:= @PartHint; //to get custom hints
       OnMouseLeave:= @BoxMouseLeave;
       Enabled:= false;
  end;
  // Next button
  NextBtn:= TButton.Create(Self);
  with NextBtn do begin
       Parent:= self;
       Caption:= rNextStr;
       Height:= GetTextHeight(rNextStr,font) + 14;
       Width:= GetTextWidth(rNextStr,font);
       if GetTextWidth(rRestTabStr,font) > Width then Width:= GetTextWidth(rRestTabStr,font);
       if GetTextWidth(rCloneTabStr,font) > Width then Width:= GetTextWidth(rCloneTabStr,font);
       if GetTextWidth(rVerifyTabStr,font) > Width then Width:= GetTextWidth(rVerifyTabStr,font);
       Width:= Width + 20; //manual adjust to get about same as quit button
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= Box4;
       BorderSpacing.top:= phBorder * 2;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= self;
       BorderSpacing.left:= pwDefault - Width - pwBorder;
       Enabled:= false;
  end;
end; //constructor TBasePanel.CreateNew
{-------------------------------------------------------------------------------
PROCEDURE TBASEPANEL.SETSCROLLBARTBOX
Turns on/off horizontal scroll bars for a checklistbox by comparing the width of
the box against the width of the text (in pixels) using GetTextWidth.

Introduced after I found that a single procedure setting scrollbars would not work for the different
tabs as the objects on each tab are different, e.g. checklistboxes on backup but a mix of
checklistboxes and listboxes on restore. Writing a single procedure covering all cases wouldnt work
as freeandnil is throwing SIGSEGV errors. And you need freeandnil to be able to test if object
exists with assigned().

Initially this was declared as private to TBasePanel, but then when used in TBack it could not be
found? Declared public and all okay. Is this because TBack is in a separate unit?

Parameters:
    aBox - the TFoxCheckBox to set scroll bars on.

Used by:

V1.0 - 26 Mar 2021
-------------------------------------------------------------------------------}
procedure TBasePanel.setScrollBarTBox(aBox : TFoxCheckBox);
const
  checksize = 40; //width of the checkbox in a TCheckListBox - determined by trial & error
var
  i : shortint;
  max : integer = 0;
begin
  if aBox.Count < 0 then aBox.ScrollWidth:= aBox.Width else
     begin
       for i:= 0 to aBox.Count - 1 do with aBox do
           if GetTextWidth(Items[i],Font) > max then max:= GetTextWidth(Items[i],Font);
       aBox.ScrollWidth:= max + checksize;
     end;
end; //end procedure TBasePanel.setScrollBarTBox
{-------------------------------------------------------------------------------
PROCEDURE TBASEPANEL.SETSCROLLBARLBOX
Turns on/off horizontal scroll bars for a checklistbox by comparing the width of
the box against the width of the text (in pixels) using GetTextWidth.

As setScrollBarTBox with minor mods for a ListBox.

Parameters:
    aBox - the TListbox to set scroll bars on.

Used by:

V1.0 - 26 Mar 2021
-------------------------------------------------------------------------------}
procedure TBasePanel.setScrollBarLBox(aBox : TListBox);
var
  i : shortint;
  max : integer = 0;
begin
  if aBox.Count < 0 then aBox.ScrollWidth:= aBox.Width else
     begin
       for i:= 0 to aBox.Count - 1 do with aBox do
           if GetTextWidth(Items[i],Font) > max then max:= GetTextWidth(Items[i],Font);
       aBox.ScrollWidth:= max;
     end;
end; //end procedure TBasePanel.setScrollBarLBox
{-------------------------------------------------------------------------------
PROCEDURE TBASEPANEL.SETSCROLLBARS
Generic procedure to reset scrollbars for each tab. Does it by calling either setScrollBarTBox for
TFoxCheckBoxes or setScrollBarLBox for TListBoxes.

Much toing and froing on this as had problems with freeandnil causing SIGSEGV errors, but think I
must have been trying to free the wrong objects.

Used by:

V1.0 - 27 Mar 2021
V1.1 - 02 Jun 2021, modified for clone tab - checking for Box3 and LBox3
V1.2 - 28 Sep 2021, references to LBox3 removed, now a tchecklistbox
-------------------------------------------------------------------------------}
procedure TBasePanel.setScrollBars;
begin
  setScrollBarTBox(Box1);
  setScrollBarTBox(Box2);
  if assigned(Box3) then setScrollBarTBox(Box3);
  if assigned(Box4) then setScrollBarTBox(Box4);
  if assigned(LBox4) then setScrollBarLBox(LBox4);
end; //end procedure TBasePanel.setScrollBars
{-------------------------------------------------------------------------------
PROCEDURE TBASEPANEL.PANELRESIZE
A generic procedure to resize objects on each of the tabs. Triggered when a resize is triggered on
the main form.

A lot of messing around because of problems with freeandnil, think I must have being to free the wrong
objects. Uses assigned to check if specific objects exist on each tab.

Used by: TMainFm.MainFormResize

V1.0 - 27 Mar 2021
V1.1 - 02 Jun 2021, modified for clonetab
V1.2 - 20 Jun 2021, changed to resize spacing on the images so that all are unique - done with case
                    statement, previously it was the same for all images.
                    Error on clone tab, next button not moving - fixed next button horizontal position
                    was in an if clause - shouldn't have been.
V1.3 - 25 Sep 2021, added RestoreTab.ReSizeFilename to resize the restore filename as the main window
                    is resized. Same again for clone.
V1.4 - 28 Sep 2021, references to LBox3 removed on change to a checklistbox.
V1.5 - 04 Dec 2021, resize filename on verifytab added
V1.6 - 11 Jan 2022, not working on clone from file, nextbtn not moving correctly. Fixed with if clause
                    but a better solution would be to tie vertical spacing of nextbtn to box1 for all
                    tabs. Implementation based on assumption that all tabs have four boxes, not true
                    for clone tab.
-------------------------------------------------------------------------------}
procedure TBasePanel.PanelResize;
var
  BoxHeight : integer;
  BoxWidth : integer;
begin
  BoxHeight:= (Parent.ClientHeight - TitleLbl.Height - 2 * Box1Lbl.Height - 10 * phBorder - phBottom) div 2;
  BoxWidth:= (Parent.ClientWidth - 3 * pwBorder) div 2;
  Box1.Height:= BoxHeight;
  Box1.Width:= BoxWidth;
  Box2.Height:= BoxHeight;
  Box2.Width:= BoxWidth;
  if assigned(Box3) then //not used in clone
     begin
       Box3.Height:= BoxHeight;
       Box3.Width:= BoxWidth;
     end;
  if assigned(Box4) then //not used in restore, not used in verify
     begin
       Box4.Height:= BoxHeight;
       Box4.Width:= BoxWidth;
     end;
  if assigned(LBox4) then //used in restore in place of Box4, not used in verify
     begin
       LBox4.Height:= BoxHeight;
       LBox4.Width:= BoxWidth;
     end;
  case Box1.BoxType of
       bkBox1     : ActionImg.BorderSpacing.left:= Parent.ClientWidth - BackupImgWidth;
       reBox1     : ActionImg.BorderSpacing.left:= Parent.ClientWidth - RestoreImgWidth;
       clBox1     : ActionImg.BorderSpacing.left:= Parent.ClientWidth - CloneImgWidth;
       clBox1File : ActionImg.BorderSpacing.left:= Parent.ClientWidth - CloneFileImgWidth;
       veBox1     : ActionImg.BorderSpacing.left:= Parent.ClientWidth - VerifyFileImgWidth;
  end;
  if (Box1.BoxType = clBox1) or (Box1.BoxType = clBox1File) then
     NextBtn.BorderSpacing.top:= phBorder * 5 + Box1.Height + Box1Lbl.Height;
  NextBtn.BorderSpacing.left:= Parent.ClientWidth - NextBtn.Width - pwBorder;
  setScrollBars;
  if assigned(RestoreTab) then RestoreTab.ReSizeFilename;
  if assigned(CloneTab) then CloneTab.ReSizeFilename;
  if assigned(VerifyTab) then VerifyTab.ReSizeFilename;
end; //end procedure TBasePanel.PanelResize
{-------------------------------------------------------------------------------
PROCEDURE TBASEPANEL.CLEARBOX
Disables and clears the contents of the specified box. Disables/greys out the title.

Parameters:
    index : 1..4 identifying which TFoxCheckBox to reset

Used by:

V1.0 - 05 Mar 2021
V1.1 - 26 Mar 2021, modified to set scroll bars directly rather than calling another procedure
V1.2 - 29 Mar 2021, amended for LBox4 on restore tab
V1.3 - 02 Jun 2021, amended for LBox3 on clone tab
V1.4 - 21 Jun 2021, error on 40 - was resetting the box3 label. Also reset the caption on Box3Lbl.
V1.5 - 22 Jun 2021, fix for box4 label on verify
V1.6 - 25 Jun 2021, error on box3 label between backup and restore, if clause added
V1.7 - 28 Sep 2021, references to LBox3 removed as now a checklistbox
-------------------------------------------------------------------------------}
procedure TBasePanel.clearBox(index : byte);
begin
  case index of
    1 : with Box1 do CheckAll(cbUnchecked);
    2 : with Box2 do begin
          Enabled:= false;
          Clear;
          Box2Lbl.Enabled:= false;
          ScrollWidth:= Width; //turn off scrollbars
        end;
    3 : with Box3 do begin
          Enabled:= false;
          Clear;
          Box3Lbl.Enabled:= false;
          ScrollWidth:= Width;
          if BoxType = bkBox3 then Box3Lbl.setCaption(rBackBox3Str) else Box3Lbl.setCaption(rRestBox3Str);
        end;
    4 : with Box4 do begin
          Enabled:= false;
          Clear;
          Box4Lbl.Enabled:= false;
          ScrollWidth:= Width;
        end;
   40 : with LBox4 do begin //restore tab
          Enabled:= false;
          Clear;
          Box4Lbl.Enabled:= false;
          Box4Lbl.SetCaption(rRestBox4Str);
          RExisting.Enabled:= false;
          RExisting.Visible:= false;
          ScrollWidth:= Width;
          //bit of a kludge but procedure has no knowledge of the tab
          Box4Lbl.setCaption(rRestBox4Str);
        end;
  end;
  //fix to reset the label on box4 on verify, there is no box4
  if Box1.BoxType = veBox1 then Box4Lbl.Visible:= false;
end; //end procedure TBasePanel.clearBox
{-------------------------------------------------------------------------------
PROCEDURE TBASEPANEL.BOLDOS
The event handler for OnDrawItem for partition TFoxCheckBoxes where Style is set lbOwnerDrawFixed.
It sets the text bold if the partition has an operating system on it.

Code was taken from https://forum.lazarus.freepascal.org/index.php?topic=42397.0 and modified.
Uses TSystem.isOS to determine if an OS is present on the partition. Initially text alignment was
incorrect - aligned top and too close to the checkbox, sorted with the adjustments. May need further
adjustment if the text size is changed.

Parameters:
    Control: TWinControl - think this allows you to pass any object, but subsequent code is for a
    CheckListBox only.
    Index: Integer - index of the item in the CheckListBox.
    ARect: TRect - I assume this is the area that the text is written to in the CheckListBox.
    State: TOwnerDrawState - not sure what this is used for. And have failed to read this to determine
    state in spite of several reference on the web using it in similar procedures.

    TOwnerDrawState = set of
       (odSelected,    The item is selected.
       odGrayed,       The item should appear grayed.
       odDisabled,     The item is disabled.
       odChecked,      The item should appear checked. (This does not apply to all objects)
       odFocused,      Keyboard input is directed to the item.
       odDefault,      The item is the default item (This does not apply to all objects)
       odHotLight,     The item is hot-lighted cos under the mouse (only if the OS supports hot-lighting)
       odInactive,     The item and its associated menu are inactive.
       odNoAccel,      Any underline of an accelerator char should be skipped when drawing the control.
       odNoFocusRect,  Do not display visual cues that indicate focus
       odReserved1,    Not currently used.
       odReserved2,    Not currently used.
       odComboBoxEdit, The drawing takes place in the edit box of a combo box control.
       odBackgroundPainted

V1.1 A LOT of messing around to get the visual appearance of a disabled box using this procedure to
     appear the same as a box using the default widgets where the TCheckListBox.style=lbStandard.
     See https://forum.lazarus.freepascal.org/index.php?topic=54324.new#new.

     There maybe improvements say using just TextRect and using a default font colour instead of bold
     or using just TextOut with FillRect to paint the background appropriately.

     NOTE - for this to work Enabled:= false MUST be explicitly set when the main form is disabled,
     setting Enabled:= false on the main form does NOT automatically set the status of Enabled on child
     objects even though the appearance does change to disabled.

     Note - for colours = https://wiki.lazarus.freepascal.org/Colors

     More on text on a canvas = http://lazplanet.blogspot.com/2013/06/how-to-draw-text-on-canvas.html

Used by:

V1.0 - 29 Mar 2021
V1.1 - 26 Apr 2021
V1.2 - 31 May 2021, modified to handle restore box3 where the info comes from RDrive not MyDrives,
                    removes the need for a separate procedure for restbox3.
V1.3 - 25 Jun 2021, modified to handle verify box3
V1.4 - 02 Sep 2021, when disabled, was using textrect, think because of background colour, now changed
                    to textout as with textrect changing font size had no effect.
                    Previously was checking to see if the TCheckList box was enabled/disabled, changed to
                    MainFm.Enabled as this avoids having to enable/disable individual boxes.
-------------------------------------------------------------------------------}
procedure TBasePanel.BoldOS(Control: TWinControl; Index: Integer; ARect: TRect; State: TOwnerDrawState);
const
  AdjustL = 5;
  AdjustT = 3;
  Offset = 1; //shadow on disabled text
var
  pIndex : shortint;
begin
  ARect.Height:= Box1.ItemHeight;
  with (Control as TFoxCheckBox) do
       if MainFm.Enabled then
          begin
            if (BoxType = reBox3) or (BoxType = veBox3) then
               begin
                 pIndex:= RDrive.Partitions.FindName(Items[Index]);
                 if pIndex <> -1 then if RDrive.Partitions[pIndex].OpSys <> '' then Canvas.Font.Bold:= true;
               end
            else if MyDrives.isOS(Items[Index]) then Canvas.Font.Bold:= true;
            Canvas.TextOut(ARect.Left + AdjustL,ARect.Top + AdjustT,Items[Index]);
          end
       else
          begin
            Canvas.Brush.Style:= bsClear; //set the background colour = transparent
            //with GTK text in a checkbox that is disabled is grey with a white shadow.
            //draw the shadow first
            Canvas.Font.Color:= clBtnHighlight; //Btn highlight color used to achieve 3D effect
            Canvas.TextOut(ARect.Left + AdjustL+Offset,ARect.Top + AdjustT+Offset,Items[Index]);
            Canvas.Font.Color:=clGrayText; //The font color of disabled element
            Canvas.TextOut(ARect.Left + AdjustL,ARect.Top + AdjustT,Items[Index]);
          end;
end; //end procedure TBasePanel.BoldOS
{-------------------------------------------------------------------------------
PROCEDURE TBASEPANEL.DRIVEHINT
a unified event handler for MouseOver on boxes containing drive information.

Initial code was from the web.

Is used to display drive specific information against each item in the checklistbox.

Parameters:
    Sender : TObject - references the control that was used to call the method. If you click on a
             Button1 control, causing the Button1Click method to be called, a reference or pointer
             to the Button1 object isBox2Lbl passed to Button1Click in the parameter called Sender.

    Shift : TShiftState - compiler shows a not used warning, but without it will not compile.
            Indicates what special keys were pressed in combination with a normal key, see
            https://www.freepascal.org/docs-html/rtl/classes/tshiftstate.html for the set of keys

    X,Y : Integer - screen coordinates?

Originally had specific procedures for each box, a general procedure required the redefinition of a
TCheckListBox to a TFoxCheckBox which has the additional property BoxType. This allows each
box to be identified, e.g. bkBox1, bkBox2. This is tested in a case statement and the hint adjusted as
appropriate. All drive hints for a drive are the same, what changes is the hint when in a blank area
of the box:

   Backup box1 - drives excluded are the boot drive, any blank drives and any loop devices.
   Backup box3 - excludes blank and the drive selected for backup.
   Restore box1 - only excludes a blank drive.

Used by:

V1.0 - 29 Apr 2021
V1.1 - 03 Jun 2021, modified for clBox2.
V1.2 - 07 Jun 2021, modified for clBox4.
V1.3 - 20 Jan 2021, modified for veBox1.
V1.4 - 13 Oct 2021, space now defined as const in utilities
-------------------------------------------------------------------------------}
procedure TBasePanel.DriveHint(Sender : TObject; Shift : TShiftState; X,Y : Integer);
const
  delim = ': ';
var
  ItemIndex : shortint;
  DriveStr : string = '';
  i : byte;
begin
  with (Sender as TFoxCheckBox) do case BoxType of
      bkBox1,clBox1,veBox1 : begin //no hint excludes boot, blank and loop
                 ItemIndex:=Box1.ItemAtPos(Point(X,Y),true);
                 if (ItemIndex > -1) and (ItemIndex < Box1.Count) then
                     Box1.hint:= MyDrives[MyDrives.FindName(Box1.Items[ItemIndex])].HintStr
                 else
                     begin
                       DriveStr:= rnotHintStr + delim + LF + LF;
                       for i:= 0 to MyDrives.Count - 1 do with MyDrives[i] do
                           begin
                             if isBoot then DriveStr+= MountPt + delim + rnotHintBootStr + LF;
                             if isLoop then DriveStr+= MountPt + delim + rnotHintLoopStr + LF;
                             if isBlank then DriveStr+= MountPt + delim + rnotHintBlankStr + LF;
                           end;
                       DriveStr+= space;
                       Box1.hint:= DriveStr;
                     end;
               end;
      bkBox3 : begin //no hint excludes source and blank
                 ItemIndex:=Box3.ItemAtPos(Point(X,Y),true);
                 if (ItemIndex > -1) and (ItemIndex < Box3.Count) then
                    Box3.hint:= MyDrives[MyDrives.FindName(Box3.Items[ItemIndex])].HintStr
                 else
                    begin
                      DriveStr:= rnotHintStr + delim + LF + LF;
                      for i:= 0 to MyDrives.Count - 1 do with MyDrives[i] do
                          begin
                            if setAction = acSource then DriveStr+= MountPt + delim + rnotHintSourceStr + LF;
                            if isBlank then DriveStr+= MountPt + delim + rnotHintBlankStr + LF;
                          end;
                      DriveStr+= space;
                      Box3.hint:= DriveStr;
                    end;
               end;
      reBox1,clBox1File : begin //no hint excludes blank
                 ItemIndex:= Box1.ItemAtPos(Point(X,Y),true);
                 if (ItemIndex > -1) and (ItemIndex < Box1.Count) then
                    Box1.hint:= MyDrives[MyDrives.FindName(Box1.Items[ItemIndex])].HintStr
                 else
                    begin
                      for i:= 0 to MyDrives.Count - 1 do with MyDrives[i] do
                          if isBlank then DriveStr+= MountPt + delim + rnotHintBlankStr + LF;
                      if DriveStr <> '' then DriveStr:= rnotHintStr + delim + LF + LF + DriveStr + Space;
                      Box1.hint:= DriveStr;
                    end;
               end;
      clBox2 : begin //no hint excludes acSource and boot drive
                 ItemIndex:=Box2.ItemAtPos(Point(X,Y),true);
                 if (ItemIndex > -1) and (ItemIndex < Box2.Count) then
                    Box2.hint:= MyDrives[MyDrives.FindName(Box2.Items[ItemIndex])].HintStr
                 else
                    begin
                      DriveStr:= rnotHintStr + delim + LF + LF;
                      for i:= 0 to MyDrives.Count - 1 do with MyDrives[i] do
                          begin
                            if setAction = acSource then DriveStr+= MountPt + delim + rnotHintSource2Str + LF;
                            if isBoot then DriveStr+= MountPt + delim + rnotHintBootStr + LF;
                          end;
                      DriveStr+= space;
                      Box2.hint:= DriveStr;
                    end;
               end;
      clBox4 : begin //no hint excludes acSource and boot drive
                 ItemIndex:=Box4.ItemAtPos(Point(X,Y),true);
                 if (ItemIndex > -1) and (ItemIndex < Box4.Count) then
                    Box4.hint:= MyDrives[MyDrives.FindName(Box4.Items[ItemIndex])].HintStr
                 else
                    begin
                      DriveStr:= rnotHintStr + delim + LF + LF;
                      for i:= 0 to MyDrives.Count - 1 do with MyDrives[i] do
                          begin
                            if setAction = acSource then DriveStr+= MountPt + delim + rnotHintSource2Str + LF;
                            if isBoot then DriveStr+= MountPt + delim + rnotHintBootStr + LF;
                          end;
                      DriveStr+= space;
                      Box4.hint:= DriveStr;
                    end;
               end;
  end; //case
  Application.ActivateHint(ClientToScreen(Point(X,Y)));
end; //end procedure TBasePanel.DriveHint
{-------------------------------------------------------------------------------
PROCEDURE TBASEPANEL.PARTHINT
a unified event handler for MouseOver on boxes containing partition information.

Initial code was from the web.

Is used to display partition specific information against each item in the checklistbox.

Parameters:
    Sender : TObject - references the control that was used to call the method. If you click on a
             Button1 control, causing the Button1Click method to be called, a reference or pointer
             to the Button1 object isBox2Lbl passed to Button1Click in the parameter called Sender.

    Shift : TShiftState - compiler shows a not used warning, but without it will not compile.
            Indicates what special keys were pressed in combination with a normal key, see
            https://www.freepascal.org/docs-html/rtl/classes/tshiftstate.html for the set of keys

    X,Y : Integer - screen coordinates?

Originally had specific procedures for each box, a general procedure required the redefinition of a
TCheckListBox to a TFoxCheckBox which has the additional property BoxType. This allows each
box to be identified, e.g. bkBox1, bkBox2. This is tested in a case statement and the hint adjusted as
appropriate. Partition hints vary dependent on what action and whether extended, swap or EFI.

Used by:

V1.0 - 29 Apr 2021
V1.1 - 03 Jun 2021, clBox2File added
V1.2 - 20 Jun 2021, veBox2 added
V1.3 - 21 Jun 2021, veBox3 added
V1.4 - 13 Oct 2021, space now defined as const in utilities
V1.5 - 28 Oct 2021, was missing clbox3 so no hints.
-------------------------------------------------------------------------------}
procedure TBasePanel.PartHint(Sender : TObject; Shift : TShiftState; X,Y : Integer);
const
  ExtendStr = 'extended';
  SwapStr = 'swap';
  LF2 = LF + LF;
var
  iIndex : shortint;
  dIndex : shortint = -1;
  pIndex : shortint;
  i : shortint;
  PartHintStr : string;
begin
  with (Sender as TFoxCheckBox) do case BoxType of
       bkBox2 : begin //extended, swap or EFI partitions are auto backed up
                  iIndex:=Box2.ItemAtPos(Point(X,Y),true);
                  if (iIndex > -1) and (iIndex < Box2.Count) then
                     begin
                       //find the drive
                       for i:= 0 to MyDrives.Count - 1 do if MyDrives[i].setAction = acSource then
                          begin
                            dIndex:= i;
                            break;
                          end;
                       if dIndex = -1 then SystemError(3,'TBasePanel.PartHint'); //close foxclone on error
                       //find the partition
                       pIndex:= MyDrives[dIndex].partitions.FindName(Box2.Items[iIndex]);
                       if pIndex = -1 then SystemError(4,'TBasePanel.PartHint'); //close foxclone on error
                       //get the partition hint
                       with MyDrives[dindex].partitions[pIndex] do
                           begin
                             PartHintStr:= HintStr(false);
                             if PartType = ExtendStr then PartHintStr+= LF2 + rExtendedStr + LF + rAutoBackStr;
                             if EFIpartition then PartHintStr+= LF2 + rEFIpartMsgStr  + LF + rAutoBackStr;
                             if FileSystem = SwapStr then PartHintStr+= LF2 + rSwapMsgStr  + LF + rAutoBackStr;
                             Box2.Hint:= PartHintStr + LF + Space;
                           end;
                     end
                  else Box2.Hint:= '';
                end;
       bkBox4 : begin //standard hint string - extended, swap and EFI parts are not shown in the box
                  iIndex:=Box4.ItemAtPos(Point(X,Y),true);
                  if (iIndex > -1) and (iIndex < Box4.Count) then
                     begin
                       //find the drive
                       for i:= 0 to MyDrives.Count - 1 do if MyDrives[i].setAction = acTarget then
                          begin
                            dIndex:= i;
                            break;
                          end;
                       if dIndex = -1 then SystemError(7,'TBasePanel.PartHint'); //close foxclone on error
                       //find the partition
                       pIndex:= MyDrives[dIndex].partitions.FindName(Box4.Items[iIndex]);
                       if pIndex = -1 then SystemError(4,'TBasePanel.PartHint'); //close foxclone on error
                       //get the partition hint
                       with MyDrives[dindex].partitions[pIndex] do Box4.Hint:= HintStr(false) + LF + Space;
                     end
                  else Box4.Hint:= '';
                end;
       reBox2,clBox2File,veBox2 : begin //standard hint string - extended, swap and EFI parts are not shown in the box
                  iIndex:=Box2.ItemAtPos(Point(X,Y),true);
                  if (iIndex > -1) and (iIndex < Box2.Count) then
                     begin
                       //find the drive
                       for i:= 0 to MyDrives.Count - 1 do if MyDrives[i].setAction = acSource then
                          begin
                            dIndex:= i;
                            break;
                          end;
                       if dIndex = -1 then SystemError(13,'TBasePanel.PartHint'); //close foxclone on error
                       //find the partition
                       pIndex:= MyDrives[dIndex].partitions.FindName(Box2.Items[iIndex]);
                       if pIndex = -1 then SystemError(4,'TBasePanel.PartHint'); //close foxclone on error
                       //get the partition hint
                       with MyDrives[dindex].partitions[pIndex] do Box2.Hint:= HintStr(false) + LF + Space;
                     end
                  else Box2.Hint:= '';
                end;
       rebox3,clbox3, vebox3 : begin
                  iIndex:= Box3.ItemAtPos(Point(X,Y),true);
                  if (iIndex > 0) and (iIndex < Box3.Count) then
                     begin
                       //the drive is RDrive, find the partition
                       pIndex:= RDrive.Partitions.FindName(Box3.Items[iIndex]);
                       if pIndex = -1 then SystemError(4,'TBasePanel.PartHint'); //close foxclone on error
                       Box3.hint:= RDrive.Partitions[pIndex].HintStr(false) + LF + Space;
                     end
                  else Box3.Hint:= '';
                end;
  end;
  Application.ActivateHint(ClientToScreen(Point(X,Y)));
end; //end procedure TBasePanel.PartHint
{-------------------------------------------------------------------------------
PROCEDURE TBASEPANEL.LBOXHINT
a unified event handler for MouseOver on list boxes containing partition information.

Based on parthint.

Is used to display partition specific information against each item in the checklistbox.

Parameters:
    Sender : TObject - references the control that was used to call the method. If you click on a
             Button1 control, causing the Button1Click method to be called, a reference or pointer
             to the Button1 object isBox2Lbl passed to Button1Click in the parameter called Sender.

    Shift : TShiftState - compiler shows a not used warning, but without it will not compile.
            Indicates what special keys were pressed in combination with a normal key, see
            https://www.freepascal.org/docs-html/rtl/classes/tshiftstate.html for the set of keys

    X,Y : Integer - screen coordinates?

Used by:

V1.0 - 05 Jun 2021
V1.1 - 28 Sep 2021, with LBox3 replaced by a checklistbox, this procedure no longer called from clone
                    so no longer a need for a case statement on BoxType.
V1.2 - 13 Oct 2021, space now defined as const in utilities
-------------------------------------------------------------------------------}
procedure TBasePanel.LBoxHint(Sender : TObject; Shift : TShiftState; X,Y : Integer);
var
  i, iIndex, dIndex, pindex : shortint;
begin
  with (Sender as TFoxListBox) do
       begin
         iIndex:=LBox4.ItemAtPos(Point(X,Y),true);
         if (iIndex > -1) and (iIndex < LBox4.Count) then
            begin
              //find the drive
              for i:= 0 to MyDrives.Count - 1 do if MyDrives[i].setAction = acTarget then
                  begin
                    dIndex:= i;
                    break;
                  end;
              if dIndex = -1 then SystemError(21,'TBasePanel.LBoxHint'); //close foxclone on error
              //find the partition
              pIndex:= MyDrives[dIndex].partitions.FindName(LBox4.Items[iIndex]);
              if pIndex = -1 then SystemError(4,'TBasePanel.LBoxHint'); //close foxclone on error
              //get the partition hint
              with MyDrives[dindex].partitions[pIndex] do LBox4.Hint:= HintStr(false) + LF + Space;
            end
         else LBox4.Hint:= '';
       end;
  Application.ActivateHint(ClientToScreen(Point(X,Y)));
end; //end procedure TBasePanel.LBoxHint
{-------------------------------------------------------------------------------
PROCEDURE TBASEPANEL.POPULATEBOX1
Originally had separate procedures for clone and restore, amalgamated to a single procedure to cover
box1 on all tabs.

Adds drive informaton for all drives in MyDrives to box1.

Sets scrollbars after adding the information.

Key differences:
    Backup - excludes boot drive, any loop devices or blank drives
    Restore - excludes blank drives
    Clone - for clone drive to drive: excludes boot drive, any loop devices or blank drives (as backup)
    Clone - for backup to drive: excludes blank drives

Used by:

V1.0 - 01 Mar 2021
V1.1 - 25 Mar 2021 this was in TBasePanel, but moved. Most box1's are slightly different, e.g. in what is
                   excluded.
V1.2 - 26 Mar 2021 set scrollbars added
-------------------------------------------------------------------------------}
procedure TBasePanel.PopulateBox1;
var
  i : byte;
begin
  box1.clear;
  box1.enabled:= true;
  for i:= 0 to MyDrives.count - 1 do
    case Box1.BoxType of
      bkBox1, clBox1     : if not (MyDrives[i].isBoot or MyDrives[i].isLoop or MyDrives[i].isBlank) then
                                   Box1.Items.add(MyDrives[i].ListStr);
      reBox1, clBox1File, veBox1 : if not MyDrives[i].isBlank then Box1.Items.add(MyDrives[i].ListStr);

    end;
  //setup CheckState
  setLength(Box1.CheckState,Box1.Count);
  for i:= 0 to Box1.Count - 1 do Box1.CheckState[i]:= false; //default, nothing selected
  setScrollBarTBox(Box1);
end; //end procedure TBasePanel.PopulateBox1
{-------------------------------------------------------------------------------
PROCEDURE TBASEPANEL.FONTRESIZE
Resizes objects on a font size change.

Used by: TSettings.SettingsFontSize

V1.0 - 25 Jun 2021
V1.1 - 24 Jul 2021, modded for next button
V1.2 - 28 Jul 2021, modded for the combo box on the clone tab
V1.3 - 25 Aug 2021, modded for the note button and associated labels on the restore tab
V1.4 - 25 Sep 2021, as previous modded for clone from file
V1.5 - 28 Sep 2021, references to LBox3 removed as now a checklistbox
-------------------------------------------------------------------------------}
procedure TBasePanel.FontResize;
begin
  with Self do begin
       //title label
       TitleLbl.Font.Size:= MainFm.Font.Size + 4;
       TitleLbl.setCaption(TitleLbl.Caption);
       //box labels
       Box1Lbl.setCaption(Box1Lbl.Caption);
       Box2Lbl.setCaption(Box2Lbl.Caption);
       Box3Lbl.setCaption(Box3Lbl.Caption);
       Box4Lbl.setCaption(Box4Lbl.Caption);
       //box item heights
       BoxItemHeight:= GetTextHeight('Ag',MainFm.Font) + ItemHeightAdjust;
       Box1.ItemHeight:= BoxItemHeight;
       Box2.ItemHeight:= BoxItemHeight;
       if assigned(Box3) then Box3.ItemHeight:= BoxItemHeight;
       if assigned(Box4) then Box4.ItemHeight:= BoxItemHeight;
       if assigned(LBox4) then LBox4.ItemHeight:= BoxItemHeight;
       //next button
       with NextBtn do begin
            Height:= GetTextHeight(rNextStr,font) + 14;
            Width:= GetTextWidth(rNextStr,font);
            if GetTextWidth(rRestTabStr,font) > Width then Width:= GetTextWidth(rRestTabStr,font);
            if GetTextWidth(rCloneTabStr,font) > Width then Width:= GetTextWidth(rCloneTabStr,font);
            if GetTextWidth(rVerifyTabStr,font) > Width then Width:= GetTextWidth(rVerifyTabStr,font);
            Width:= Width + 20; //manual adjust to get about same as quit button
       end;
       //clone combobox
       if assigned(CloneComboBox) then //assigned as created by the clone tab
          begin
            if GetTextWidth(rCloneComboStr1,font) > GetTextWidth(rCloneComboStr2,font) then
            CloneComboBox.Width:= GetTextWidth(rCloneComboStr1,font)
            else CloneComboBox.Width:= GetTextWidth(rCloneComboStr2,font);
            CloneComboBox.Width:= CloneComboBox.Width + 40;
          end;
       //labels and buttons on restore tab
       //REexisting is declared in basepanel the others are declared in restore
       //RDateLbl, Rdate, RNowLbl, RNow, NoteBtn
       if assigned(RExisting) then RExisting.setCaption(RExisting.Caption);
       with RestoreTab do
           begin
             if assigned(RDateLbl) then RDateLbl.setCaption(RDateLbl.Caption);
             if assigned(Rdate) then Rdate.setCaption(Rdate.Caption);
             if assigned(RNowLbl) then RNowLbl.setCaption(RNowLbl.Caption);
             if assigned(RNow) then RNow.setCaption(RNow.Caption);
             if assigned(NoteBtn) then
                begin
                  NoteBtn.Width:= GetTextWidth(rNoteBtnStr,font) + 20;
                  NoteBtn.Height:= GetTextHeight(rNoteBtnStr,font) + 14;
                end;
           end;
       //repeat the same for the clonetab on clone from file
       with CloneTab do
           begin
             if assigned(RDateLbl) then RDateLbl.setCaption(RDateLbl.Caption);
             if assigned(Rdate) then Rdate.setCaption(Rdate.Caption);
             if assigned(RNowLbl) then RNowLbl.setCaption(RNowLbl.Caption);
             if assigned(RNow) then RNow.setCaption(RNow.Caption);
             if assigned(NoteBtn) then
                begin
                  NoteBtn.Width:= GetTextWidth(rNoteBtnStr,font) + 20;
                  NoteBtn.Height:= GetTextHeight(rNoteBtnStr,font) + 14;
                end;
           end;
  end;
end;
{-------------------------------------------------------------------------------
PROCEDURE TBASEPANEL.BOXMOUSELEAVE
This procedure is purely cosmetic - to get a consistent visual appearance across all boxes.

Drive checklistboxes do not have lbownerdraw and as the cursor moves over the box, individual items are
highlighted. When the mouse leaves the box the last item the mouse was on is left highlighted.

Partition checklistboxes do have lbownerdraw so that partitions with an OS = text bolded. When a
partition is clicked the item is highlighted and stays highlighted when the mouse leaves the box.

All this procedure does is remove highlighting on all items in the box when the mouse leaves the box.
I could setup lbownerdraw for drive checklistboxes, but this is a simple one line solution.

Used by: set as the event handler for mouseleave in all TFoxCheckBoxes.

V1.0 - 12 Jan 2022
V1.1 - 29 Jan 2022, if statement added to check that the box has items before clearing select. Was
                    generating an index out of bounds.
-------------------------------------------------------------------------------}
procedure TBasePanel.BoxMouseLeave(Sender: TObject);
var
  i : byte;
begin
  with (Sender as TFoxCheckBox) do if Count > 0 then for i:= 0 to Count - 1 do Selected[i]:= false;
end;

end.

