{ FoxClone
  Copyright (C) 2021 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}
{-------------------------------------------------------------------------------
UNIT - MAINFORM

The main form in foxclone containing the tabs for backup, restore, clone, verify, settings and about.

It contains ReadDrives that reads the system and creates the TSystem (a list of drives), TDrive (drive
parameters) and TPart (partition parameters).

The form has a quit button and a version label.

V1.0 - 10 Dec 2021
-------------------------------------------------------------------------------}
unit mainform;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ComCtrls, StdCtrls, StrUtils, DateUtils,
  utilities, il8nstrings, GTKlabel, backup, restore, clone, verify, settings, startupform, about;

type
  TMainFm = class(TForm)
    const
      fixedFontSize = 10; //fixed for tab labels and version label
      fwBorder = 15; //default border unit for gaps horizontally in the form, same as panels (pwBorder)
      fBtnHeight = 30; //default height for buttons
    private
      PageCtrl : TPageControl; //setup the tabs
      VersionLbl : TLabelGTK;
      FormActivated : boolean;
      procedure MainFormResize(Sender:Tobject);
      procedure QuitBtnCK(Sender: TObject);
      procedure MainFormClose(Sender: TObject; var CloseAction: TCloseAction);
    public
      QuitBtn : Tbutton; //was private, moved public for font resize
      constructor Create(TheOwner: TComponent); override;
      procedure ReadDrives(Sender: TObject);
  end;

var
  MainFm : TMainFm;

implementation

{$R *.lfm}
{-------------------------------------------------------------------------------
CONSTRUCTOR
Creates the main window and the tabs for backup, restore, clone & settings
Calls procedures to set up each individual tab and start with the backup tab.

Also runs GetDriveInfo to read the system and get information on all drives and
partitions, and sets the initial defaults for compression, split files, etc.

V1.3 - 21 Feb 2021, added resourcestrings to message dialog
V1.4 - 31 May 2021, clone tab added
V1.5 - 01 Jun 2021, version label changed to TLabelGTK
V1.6 - 10 Jun 2021, modified for about page
V1.7 - 14 Jun 2021, modified for verify tab
V1.8 - 25 Jun 2021, parentfont false on version label
V1.9 - 20 Nov 2021, added create the default folder /tmp/foxclone if it doesn't exist.
-------------------------------------------------------------------------------}
constructor TMainFM.Create(TheOwner: TComponent);
var
  i : byte;
begin
  inherited CreateNew(TheOwner,1);
  //Font.Name:= 'Noto Sans';
  FormActivated:= false;
  Font.Size:= DefaultFontSize;
  Caption:= 'Foxclone'; //caption for the title bar
  Position:=poScreenCenter;
  SetInitialBounds(0,0,wWidth,wHeight); //defines initial size
  Constraints.MinHeight:= wHeight;
  Constraints.MinWidth:= wWidth;
  OnResize:= @MainFormResize;
  OnClose:= @MainFormClose;
  OnActivate:= @ReadDrives;
  //Create tab sheets
  PageCtrl:=TPageControl.Create(self);
  with PageCtrl do
    begin
      Parent:=Self;
      Align:=alTop; //may need changing if we put in a menu bar
      //leave some space at the bottom of the tab windows for a status bar
      Height:=Parent.ClientHeight - wGap;
      Width:= Parent.ClientWidth;
      for i:=0 to 5 do with TTabSheet.Create(PageCtrl) do
          begin
            PageControl:=PageCtrl;
            Caption:=Name;
            //no change so the height of the tab is fixed, used in calcs on resize
            Font.Size:= fixedFontSize;
            //Font.Color:= clRed;  this changes everything - panels, objects on panels
            case i of
                 0 : Name:= rBackTabStr;
                 1 : Name:= rRestTabStr;
                 2 : Name:= rCloneTabStr;
                 3 : Name:= rVerifyTabStr;
                 4 : Name:= rSetTabStr;
                 5 : Name:= rAboutTabStr;
            end;
          end;
      Anchors:=[akLeft,akTop,akRight,akBottom];
    end;
  BackTab:= TBack.Create(self);
  with BackTab do
      begin
        Anchors:=[akLeft,akTop,akRight,akBottom]; //dont think does anything.
        Parent:= PageCtrl.Page[0];
        Width:= Parent.ClientWidth;
        Height:= Parent.ClientHeight;
        BevelOuter := bvNone; //turn off the border
      end;
  RestoreTab:= TRestore.Create(self);
  with RestoreTab do
      begin
        Anchors:=[akLeft,akTop,akRight,akBottom]; //dont think does anything.
        Parent:= PageCtrl.Page[1];
        Width:= Parent.ClientWidth;
        Height:= Parent.ClientHeight;
        BevelOuter := bvNone; //turn off the border
      end;
  CloneTab:= TClone.Create(self);
  with CloneTab do
      begin
        Anchors:=[akLeft,akTop,akRight,akBottom]; //dont think does anything.
        Parent:= PageCtrl.Page[2];
        Width:= Parent.ClientWidth;
        Height:= Parent.ClientHeight;
        BevelOuter := bvNone; //turn off the border
      end;
  VerifyTab:= TVerify.Create(self);
  with VerifyTab do
      begin
        Anchors:=[akLeft,akTop,akRight,akBottom]; //dont think does anything.
        Parent:= PageCtrl.Page[3];
        Width:= Parent.ClientWidth;
        Height:= Parent.ClientHeight;
        BevelOuter := bvNone; //turn off the border
      end;
  Defaults:= TSettings.Create(self);
  with Defaults do
      begin
        Anchors:=[akLeft,akTop,akRight,akBottom]; //dont think does anything.
        Parent:= PageCtrl.Page[4];
        Width:= Parent.ClientWidth;
        Height:= Parent.ClientHeight;
        BevelOuter := bvNone; //turn off the border
      end;
  AboutTab:= TAboutPanel.Create(self);
  with AboutTab do
      begin
        Anchors:=[akLeft,akTop,akRight,akBottom]; //dont think does anything.
        Parent:= PageCtrl.Page[5];
        Width:= Parent.ClientWidth;
        Height:= Parent.ClientHeight;
        BevelOuter := bvNone; //turn off the border
      end;
  // Quit button
  QuitBtn:= TButton.Create(Self);
  with QuitBtn do begin
       Parent:= self;
       Caption:= rQuitStr;
       Height:= fBtnHeight;
       Width:= GetTextWidth(rQuitStr,font) + 60; //manual adjust for appearance
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= PageCtrl;
       BorderSpacing.top:= 10;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= self;
       BorderSpacing.left:= fwBorder;
       OnClick:= @QuitBtnCK;
  end;
  // Version label
  VersionLbl:= TLabelGTK.Create(Self);
  with VersionLbl do
      begin
        Caption:= replaceStr(getStr('uname -r'),'-generic','');
        setCaption(rVerStr + ' ' + FormatFloat('#,##0.00',Version/100)+ ' k' + Caption + ', ' + rVerDateStr);
        Parent:= self;
        //Neither DefaultFontSize or fixedFontSize fix the font size, it changes as the mainform
        //font size changes, and setting it to 10 doesn't work either? Need parentfont false.
        ParentFont:= false;
        Font.Size:= FixedFontSize;
        //anchors only seem to work left to right and top to bottom
        AnchorSide[akTop].Side:= asrCenter;
        AnchorSide[akTop].Control:= QuitBtn;
        BorderSpacing.top:= 10;
        AnchorSide[akLeft].Side:= asrLeft;
        AnchorSide[akLeft].Control:= PageCtrl;
        BorderSpacing.Left:= PageCtrl.Width - GetTextWidth(Caption,font) - 20;
      end;
   //DefaultFolder is /tmp/foxclone used to temp copies of sfdisk and partclone log files.
   if not DirectoryExists(DefaultFolder) then CreateDir(DefaultFolder);
end; //end constructor
{-------------------------------------------------------------------------------
PROCEDURE TMAINFM.MAINFORMRESIZE
Resizes child objects - VersionLbl and TPanels, etc. on a resize of the main window

Used by: called by the OnResize event of the main form

V1.0 - 21 Feb 2021
V1.1 - 22 Jun 2021, verifytab
-------------------------------------------------------------------------------}
procedure TMainFm.MainFormResize(Sender : TObject);
begin
  with VersionLbl do BorderSpacing.Left:= PageCtrl.Width - GetTextWidth(Caption,font) - 20;
  BackTab.PanelResize;
  RestoreTab.PanelResize;
  CloneTab.PanelResize;
  VerifyTab.PanelResize;
  AboutTab.PanelResize;
  Defaults.SettingsResize;
end; //end procedure TMainFm.MainFormResize
{-------------------------------------------------------------------------------
PROCEDURE TMAINFM.MAINFORMCLOSE
Needs to check that it is safe to close foxclone
how to check if there is a restore/clone in progress?

    caNone - Do nothing.
    caHide - The form is hidden.
    caFree - The form is destroyed.
    caMinimize - The form is minimized.

Used by: called by the OnClose event of the main form

V1.0 - 21 Feb 2021
       11 Nov 2021, think redundant, the onclose handler for the progress form checks to see if
                    partclone is running.
-------------------------------------------------------------------------------}
procedure TMainFm.MainFormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  //writeln('close has been called...');
  CloseAction:= caFree;
end; //end procedure TMainFm.MainFormClose
procedure TMainFm.QuitBtnCK(Sender: TObject);
begin
  Self.Close;
end; //end procedure TMainFm.QuitBtnCK
procedure TMainFm.ReadDrives(Sender: TObject);
var
  StartupFm : TStartupFm;
begin
  if not FormActivated then
     begin
       FormActivated:= true; //so this is only run once
       //writeln('read drives called...');
       Self.enabled:= false; //has to be before process messages otherwise mainfm doesn't show disabled
       StartupFm:= TStartupFm.create(Self);
       StartupFm.Show; //it won't show without it
       Application.ProcessMessages; //without this the main form does not show the objects on it.
       //setup drive and partition objects, read the system
       StartupFm.ReadSystem;
       StartupFm.Close;
       Self.enabled:= true;
     end;
end;
end.

