{ FoxClone
  Copyright (C) 2021 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}
{-------------------------------------------------------------------------------
UNIT - NOTEFORM

Noteform object and methods. It is used for

         Backup - for creating a note associated with a backup. It is activated from the note button
         on the progress form after a backup has completed.

         Restore and 'clone from file' - viewing a note created by a backup. Activated by the 'view
         note' button on the restore and clone tabs.

         Viewing log files.

V1.0 - 10 Dec 2021
-------------------------------------------------------------------------------}
unit noteform;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, utilities, il8nstrings;

type
  TNoteForm = class(TForm)
  const
    nHeight = 300;  //main window initial height
    nWidth = 500;   //main window initial width
    phBorder = 10; //default border unit for gaps vertically in the panel
    pwBorder = 20; //default border unit for gaps horizontally in the panel
  private
    procedure noteCancel(Sender: TObject);
    procedure noteSave(Sender: TObject);
    procedure noteFormResize(Sender: TObject);
  public
    BackMemo : TMemo;
    NoteFname : string;
    CancelBtn, OKBtn : TButton;
    constructor Create(TheOwner: TComponent);
  end;

implementation

uses mainform; //required for font resize

{-------------------------------------------------------------------------------
CONSTRUCTOR TNOTEFORM.CREATE
Calls inherited createnew so that an LFM file is not needed.

Creates the note form with buttons neither visible or enabled. That must be done by the calling
method. Content of the memo is also blank - to be populated by the calling method.

Note - TNoteForm.visible must NOT be set visible explicitly. The form is displayed with showmodel - so
       that the mainform can be disabled and the noteform freed in the calling method. Tried it without
       showmodal and it wouldn't work.

V1.0 - 05 May 2021
V1.1 - 25 Aug 2021, modified to set font size = main form to pick up any changes to font size. uses
                    clause added above
-------------------------------------------------------------------------------}
constructor TNoteForm.Create(TheOwner: TComponent);
var
  ButtonHeight, ButtonWidth, CancelW, SaveW : integer;
begin
  inherited CreateNew(TheOwner,1);
  Font.Size:= MainFm.Font.Size; //added so that note form font is resized
  ButtonHeight:= GetTextHeight('AHlg',Font) + 15;
  SaveW:= GetTextWidth(rNoteSave,Font);
  CancelW:= GetTextWidth(rNoteCancel,Font);
  if SaveW > CancelW then ButtonWidth:= SaveW else ButtonWidth:= CancelW;
  ButtonWidth+= 30;
  TabStop:= false;
  Caption:= rNoteTitle; //caption for the title bar
  Position:= poScreenCenter;
  Constraints.MinHeight:= nHeight;
  Constraints.MinWidth:= nWidth;
  SetInitialBounds(0,0,nWidth,nHeight); //defines initial size
  //Visible:= true; needs to be turned off for showmodal to work
  OnResize:= @NoteFormResize;
  BackMemo:= TMemo.Create(Self);
  with BackMemo do begin
       WordWrap:= True;
       Parent:= Self;
       ReadOnly:= true;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= Self;
       BorderSpacing.top:= phBorder;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Self;
       BorderSpacing.left:= pwBorder;
       Width:= Self.Width - 2 * pwBorder;
       Height:= Self.Height - 90;
       Scrollbars:= ssAutoBoth; //not sure this does anything with wordwrap on
  end;
  OKBtn:= TButton.Create(Self);
  with OKBtn do begin
       Default:= true;
       parent:= Self;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= BackMemo;
       BorderSpacing.top:= phBorder * 2;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= BackMemo;
       BorderSpacing.Left:= BackMemo.Width - ButtonWidth;
       Height:= ButtonHeight;
       Width:= ButtonWidth;
       Caption:= rNoteSave;
       OnClick:= @NoteSave;
       Visible:= false;
       Enabled:= false;
  end;
  CancelBtn:= TButton.Create(Self);
  with CancelBtn do begin
       Cancel:= true;
       parent:= Self;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= BackMemo;
       BorderSpacing.top:= phBorder * 2;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= BackMemo;
       BorderSpacing.Left:= BackMemo.Width - 2 * ButtonWidth - pwBorder;
       Height:= ButtonHeight;
       Width:= ButtonWidth;
       Caption:= rNoteCancel;
       OnClick:= @noteCancel;
       Visible:= false;
       Enabled:= false;
   end;
end; //end constructor
{-------------------------------------------------------------------------------
PROCEDURE TNOTEFORM.NOTEFORMRESIZE
Resizes the note form when the user changes the size - adjusts the size of the memo and position
of the buttons.

V1.0 - 05 May 2021
-------------------------------------------------------------------------------}
procedure TNoteForm.noteFormResize(Sender: TObject);
begin
  BackMemo.Width:= Self.Width - 2 * pwBorder;
  BackMemo.Height:= Self.Height - 90;
  OKBtn.BorderSpacing.Left:= BackMemo.Width - OKBtn.Width;
  CancelBtn.BorderSpacing.Left:= BackMemo.Width - 2 * OKBtn.Width - pwBorder;
end; //end TNoteForm.noteFormResize
{-------------------------------------------------------------------------------
PROCEDURE TNOTEFORM.NOTECANCEL
Event handler for the cancel button.

V1.0 - 05 May 2021
-------------------------------------------------------------------------------}
procedure TNoteForm.noteCancel(Sender: TObject);
begin
  Close;
end; //end TNoteForm.noteCancel
{-------------------------------------------------------------------------------
PROCEDURE TNOTEFORM.NOTESAVE
The event handler for the okay button. When adding a note from a backup it saves the file, when
viewing log files or note files just closes. The caption is used as the test on what to do.

V1.0 - 21 Nov 2021
-------------------------------------------------------------------------------}
//21 nov uses caption to not save when closing on viewing log files
procedure TNoteForm.noteSave(Sender: TObject);
begin
  if OKBtn.Caption <> rClose then BackMemo.Lines.SaveToFile(NoteFname);
  Close;
end; //end TNoteForm.noteSave

end.

