{ FoxClone
  Copyright (C) 2021 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}
{-------------------------------------------------------------------------------
UNIT - PARTCLONE

Defines the progress form used to display progress on a backup, restore, etc.

The objects and methods to run partclone for backup, restore, etc., and save/restore grub and the
partition table.

Access to the methods is via the procedure runFoxClone.

V1.0 - 10 Oct 2021
V1.1 - 18 Jan 2022, default width of form increased by 10px, was cropping the progress text in iso
                    think because it is ubuntu in iso and probably using noto sans in mint.
-------------------------------------------------------------------------------}
unit partclone;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, ComCtrls, StdCtrls, Graphics, Dialogs, process, dateutils,
  utilities, verify, clone, GTKlabel, il8nstrings, drives, strutils, fileutil, partitions,
  settings, fdialog, noteform, restore, backup, startupform;

type
  TProgressFm = class(TForm)
    const
      pwBorder = 15; //default border unit for gaps horizontally in the form, same as panels (pwBorder)
      phBorder = 15;
      pxWidth = 510; //initial form width and height, increased by 10
      pxHeight = 320;
    private
      pfAction : TFoxType; //prog form action, e.g. acBackup, acRestore, etc. populated by runFoxClone
      DeviceName : string; //so that the device/backup filename can be passed to the note form
      CancelFlag : boolean;
      BtnWidth, BtnHeight : integer;
      TotalBlocks : int64;  //total sectors to backup, restore, etc.
      PastBlocks : int64; //blocks on previous partitions that have been backed up, restored etc.
      Start : TDatetime; //when the backup, restore, etc. started
      FileSysLbl, DeviceLbl, SpaceLbl: TLabelGTK;
      ProgBar : TProgressBar;
      ProgBarLbl : TLabelGTK;
      ProgBarAll : TProgressBar;
      ProgLblAll : TLabelGTK;
      PartListBox : TListBox;
      CancelBtn : TButton;
      FinishBtn : TButton;
      AddLogBtn : TButton; //dual purpose, log button and add note button
      procedure ProgFormResize(Sender: TObject);
      procedure runBackup;
      procedure runRestore;
      procedure runClone;
      procedure runVerify;
      function runPartClone(pAction : TFoxType; Part : Tpart) : boolean;
      procedure ParseHeader(var HeaderStr : string);
      procedure UpdateProgress(UpdateStr : string);
      procedure KillPartClone(FileSystem : string);
      procedure CancelBtnCK(Sender : TObject);
      procedure FinishBtnCK(Sender : TObject);
      procedure AddLogBtnCK(Sender: TObject);
      procedure ProgFormClose(Sender: TObject; var CloseAction: TCloseAction);
      procedure UpdateLog(PartRef : string);
      procedure UmountAll; //unmounts any parts mounted by foxclone, run on prog form close
    public
      constructor CreateNew(TheOwner: TComponent);
  end;

procedure runFoxClone(fAction : TFoxType; FileName : string = '');

implementation

uses mainform; //for font characteristics and ownership of progform

{-------------------------------------------------------------------------------
CONSTRUCTOR TPROGRESSFM.CREATENEW
Constructor for the progress form. Note createnew not create and it is displayed with showmodal.

Used by:

V1.0 - 10 Oct 2021
-------------------------------------------------------------------------------}
constructor TProgressFm.CreateNew(TheOwner: TComponent);
begin
  inherited CreateNew(TheOwner,1);
  Caption:= 'Progress'; //caption for the title bar will be replaced
  Position:= poScreenCenter;
  SetInitialBounds(0,0,pxWidth,pxHeight); //defines initial size
  Constraints.MinHeight:= pxHeight;
  Constraints.MinWidth:= pxWidth;
  Font:= MainFm.Font; //to pick up the default font used
  OnResize:= @ProgFormResize;
  OnClose:= @ProgFormClose;
  CancelFlag:= false;
  TotalBlocks:= 0;
  PastBlocks:= 0;
  FileSysLbl:= TLabelGTK.Create(Self);
  with FileSysLbl do begin
       Parent:= self;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= self;
       BorderSpacing.top:= phBorder;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= self;
       BorderSpacing.left:= pwBorder;
       setCaption('File system:');
  end;
  DeviceLbl:= TLabelGTK.Create(Self);
  with DeviceLbl do begin
       Parent:= self;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= FileSysLbl;
       BorderSpacing.top:= 0;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= FileSysLbl;
       BorderSpacing.left:= 0;
       setCaption('Device Size:');
  end;
  SpaceLbl:= TLabelGTK.Create(Self);
  with SpaceLbl do begin
       Parent:= self;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= DeviceLbl;
       BorderSpacing.top:= 0;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= FileSysLbl;
       BorderSpacing.left:= 0;
       setCaption('Space in use:');
  end;
  ProgBar:= TProgressBar.Create(Self);
  with ProgBar do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= SpaceLbl;
       BorderSpacing.top:= phBorder;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= SpaceLbl;
       BorderSpacing.left:= 0;
       Width:= Self.Width - 2 * pwBorder;
       Height:= phBorder;
       Smooth:= True;
       Min:= 0;
  end;
  ProgBarLbl:= TLabelGTK.Create(Self);
  with ProgBarLbl do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= ProgBar;
       BorderSpacing.top:= phBorder div 2;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= ProgBar;
       BorderSpacing.left:= 0;
  end;
  ProgBarAll:= TProgressBar.Create(Self);
  with ProgBarAll do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= ProgBarLbl;
       BorderSpacing.top:= phBorder;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= ProgBarLbl;
       BorderSpacing.left:= 0;
       Width:= Self.Width - 2 * pwBorder;
       Height:= phBorder;
       Smooth:= True;
       Min:= 0;
  end;
  ProgLblAll:= TLabelGTK.Create(Self);
  with ProgLblAll do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= ProgBarAll;
       BorderSpacing.top:= phBorder div 2;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= ProgBarAll;
       BorderSpacing.left:= 0;
  end;
  //find out which button text is longest, Cancel Add Note Finish view log.
  //logbtn is both add note and logbtn
  //height is +14, width is +20
  BtnWidth:= getTextWidth(rNoteCancel,Font);
  if getTextWidth(rFinishStr,Font) > BtnWidth then BtnWidth:= getTextWidth(rFinishStr,Font);
  if getTextWidth(rAddNoteStr,Font) > BtnWidth then BtnWidth:= getTextWidth(rAddNoteStr,Font);
  if getTextWidth(rViewLogStr,Font) > BtnWidth then BtnWidth:= getTextWidth(rViewLogStr,Font);
  BtnWidth+= 20;
  BtnHeight:= getTextHeight(rViewLogStr,Font) + 14;
  FinishBtn:= TButton.Create(Self);
  with FinishBtn do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= Self;
       BorderSpacing.top:= Self.Height - BtnHeight - phBorder;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Self;
       BorderSpacing.left:= Self.Width - BtnWidth - pwBorder;
       Height:= BtnHeight;
       Width:= BtnWidth;
       Caption:= rFinishStr;
       Enabled:= false;
       OnClick:= @FinishBtnCK;
  end;
  CancelBtn:= TButton.Create(Self);
  with CancelBtn do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= FinishBtn;
       BorderSpacing.top:= 0;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Self;
       BorderSpacing.left:= Self.Width - 2 * BtnWidth - 2 *pwBorder;
       Height:= BtnHeight;
       Width:= BtnWidth;
       Caption:= rNoteCancel;
       OnClick:= @CancelBtnCK;
  end;
  AddLogBtn:= TButton.Create(Self);
  with AddLogBtn do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= Self;
       BorderSpacing.top:= Self.Height - 2 * BtnHeight - 2 * phBorder;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= FinishBtn;
       BorderSpacing.left:= 0;
       Height:= BtnHeight;
       Width:= BtnWidth;
       Caption:= rAddNoteStr;
       Enabled:= false;
       OnClick:= @AddLogBtnCK;
  end;
  PartListBox:= TListBox.Create(Self);
  with PartListBox do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= ProgLblAll;
       BorderSpacing.top:= phBorder;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= ProgLblAll;
       BorderSpacing.left:= 0;
       Width:= Self.Width - 2 * FinishBtn.Width - 5 * pwBorder; //div 2 - 2 * pwBorder;
       Height:= Self.Height - 3 * FileSysLbl.Height - 2 * ProgBar.Height - 2 * ProgBarLbl.Height - 6 * phBorder;
  end;
end; //end constructor TProgressFm.CreateNew
{-------------------------------------------------------------------------------
FUNCTION TPROGRESSFM.PROGFORMRESIZE
Event hander for resizing the progress form.

Note when running, partclone is on a 3 second cycle and the UI is not very responsive. The answer is
to click and drag the window when resizing and ONLY let go when the window has resized.

Used by:

V1.0 - 11 Oct 2021
-------------------------------------------------------------------------------}
procedure TProgressFm.ProgFormResize(Sender: TObject);
begin
  //width
  ProgBar.Width:= Self.Width - 2 * pwBorder;
  ProgBarAll.Width:= Self.Width - 2 * pwBorder;
  FinishBtn.BorderSpacing.left:= Self.Width - BtnWidth - pwBorder;
  CancelBtn.BorderSpacing.left:= Self.Width - 2 * BtnWidth - 2 *pwBorder;
  PartListBox.Width:= Self.Width - 2 * FinishBtn.Width - 5 * pwBorder;
  //height
  FinishBtn.BorderSpacing.top:= Self.Height - BtnHeight - phBorder;
  AddLogBtn.BorderSpacing.top:= Self.Height - 2 * BtnHeight - 2 * phBorder;
  PartListBox.Height:= Self.Height - 3 * FileSysLbl.Height - 2 * ProgBar.Height - 2 * ProgBarLbl.Height - 6 * phBorder;
  application.processmessages; //does this help resize display
end; //end TProgressFm.ProgFormResize
{-------------------------------------------------------------------------------
FUNCTION TPROGRESSFM.RUNBACKUP
The method to run a backup:

    backs up grub, but only if the source drive is legacy partition table. For a gpt partitition table
    grub is in the EFI partition. The grub backup uses dd.

    backs up the partition table with sfdisk.

    backs up each partition with runPartclone in a loop. Only partitions where selected=true are
    backed up.

Each of the functions doing the backup are boolean, when false the backup is terminated.

The function checks if the destination is fat32 and automatically sets split ON so that the 4GB file
size limit is not breached. Split is set OFF on completion.

On completion details are written to the backup file. This is as previous versions with the addition
of Version and Sector Size.

Used by:

V1.0 - 11 Oct 2021
V1.1 - 06 Dec 2021, cancel changed, break in the for loop on partclone and then an if clause to tidy up.
V1.2 - 18 Jan 2022, major error, was not saving the serial number in the backup file, so on a restore
                    was failing to find the drive to restore to - fixed.
V1.3 - 19 Jan 2022, cosmetic proglbl changed to show backup complete on finish
V1.4 - 25 Sep 2023, changed call to grub to grubbackup
-------------------------------------------------------------------------------}
procedure TProgressFm.runBackup;
const
  NoStartStr = ' [Not Started]';
  StartStr = ' [Started]';
  FailStr = ' [Failed xx]';
  NoSelectStr = ' [Not Selected]';
  CompleteStr = ' [Complete xx]';
  TimeStr = 'xx';
  BackExtn = '.backup';
  GrubExtn = '.grub';
  SFdiskExtn = '.sfdisk';
  CompSearchStr = 'Completed:'; //searched for in the overall progress label
  RemSearchStr = 'Remaining:';  //searched for in the overall progress label
  CompressStr = 'Compression:';
  SplitStr = 'Split files:';
  YesStr = 'YES';
  NoStr = 'NO';
  ModelStr = 'Model:';
  SectorStr = 'Sector size:';
  SerialStr = 'Serial:';
  MountStr = 'Mount point:';
  BlockStr = 'Blocks:';
  VersionStr = 'Version:';
  pTableStr = 'Part table:YES';
var
  i, dIndex : shortint;
  Elapsed : TDateTime;
  CompleteTime : string;
  pStart : TDateTime;
  TempStr1, TempStr2 : string;
  BackList : TStringList;
begin
  Start:= now;
  DebugInfo(true,'TProgressFm.runBackup','Filename=',DeviceName);
  //writeln('run backup called...');
  //find the source drive
  dIndex:= MyDrives.getDrive(acSource);
  if dIndex = -1 then SystemError(1,'TProgressFm.runBackup');
  //populate the list box
  PartListBox.Items.Add(rPartTableStr + NoStartStr);
  with MyDrives[dIndex] do
       for i:= 0 to TotalParts - 1 do
           if Partitions[i].selected then PartListBox.Items.Add(Partitions[i].ShortStr + NoStartStr)
           else PartListBox.Items.Add(Partitions[i].ShortStr + NoSelectStr);
  application.processmessages; //otherwise the progesss form will not appear
  //backup grub
  PartListBox.Items[0]:= replacestr(PartListBox.Items[0],NoStartStr,StartStr);
  //need to check if gpt drive - no grub checks that
  if not MyDrives[dIndex].GrubBackup(replacestr(DeviceName,BackExtn,GrubExtn)) then
     begin
       PartListBox.Items[0]:= replacestr(PartListBox.Items[0],StartStr,FailStr);
       exit;
     end;
  //backup the partition table - sfdisk
  if not MyDrives[dIndex].sfdisk(acBackup,replacestr(DeviceName,BackExtn,SFdiskExtn)) then
     begin
       PartListBox.Items[0]:= replacestr(PartListBox.Items[0],StartStr,FailStr);
       exit;
     end;
  //update listbox on time to do grub and partition table
  Elapsed:= now - Start;
  CompleteTime:= replacestr(CompleteStr,TimeStr,formatdatetime('hh:mm:ss',Elapsed));
  PartListBox.Items[0]:= replacestr(PartListBox.Items[0],StartStr,CompleteTime);
  //check if destination is fat32 and set split on
  mydrives.SetFat32Split(true); //true will turn it on if fat32, false turns it off
  with MyDrives[dIndex] do for i:= 0 to TotalParts - 1 do
       if Partitions[i].selected then TotalBlocks+= Partitions[i].Used;
  ProgBarAll.Position:= 0;
  ProgBarAll.Max:= TotalBlocks;
  //run the backup but only on selected partitions
  with MyDrives[dIndex] do for i:= 0 to TotalParts - 1 do with Partitions[i] do if selected then
       begin
         if CancelFlag then break;
         //set caption
         Self.Caption:= rBackupTitleStr + Partitions[i].Drive + Partitions[i].Number;
         application.processmessages; //otherwise the progesss form will not appear
         //set started in listbox which will be i+1
         PartListBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],NoStartStr,StartStr);
         //run partclone and get the time to backup the partition
         pStart:= now;
         if not UMount then //unmount the partition if mounted
            begin
              PartListBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],StartStr,FailStr);
              CompleteTime:= FormatDateTime('hh:nn:ss',now - pStart);
              PartlistBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],TimeStr,CompleteTime);
              break;
            end;
         //run partclone
         if runPartClone(acBackup,Partitions[i]) then //update the partlistbox, remount if previously mounted
            begin
              PartlistBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],StartStr,CompleteStr);
              CompleteTime:= FormatDateTime('hh:nn:ss',now - pStart);
              PartlistBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],TimeStr,CompleteTime);
              if PartMountPt <> '' then Mount(true,PartMountPt); //remount part if previously mounted
            end
         else //error
            begin
              PartlistBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],StartStr,FailStr);
              CompleteTime:= FormatDateTime('hh:nn:ss',now - pStart);
              PartlistBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],TimeStr,CompleteTime);
            end;
         //update completed blocks:
         PastBlocks+= Partitions[i].Used;
       end;
  if CancelFlag then //cancel the backup
     begin
       Self.Enabled:= false;
       FoxMsg(rNoteCancel,[rPartCloneCancelStr],mtInformation,[mbOk]);
       Close;
       exit;
     end;
  //finished - clear the labels
  FileSysLbl.setCaption(Space);
  DeviceLbl.setCaption(Space);
  SpaceLbl.setCaption(Space);
  //set overall % complete to 100% - it is likely to be calculated as 99.xx%
  TempStr1:= copy(ProgLblAll.Caption,1,pos(CompSearchStr,ProgLblAll.Caption)+10);
  TempStr2:= copy(ProgLblAll.Caption,pos('%',ProgLblAll.Caption),99);
  ProgLblAll.setCaption(TempStr1 + '100.00' + TempStr2);
  //time remaining is probably showing greater than zero
  TempStr1:= copy(ProgLblAll.Caption,1,pos(RemSearchStr,ProgLblAll.Caption)+10);
  TempStr2:= copy(ProgLblAll.Caption,pos(CompSearchStr,ProgLblAll.Caption),99);
  ProgLblAll.setCaption(TempStr1 + '00:00:00, ' + TempStr2);
  //set overall progress bar to 100%
  ProgBarAll.Position:= ProgBarAll.Max;
  //progbar label will be saying clone complete, not appropriate for backup
  ProgBarLbl.setCaption(rBackFinishStr);
  application.processmessages; //otherwise updated labels will not appear
  //set split off if it was turned on.
  MyDrives.SetFat32Split(false);
  //create the backup file
  BackList:= TStringList.Create;
  BackList.Add(FormatDateTime('dd mmm yyyy, hh:nn',now));
  BackList.Add(VersionStr + IntToStr(Version));
  BackList.Add(pTableStr); //automatic, no user choice, compatability with previous versions
  Case Defaults.BackSettings of
       seNone  : begin
                   BackList.Add(CompressStr + NoStr);
                   BackList.Add(SplitStr + NoStr);
                 end;
       seComp  : begin
                   BackList.Add(CompressStr + YesStr);
                   BackList.Add(SplitStr + NoStr);
                 end;
       seSplit : begin
                   BackList.Add(CompressStr + NoStr);
                   BackList.Add(SplitStr + YesStr);
                 end;
       seBoth  : begin
                   BackList.Add(CompressStr + YesStr);
                   BackList.Add(SplitStr + YesStr);
                 end;
  end;
  BackList.Add(ModelStr + MyDrives[dIndex].Model);
  BackList.Add(SerialStr + MyDrives[dIndex].Serial);
  BackList.Add(SectorStr + IntToStr(MyDrives[dIndex].DriveSector));
  BackList.Add(MountStr + MyDrives[dIndex].MountPt);
  with MyDrives[dIndex] do for i:= 0 to TotalParts - 1 do with Partitions[i] do
       begin
         BackList.Add(BackFileStr(MyDrives[dIndex].pTable));
         if Selected then BackList.Add(BlockStr + Drive + Number + ',' + IntToStr(Used))
         else BackList.Add(BlockStr + Drive + Number + ',0');
       end;
  BackList.Add('End');
  BackList.SaveToFile(DeviceName);
  BackList.free;
  //finished enable the buttons
  FinishBtn.Enabled:= true;
  AddLogBtn.Enabled:= true;
  CancelBtn.Enabled:= false;
  Self.Caption:= rBackFinishStr;
end; //end procedure TProgressFm.runBackup
{-------------------------------------------------------------------------------
FUNCTION TPROGRESSFM.RUNPARTCLONE
Runs partclone and through 'helper' procedures updates the progress form. runpartclone have been a method
of TPart, but partitions would need access to objects on the progress form. As a method of the
progress form, it needs access to some parameters in TPart, but these were already available publically.

If no errors are reported running partclone the function returns true, any errors returns false.

The output from partclone is on stderr so any error messages are mixed in with the 'normal' output.
Output is read in two stages:

    To capture the initial block containing partclone output on filesystem, drive size, etc.
    To capture progress updates from partclone. When partclone has finished it outputs a sync message
    and then a complete message, this is checked in the helper procedure

Each of these outputs is checked for 'error' and if found displays a message box, sets runpartclone
false and exits the procedure. If an error is found in the progress updates then the procedure kills
partclone, otherwise it could continue running in the background, potentially in an infinite loop.

The initial block is checked for Calculating bitmap as each output is read rather than waiting until the
complete block has been read - waiting for the end would mean this output would be overwritten by later
output.

Output from partclone has ESC at the end of each block of output (not quite, it is a couple of chars in).
The procedure checks output for ESC to determine if the required block of output has been read.

If the user cancels, partclone is killed and the procedure is exited. The check is made on both
repeat loops, initial and when looping on progress

Parameters:
    pAction - of acBackup, acRestore, etc. determines what command string is used to run partclone

    Part - the partition to be backed up, restored, etc.

    Filename - filename for backup, restore, etc. For a clone this will be a string containing the device

THIS WILL NEED AMENDING FOR RESTORE ETC. TO GET THE CORRECT COMMAND STRING FOR PARTCLONE

V1.0 - 20 Oct 2021
V1.1 - 03 Nov 2021, final version after some changes, now running error checks here.
V1.2 - 24 Nov 2021, amended for restore, added in case statement and restoring swap part
V1.3 - 26 Nov 2021, when running clone from file had errors not trapped, e.g.
                    write block 151523 ERROR:No space left on device
                    no ESC char in output and not trapped. Test on repeat on init string expanded
                    to test for ERROR and error checks on progress output expanded as well.
V1.4 - 04 Dec 2021, acVerify added to case statement on pfaction.
-------------------------------------------------------------------------------}
function TProgressFm.runPartClone(pAction : TFoxType; Part : Tpart) : boolean;
const
  bufsize = 2048; // Buffer size for reading output
  ESC = char($1B); //partclone terminates each update with an Esc character, but in two lines don't want the 2nd line
  BitmapStr = 'Calculating bitmap...'; //initial output from partclone
  ErrorStr = 'error'; //searched for in partclone output
var
  PartCloneStr : string;
  Buffer : string = '';
  tempstr : string;
  InitialStr : string = '';
  partProcess : TProcess; //process to run partclone and read output
  BytesRead : integer;
  dIndex : shortint;
  DestDrive : string;
begin
  runPartClone:= false; //default is failure
  DebugInfo(false,'TProgressFm.runPartClone',Part.Drive + Part.Number,DeviceName);
  case pAction of
       acBackup  : PartCloneStr:= Part.BackStr(DeviceName);
       acRestore,acCloneF : begin
                              //need to find the destination drive set acTarget
                              dIndex:= MyDrives.getDrive(acTarget);
                              if dIndex = -1 then SystemError(12,'TProgressFm.runPartClone');
                              DestDrive:= MyDrives[dIndex].MountPt + MyDrives[dIndex].Prefix;
                              PartCloneStr:= Part.RestoreStr(DeviceName,DestDrive);
                            end;
       acClone  : begin
                    PartCloneStr:= Part.CloneStr(DeviceName);
                  end;
       acVerify : begin
                    PartCloneStr:= Part.VerifyStr(DeviceName);
                  end;
  end;
  if PartCloneStr = '' then //nothing to do, a swap or extended part, deal with swap separately
     begin
       runPartClone:= true;
       exit;
     end;
  //writeln('partclone commmand =',PartCloneStr);
  //set current progress bar
  ProgBar.Position:= 0;
  ProgBar.Max:= Part.Used;
  //run partclone
  partProcess:= TProcess.Create(Self);
  termCmd(PartCloneStr,partProcess);
  //get the header info on file system, drive size, used blocks
  //need to keep reading because will not get the header info in one hit, at the end of the header
  //output partclone generates an ESC character before then reporting progress
  //BytesRead=0 indicates an error of some sort
  repeat
    if CancelFlag then //kill partclone, exit
       begin
         KillPartClone(Part.FileSystem);
         partProcess.free;
         exit;
       end;
    setlength(Buffer,bufsize);
    BytesRead:= partProcess.StdErr.Read(Buffer[1],length(Buffer)); //this works
    setlength(Buffer,BytesRead);
    if pos(BitmapStr,Buffer) > 0 then
       begin
         ProgBarLbl.setCaption(BitmapStr);
         application.processmessages; //otherwise updated labels will not appear
       end;
    InitialStr+= Buffer;
    //writeln('buffer=',Buffer);
  until (pos(ESC,Buffer) > 0) or (pos('ERROR',Buffer) > 0) or (BytesRead = 0);
  DebugInfo(false,'runPartClone','Initialstr=',InitialStr);
  //writeln('initial string=',InitialStr);
  //test for error
  if (pos(ErrorStr,InitialStr) > 0) or (pos('ERROR',InitialStr) > 0) then
     begin
       FoxMsg(rErrorStr,[rPartCloneMsgStr,InitialStr],mtError,[mbOK]);
       partProcess.free;
       exit;
     end;
  ParseHeader(InitialStr); //pull out filesystem, size, used
  //now get the rest of the partclone output where it is updating progress
  repeat
    if CancelFlag then //kill partclone, exit
       begin
         KillPartClone(Part.FileSystem);
         partProcess.free;
         exit;
       end;
    setlength(Buffer,bufsize);
    BytesRead:= partProcess.StdErr.Read(Buffer[1],length(Buffer));
    setlength(Buffer,BytesRead);
    if BytesRead > 0 then
       begin
         //each update is terminated with an ESC char, if not ESC then we don't have a complete line
         //so read again, only once. If put in a repeat/until could end up in an infinite loop if an
         //error and partclone terminates without an ESC output
         if pos(ESC,Buffer) = 0 then
            begin
              tempstr:= buffer;
              setlength(Buffer,bufsize);
              BytesRead:= partProcess.StdErr.Read(Buffer[1],length(Buffer)); //this works
              if BytesRead > 0 then
                 begin
                   setlength(Buffer,BytesRead);
                   Buffer:= tempstr + Buffer;
                 end;
            end;
         DebugInfo(false,'TProgressFm.runPartClone',Buffer);
         //test for error
         if (pos(ErrorStr,Buffer) > 0) or (pos('ERROR',Buffer) > 0) then
            begin
              KillPartClone(Part.FileSystem);
              FoxMsg(rErrorStr,[rPartCloneMsgStr,Buffer],mtError,[mbOK]);
              exit;
            end;
         //update the progress form
         UpdateProgress(Buffer);
       end;
    //writeln('runpart progress output=',Buffer);
  until BytesRead = 0;
  runPartClone:= true;
  partProcess.free;
end; //end function TProgressFm.runPartClone
{-------------------------------------------------------------------------------
PROCEDURE TPROGRESSFM.UPDATEPROGRESS
Parses the string passed and updates the progress bars and labels on the progress form.

The string passed is normally of the form:

    Elapsed: 00:00:22, Remaining: 00:00:38, Completed:  36.42%,   6.47GB/min
    plus additional output, it is terminated with an ESC character near the end of the string.

But it may also be at the end of a backup/restore, etc. where partclone is either syncing the outputs
or reporting completion.

The procedure tests for a valid progress output or the sync or complete outputs.

Where the output is a progress update, the elapsed time string is parsed out and used 'as is' in the
current progress label. The % complete is pulled out and used to update the progress bar based on
'current blocks' - a 'global' parameter in the TProgressFm definition. Partclone appears to calculate
used blocks on a different basis to that used in foxclone, so will give slightly different estimates
for time to complete, hence using the % complete not the completed blocks reported.

The procedure creates the overall progress label based on the template in OverallStr using the % complete
reported by partclone to calculate the overall progress parameters and set the overall progress bar.

As with 'current blocks', 'past blocks' and total blocks are global parameters in the TProgressFm
definition.

There is no error checking, error checking is done before the procedure is called.

Bug in partclcone, UpdateProgress is parsing the output from partclone, viz:

    Elapsed: 00:05:00, Remaining: 00:00:01, Completed:  99,39%,   4,85GB/min,
    Elapsed: 00:05:02, Remaining: 00:00:00, Completed: 100.00%, Rate:   4,84GB/min,

This is foxclone running where LC_NUMERIC is set de = german. If foxclone is running where LC_NUMERIC
is not dot (.), e.g. comma (,) as in most European countries, notice how the output from partclone
has changed from ',' to '.' when reporting 100% progress.

Parameters:

    UpdateStr - string containing the text above, read from partclone.

V1.0 - 31 Oct 2021
V1.1 - 03 Nov 2021, final version after some changes, it was a function then back to procedure
V1.2 - 08 Jan 2021, change due to bug found in partclone, see comments above. Try statement added.
-------------------------------------------------------------------------------}
procedure TProgressFm.UpdateProgress(UpdateStr : string);
const
  OverallStr = 'Overall:   [e], Remaining: [r], Completed:  [c]%,  [s]GB/min'; //overall prog lbl format
  ElapsedStr = '[e]'; //kept short - replace function should be quicker
  RemainStr = '[r]';
  CompleteStr = '[c]';
  SpeedStr = '[s]';
  BlockSize = 1024; //used to calculate the rate, default size unit in foxclone is 1k blocks
  Gigabyte = 1e9;   //used to calculate the rate
  ESC = char($1B); //partclone terminates each update with an Esc character
  Elapsed = 'Elapsed'; //used in the search for a complete progress update
  MinStr = '/min'; //appended to the parsed output string for current progress label
  Completed = 'Completed:';
  SyncStr = 'Syncing';
  SyncMsg = SyncStr + '...';
  SuccessStr = 'successfully';
  SuccessMsg = 'Cloned ' + SuccessStr;
var
  Overall : string; //contains overallstr which is then replaced by the elements
  Percent, AllPercent : real;
  CurrentBlocks : int64; //blocks backed up/restored inc. previous partitions
  ElapsedSecs, RemainingSecs : integer;
  Rate : int64; //rate in blocks/minute
  RateGB : real; //rate in GB/minute
begin
  if (pos(Elapsed,UpdateStr) > 0) and (pos(ESC,UpdateStr) > 0) then //a complete update line, parse it
     begin
       UpdateStr:= copy(UpdateStr,pos(Elapsed,UpdateStr),99);
       //set the progress label
       ProgBarLbl.setCaption(copy2Symb(UpdateStr,'/')+MinStr);
       //set the progress bar
       UpdateStr:= copy(UpdateStr,pos(Completed,UpdateStr),99);
       UpdateStr:= replacestr(UpdateStr,Completed,'');
       //writeln('updatestr before copy2symb=',UpdateStr);
       UpdateStr:= Copy2Symb(UpdateStr,'%');
       //bug fix due to partclone not reporting 100% correctly
       try
         Percent:= StrToFloat(UpdateStr)/100;
       except
         Percent:= 1.00;
       end;
       CurrentBlocks:= round(ProgBar.Max * Percent);
       ProgBar.Position:= round(CurrentBlocks);
       //set the overall progress label format, replace the labels with current values
       //'Overall:  'Syncing' [e], Remaining: [r], Completed:  [c]%,  [s]GB/min'
       Overall:= OverallStr;
       //elapsed time
       Overall:= replacestr(Overall,ElapsedStr,formatdatetime('hh:mm:ss',Now - Start));
       //percent complete
       AllPercent:= (PastBlocks + CurrentBlocks) / TotalBlocks;
       Overall:= replacestr(Overall,CompleteStr,FormatFloat('##0.00',AllPercent * 100));
       //writeln('overall %=',FormatFloat('##0.00',AllPercent * 100));
       //remaining time
       ElapsedSecs:= SecondsBetween(Now,Start);
       //TotalSecs:= round(TotalBlocks/(PastBlocks + CurrentBlocks) * ElapsedSecs);
       RemainingSecs:= round((TotalBlocks/(PastBlocks + CurrentBlocks) - 1) * ElapsedSecs);
       Overall:= replacestr(Overall,RemainStr,formatdatetime('hh:mm:ss',IncSecond(0,RemainingSecs)));
       //rate
       Rate:= round((PastBlocks + CurrentBlocks)/ElapsedSecs*60); //rate per minute in blocks
       RateGB:= Rate * BlockSize / Gigabyte;
       Overall:= replacestr(Overall,SpeedStr,FormatFloat('##0.00',RateGB));
       //set the overall progress label and progress bar
       ProgLblAll.setCaption(Overall);
       ProgBarAll.Position:= round(TotalBlocks * AllPercent);
     end
  else //check for syncing or complete
     begin
       if pos(SyncStr,UpdateStr) > 0 then //partclone has finished and is syncing
          begin
            ProgBarLbl.setCaption(SyncMsg);
            ProgBar.Position:= ProgBar.Max;
          end;
       if pos(SuccessStr,UpdateStr) > 0 then //partclone has finished
          begin
            ProgBarLbl.setCaption(SuccessMsg);
            ProgBar.Position:= ProgBar.Max;
          end;
     end;
  application.processmessages; //otherwise updated labels will not appear
end; //end procedure TProgressFm.UpdateProgress
{-------------------------------------------------------------------------------
PROCEDURE TPROGRESSFM.PARSEHEADER
The text that the procedure is looking to parse is of the form:
     File system:  EXTFS
     Device size:   15.7 GB = 3840000 Blocks
     Space in use:   6.5 GB = 1590461 Blocks
     Free Space:     9.2 GB = 2249539 Blocks
     Block size:   4096 Byte

This is parsed into the header labels on the progress form. Block size & Free space are not used
and blocks after '=' are stripped out.

The procedure tests to check that the string passed contains the above info, if not it exits doing
nothing.

There is no error checking, error checking is done before the procedure is called.

Parameters:

    HeaderStr - string containing the text above, read from partclone.

V1.0 - 30 Oct 2021
V1.1 - 03 Nov 2021, final version after some changes, it was a function then back to procedure
-------------------------------------------------------------------------------}
procedure TProgressFm.ParseHeader(var HeaderStr : string);
const
  FileStr = 'File system';
  BlockStr = 'Block size';
  DeviceStr = 'Device size';
  SpaceStr = 'Space in use';
var
  tempstr : string;
begin
  //check we have the info in the headerstr
  if (pos(FileStr,HeaderStr) > 0) and (pos(BlockStr,HeaderStr) > 0) then
     begin
       tempstr:= copy(HeaderStr,pos(FileStr,HeaderStr),99);
       FileSysLbl.setCaption(Copy2Symb(tempstr,LF));
       tempstr:= copy(tempstr,pos(DeviceStr,tempstr),99);
       DeviceLbl.setCaption(Copy2Symb(tempstr,'='));
       tempstr:= copy(tempstr,pos(SpaceStr,tempstr),99);
       SpaceLbl.setCaption(Copy2Symb(tempstr,'='));
       application.processmessages; //otherwise updated labels will not appear
     end;
end; //end procedure TProgressFm.ParseHeader
{-------------------------------------------------------------------------------
FUNCTION TPROGRESSFM.RUNVERIFY
The method to verify a backup image file by running partclone.chkimg:

    does nothing about the part table or grub.

    cannot check images of unknown filesystems, i.e. using dd or extended or swap partitions.

Simplest of the all foxclone backup/restore/clone methods.

Used by:

V1.0 - 11 Oct 2021
V1.1 - 06 Dec 2021, cancel changed, break in the for loop on partclone and then an if clause to tidy up.
V1.2 - 19 Jan 2021, cosmetic changed proglbl at end to show verify complete.
-------------------------------------------------------------------------------}
procedure TProgressFm.runVerify;
const
  NoStartStr = ' [Not Started]';
  StartStr = ' [Started]';
  FailStr = ' [Failed xx]';
  NoSelectStr = ' [Not Selected]';
  CompleteStr = ' [Complete xx]';
  TimeStr = 'xx';
  CompSearchStr = 'Completed:'; //searched for in the overall progress label
  RemSearchStr = 'Remaining:';  //searched for in the overall progress label
var
  i : shortint;
  pStart  : TDateTime;
  CompleteTime : string;
  TempStr1, TempStr2 : string;
begin
  Self.Caption:= rVerifyTabStr;
  //writeln('run verify called...');
  //writeln('verify fname=',DeviceName);
  Start:= now;
  DebugInfo(true,'TProgressFm.runVerify');
  //Populate the listbox, data is in RDrive
  with RDrive do for i:= 0 to TotalParts - 1 do
       if VerifyTab.RestoreSelect(i) then PartListBox.Items.Add(Partitions[i].ShortStr + NoStartStr)
       else PartListBox.Items.Add(Partitions[i].ShortStr + NoSelectStr);
  TotalBlocks:= 0;
  with RDrive do for i:= 0 to TotalParts - 1 do
       if VerifyTab.RestoreSelect(i) then TotalBlocks+= Partitions[i].Used;
  ProgBarAll.Position:= 0;
  ProgBarAll.Max:= TotalBlocks;
  //writeln('before verify for loop');
  for i:= 0 to RDrive.TotalParts - 1 do with RDrive do //verify partitions
      begin
        if CancelFlag then break;
        //set caption
        Self.Caption:= rVerifyTabStr + Space + Partitions[i].Drive + Partitions[i].Number;
        //set started in listbox
        PartListBox.Items[i]:= replacestr(PartListBox.Items[i],NoStartStr,StartStr);
        if VerifyTab.RestoreSelect(i) then
           begin
             pStart:= now;
             //run partclone
             if runPartClone(pfAction,Partitions[i]) then
                begin
                  PartlistBox.Items[i]:= replacestr(PartListBox.Items[i],StartStr,CompleteStr);
                  CompleteTime:= FormatDateTime('hh:nn:ss',now - pStart);
                  PartlistBox.Items[i]:= replacestr(PartListBox.Items[i],TimeStr,CompleteTime);
                end
             else //error
                begin
                  PartlistBox.Items[i]:= replacestr(PartListBox.Items[i],StartStr,FailStr);
                  CompleteTime:= FormatDateTime('hh:nn:ss',now - pStart);
                  PartlistBox.Items[i]:= replacestr(PartListBox.Items[i],TimeStr,CompleteTime);
                end;
             //update the log file
             UpdateLog(Partitions[i].Drive + Partitions[i].Number);
             //update completed blocks
             PastBlocks+= Partitions[i].Used;
           end;
      end;
  if CancelFlag then //cancel the verify
     begin
       Self.Enabled:= false;
       FoxMsg(rNoteCancel,[rPartCloneCancelVStr],mtInformation,[mbOk]);
       MainFm.Enabled:= true;
       Close;
       exit;
     end;
  //finished - clear the labels
  FileSysLbl.setCaption(Space);
  DeviceLbl.setCaption(Space);
  SpaceLbl.setCaption(Space);
  //set overall % complete to 100% - it is likely to be calculated as 99.xx%
  TempStr1:= copy(ProgLblAll.Caption,1,pos(CompSearchStr,ProgLblAll.Caption)+10);
  TempStr2:= copy(ProgLblAll.Caption,pos('%',ProgLblAll.Caption),99);
  ProgLblAll.setCaption(TempStr1 + '100.00' + TempStr2);
  //time remaining is probably showing greater than zero
  TempStr1:= copy(ProgLblAll.Caption,1,pos(RemSearchStr,ProgLblAll.Caption)+10);
  TempStr2:= copy(ProgLblAll.Caption,pos(CompSearchStr,ProgLblAll.Caption),99);
  ProgLblAll.setCaption(TempStr1 + '00:00:00, ' + TempStr2);
  //set overall progress bar to 100%
  ProgBarAll.Position:= ProgBarAll.Max;
  //progbar label will be saying clone complete, not appropriate for verify
  ProgBarLbl.setCaption(rVerifyFinStr);
  application.processmessages; //otherwise updated labels will not appear
  //finished enable the buttons
  FinishBtn.Enabled:= true;
  CancelBtn.Enabled:= false;
  AddLogBtn.Enabled:= true;
  Self.Caption:= rVerifyFinStr;
end; //end procedure TProgressFm.runVerify
{-------------------------------------------------------------------------------
FUNCTION TPROGRESSFM.RUNRESTORE
The method to run a restore and clone from file:

    restores grub, but only if the source drive is legacy partition table. For a gpt partitition table
    grub is in the EFI partition.

    restores the partition table with sfdisk. Failure to restore the part table results in foxclone
    terminating.

    restores each partition with runPartclone in a loop. Only partitions where selected=true are
    restored. All partitions are restored on a clone from file.

Each of the functions doing the restore are boolean, when false the restores of partitions are not
terminated, the listbox shows failed status.

Log files are dumped in the default folder.

Used by:

V1.0 - 12 Nov 2021
V1.1 - 21 Nov 2021, final version
V1.2 - 24 Nov 2021, modified so that it can be used for clone from file
V1.3 - 06 Dec 2021, added check that parts > 0 before unmounting destination parts
V1.4 - 05 Jan 2022, enable finish button on error exit from sfdisk and grub
V1.5 - 19 Jan 2022, cosmetic show restore complete on proglbl when finished
-------------------------------------------------------------------------------}
procedure TProgressFm.runRestore;
const
  NoStartStr = ' [Not Started]';
  StartStr = ' [Started]';
  FailStr = ' [Failed xx]';
  FailStr2 = 'Failed';
  NoSelectStr = ' [Not Selected]';
  CompleteStr = ' [Complete xx]';
  CompleteStr2 = 'Complete';
  TimeStr = 'xx';
  BackExtn = '.backup';
  GrubExtn = '.grub';
  SFdiskExtn = '.sfdisk';
  CompSearchStr = 'Completed:'; //searched for in the overall progress label
  RemSearchStr = 'Remaining:';  //searched for in the overall progress label
var
  i, dIndex : shortint;
  DestDrive : string;
  pStart  : TDateTime;
  CompleteTime : string;
  TempStr1, TempStr2 : string;
  RestoreFlag : array of boolean;
  TitleStr : string;
begin
  //procedure has to do with both restore and clone from file, for restore the action is determined by
  //RestoreTab.RestoreSelect. Calling this when doing a clone will cause an error. Checking throughout
  //the procedure if acRestore or acCloneF is messy. Set RestoreFlag up front depending on if acRestore
  //or acCloneF, rest of procedure only has to check RestoreFlag
  setlength(RestoreFlag,RDrive.TotalParts+1); //item 0 in the listbox is always the partition table
  case pfAction of
       acRestore : begin
                     for i:= 0 to length(RestoreFlag) - 1 do RestoreFlag[i]:= RestoreTab.RestoreSelect(i);
                     TitleStr:= rRestTabStr;
                   end;
       acCloneF  : begin
                     for i:= 0 to length(RestoreFlag) - 1 do RestoreFlag[i]:= CloneTab.RestoreSelect(i);
                     TitleStr:= rCloneFtitleStr;
                   end;
  end;
  Self.Caption:= TitleStr;
  //writeln('run restore called...');
  Start:= now;
  //turn off swap
  SwapOn(false);
  DebugInfo(true,'runRestore',DeviceName);
  //delete any log files in the default folder /tmp/foxclone so that view logs only contains log files
  //from the current restore. The pascal DeleteFile fn does not seem to work with wildcards
  //works but need a polkit for rm - done
  getStr('pkexec rm ' + DefaultFolder + '*.log');
  //populate the list box, first item is the partition table
  if RestoreFlag[0] then PartListBox.Items.Add(rPartTableStr + NoStartStr)
  else PartListBox.Items.Add(rPartTableStr + NoSelectStr);
  //now the rest of the list box, part references need to match the destination, what was sda on the
  //backup may now be sdc or on a clone from file will always be different
  //first find target drive
  dIndex:= MyDrives.getDrive(acTarget);
  if dIndex = -1 then SystemError(12,'TProgresFm.runRestore');
  with RDrive do for i:= 0 to TotalParts - 1 do
       if RestoreFlag[i+1] then PartListBox.Items.Add(Partitions[i].ShortStr + NoStartStr)
       else PartListBox.Items.Add(Partitions[i].ShortStr + NoSelectStr);
  //now change the part reference
  for i:= 0 to PartListBox.Count - 1 do
      PartListBox.Items[i]:= replacestr(PartListBox.Items[i],RDrive.MountPt + RDrive.Prefix,
                                        MyDrives[dIndex].MountPt + MyDrives[dIndex].Prefix);
  application.processmessages; //otherwise the progess form will not appear
  //umount any partitions on the destination drive
  if MyDrives[dIndex].TotalParts > 0 then
     for i:= 0 to MyDrives[dIndex].TotalParts - 1 do MyDrives[dIndex].Partitions[i].UMount;
  //DestDrive - mountpoint may have changed between the backup and restore, e.g. was sda now sdd and
  //it will be different when cloning from file
  DestDrive:= MyDrives[dIndex].MountPt + MyDrives[dIndex].Prefix;
  //restore grub and the partition table, but only if selected
  if RestoreFlag[0] then
     begin
       Self.Caption:= rRestPTable;
       application.processmessages; //otherwise the progesss form will not appear
       PartListBox.Items[0]:= replacestr(PartListBox.Items[0],NoStartStr,StartStr);
       //writeln('checked that part table selected...');
       //restore the part table
       if not MyDrives[dIndex].sfdisk(pfAction,replacestr(DeviceName,BackExtn,SFdiskExtn)) then
       with PartListBox do
            begin
              Items[0]:= replacestr(Items[0],StartStr,FailStr);
              Items[0]:= replacestr(Items[0],TimeStr,formatdatetime('hh:mm:ss',now-Start));
              FinishBtn.Enabled:= true;
              exit;
            end;
       //restore grub
       //writeln('after sfdisk restore');
       if not MyDrives[dIndex].GrubRestore(replacestr(DeviceName,BackExtn,GrubExtn)) then
       with PartListBox do
            begin
              Items[0]:= replacestr(Items[0],StartStr,FailStr);
              Items[0]:= replacestr(Items[0],TimeStr,formatdatetime('hh:mm:ss',now-Start));
              FinishBtn.Enabled:= true;
              exit;
            end;
       PartListBox.Items[0]:= replacestr(PartListBox.Items[0],StartStr,CompleteStr);
       PartListBox.Items[0]:= replacestr(PartListBox.Items[0],TimeStr,formatdatetime('hh:mm:ss',now-Start));
     end;
  //restore partitions has to be run on rdrive, while sfdisk has created the partition table the
  //filesystems may not show correctly.
  //calculate total blocks to restore, no need to test for extended part, pused is already set = 0
  TotalBlocks:= 0;
  with RDrive do for i:= 0 to TotalParts - 1 do
       if RestoreFlag[i+1] then TotalBlocks+= Partitions[i].Used;
  ProgBarAll.Position:= 0;
  ProgBarAll.Max:= TotalBlocks;
  for i:= 0 to RDrive.TotalParts - 1 do with RDrive do //restore partitions
      begin
        //set caption
        Self.Caption:= TitleStr + Space + DestDrive + Partitions[i].Number;
        //set started in listbox which will be i+1
        PartListBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],NoStartStr,StartStr);
        if RestoreFlag[i+1] then //i+1 because box3 restore/cloneF has part table as item 0
           begin
             pStart:= now;
             //run partclone
             if runPartClone(pfAction,Partitions[i]) then //update the partlistbox
                begin
                  PartlistBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],StartStr,CompleteStr);
                  CompleteTime:= FormatDateTime('hh:nn:ss',now - pStart);
                  PartlistBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],TimeStr,CompleteTime);
                end
             else //error
                begin
                  PartlistBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],StartStr,FailStr);
                  CompleteTime:= FormatDateTime('hh:nn:ss',now - pStart);
                  PartlistBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],TimeStr,CompleteTime);
                end;
             //update completed blocks
             PastBlocks+= Partitions[i].Used;
           end;
      end;
  //check for and restore any swap partitions
  for i:= 0 to RDrive.TotalParts - 1 do with RDrive do
      if (RestoreFlag[i+1]) and (Partitions[i].FileSystem = 'swap') then
         if not Partitions[i].MakeSwap(DestDrive) then //error restoring swap
            PartlistBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],CompleteStr2,FailStr2);
  //finished - clear the labels
  FileSysLbl.setCaption(Space);
  DeviceLbl.setCaption(Space);
  SpaceLbl.setCaption(Space);
  //set overall % complete to 100% - it is likely to be calculated as 99.xx%
  TempStr1:= copy(ProgLblAll.Caption,1,pos(CompSearchStr,ProgLblAll.Caption)+10);
  TempStr2:= copy(ProgLblAll.Caption,pos('%',ProgLblAll.Caption),99);
  ProgLblAll.setCaption(TempStr1 + '100.00' + TempStr2);
  //time remaining is probably showing greater than zero
  TempStr1:= copy(ProgLblAll.Caption,1,pos(RemSearchStr,ProgLblAll.Caption)+10);
  TempStr2:= copy(ProgLblAll.Caption,pos(CompSearchStr,ProgLblAll.Caption),99);
  ProgLblAll.setCaption(TempStr1 + '00:00:00, ' + TempStr2);
  //set overall progress bar to 100%
  ProgBarAll.Position:= ProgBarAll.Max;
  //progbar label will be saying clone complete, not appropriate for restore
  if pfAction = acRestore then ProgBarLbl.setCaption(rRestFinishStr)
  else ProgBarLbl.setCaption(rCloneFfinStr);
  application.processmessages; //otherwise updated labels will not appear
  //finished enable the buttons
  FinishBtn.Enabled:= true;
  AddLogBtn.Enabled:= true;
  case pfAction of
       acRestore : Self.Caption:= rRestFinishStr;
       acCloneF  : Self.Caption:= rCloneFfinStr;
  end;
  //turn swap back on
  SwapOn(true);
end; //end procedure TProgressFm.runRestore
{-------------------------------------------------------------------------------
PROCEDURE RUNCLONE
The method to run a clone:

    restores grub, but only if the source drive is legacy partition table. For a gpt partitition table
    grub is in the EFI partition.

    restores the partition table with sfdisk. Failure to restore the part table results in foxclone
    terminating.

    restores each partition with runPartclone in a loop. Only partitions where selected=true are
    restored. All partitions are restored on a clone from file.

Each of the functions doing the restore are boolean, when false the restores of partitions are not
terminated, the listbox shows failed status.

Log files are dumped in the default folder.

Updates the progress form with progress, on the progress bars and labels.

Follows the same form as restore and backup, but slightly simpler.

Used by:

V1.0 - 26 Nov 2021
V1.1 - 05 Jan 2022, finish button enabled on error exit from grub and sfdisk
V1.2 - 19 Jan 2022, cosmetic proglbl at end showing clone drive to drive complete
V1.3 - 27 Sep 2023, change to grubClone instead of generic grub.
-------------------------------------------------------------------------------}
procedure TProgressFm.runClone;
const
  NoStartStr = ' [Not Started]';
  StartStr = ' [Started]';
  FailStr = ' [Failed xx]';
  FailStr2 = 'Failed';
  CompleteStr = ' [Complete xx]';
  CompleteStr2 = 'Complete';
  TimeStr = 'xx';
  CompSearchStr = 'Completed:'; //searched for in the overall progress label
  RemSearchStr = 'Remaining:';  //searched for in the overall progress label
var
  i, sIndex, tIndex : shortint;  //index for source and destination drives
  pStart  : TDateTime;
  CompleteTime : string;
  TempStr1, TempStr2 : string;
  sDevice : string; //in the form sda or nvme0n1p
begin
  Self.Caption:= rCloneTitleStr;
  //writeln('run clone drive to drive called...');
  Start:= now;
  //turn off swap
  SwapOn(false);
  DebugInfo(true,'runClone');
  //delete any log files in the default folder /tmp/foxclone so that view logs only contains log files
  //from the current clone. The pascal DeleteFile fn does not seem to work with wildcards
  //works but need a polkit for rm - done
  getStr('pkexec rm ' + DefaultFolder + '*.log');
  //find the source and destination drives
  sIndex:= MyDrives.getDrive(acSource);
  if sIndex = -1 then SystemError(20,'TProgresFm.runClone');
  tIndex:= MyDrives.getDrive(acTarget);
  if tIndex = -1 then SystemError(24,'TProgresFm.runRestore');
  //populate DeviceName - for other actions it is the backupfile name, for clone it is the
  //destination drive plus prefix
  DeviceName:= MyDrives[tIndex].MountPt + MyDrives[tIndex].Prefix;
  //populate the list box, first item is the partition table
  PartListBox.Items.Add(rPartTableStr + NoStartStr);
  //now the rest of the list box, populate from source drive, but replace drive reference to match
  //target drive
  with MyDrives[sIndex] do for i:= 0 to TotalParts - 1 do PartListBox.Items.Add(Partitions[i].ShortStr + NoStartStr);
  //now change the part reference
  sDevice:= MyDrives[sIndex].MountPt + MyDrives[sIndex].Prefix;
  for i:= 0 to PartListBox.Count - 1 do PartListBox.Items[i]:= replacestr(PartListBox.Items[i],sDevice,DeviceName);
  application.processmessages; //otherwise the progess form will not appear
  //umount any partitions on the source and destination drives
  for i:= 0 to MyDrives[sIndex].TotalParts - 1 do MyDrives[sIndex].Partitions[i].UMount;
  //if the target is blank no partitions
  with MyDrives[tIndex] do if TotalParts > 0 then for i:= 0 to TotalParts - 1 do Partitions[i].UMount;
  //restore grub and the partition table
  Self.Caption:= rClonePTable;
  application.processmessages; //otherwise the progesss form will not appear
  PartListBox.Items[0]:= replacestr(PartListBox.Items[0],NoStartStr,StartStr);
  //restore the part table, called on the source drive because sfdisk needs the part table
  if not MyDrives[sIndex].sfdisk(pfAction) then with PartListBox do
     begin
       Items[0]:= replacestr(Items[0],StartStr,FailStr);
       Items[0]:= replacestr(Items[0],TimeStr,formatdatetime('hh:mm:ss',now-Start));
       FinishBtn.Enabled:= true;
       exit;
     end;
  //restore grub - run on source drive so grub can find out if gpt or msdos
  if not MyDrives[sIndex].GrubClone then
       with PartListBox do
            begin
              Items[0]:= replacestr(Items[0],StartStr,FailStr);
              Items[0]:= replacestr(Items[0],TimeStr,formatdatetime('hh:mm:ss',now-Start));
              FinishBtn.Enabled:= true;
              exit;
            end;
  PartListBox.Items[0]:= replacestr(PartListBox.Items[0],StartStr,CompleteStr);
  PartListBox.Items[0]:= replacestr(PartListBox.Items[0],TimeStr,formatdatetime('hh:mm:ss',now-Start));
  //calculate total blocks to restore, no need to test for extended part, pused is already set = 0
  TotalBlocks:= 0;
  for i:= 0 to MyDrives[sIndex].TotalParts - 1 do TotalBlocks+= MyDrives[sIndex].Partitions[i].Used;
  ProgBarAll.Position:= 0;
  ProgBarAll.Max:= TotalBlocks;
  //running the partition clone on the source drive
  for i:= 0 to MyDrives[sIndex].TotalParts - 1 do with MyDrives[sIndex] do //restore partitions
      begin
        //set caption
        Self.Caption:= rCloneTitleStr + Space + DeviceName + Partitions[i].Number;
        //set started in listbox which will be i+1
        PartListBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],NoStartStr,StartStr);
        pStart:= now;
        //run partclone
        if runPartClone(pfAction,Partitions[i]) then //update the partlistbox
           begin
             PartlistBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],StartStr,CompleteStr);
             CompleteTime:= FormatDateTime('hh:nn:ss',now - pStart);
             PartlistBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],TimeStr,CompleteTime);
           end
        else //error
           begin
             PartlistBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],StartStr,FailStr);
             CompleteTime:= FormatDateTime('hh:nn:ss',now - pStart);
             PartlistBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],TimeStr,CompleteTime);
           end;
        //update completed blocks
        PastBlocks+= Partitions[i].Used;
      end;
  //check for and restore any swap partitions
  for i:= 0 to MyDrives[sIndex].TotalParts - 1 do with MyDrives[sIndex] do
      if Partitions[i].FileSystem = 'swap' then
         if not Partitions[i].MakeSwap(DeviceName) then //error restoring swap
            PartlistBox.Items[i+1]:= replacestr(PartListBox.Items[i+1],CompleteStr2,FailStr2);
  //finished - clear the labels
  FileSysLbl.setCaption(Space);
  DeviceLbl.setCaption(Space);
  SpaceLbl.setCaption(Space);
  //set overall % complete to 100% - it is likely to be calculated as 99.xx%
  TempStr1:= copy(ProgLblAll.Caption,1,pos(CompSearchStr,ProgLblAll.Caption)+10);
  TempStr2:= copy(ProgLblAll.Caption,pos('%',ProgLblAll.Caption),99);
  ProgLblAll.setCaption(TempStr1 + '100.00' + TempStr2);
  //time remaining is probably showing greater than zero
  TempStr1:= copy(ProgLblAll.Caption,1,pos(RemSearchStr,ProgLblAll.Caption)+10);
  TempStr2:= copy(ProgLblAll.Caption,pos(CompSearchStr,ProgLblAll.Caption),99);
  ProgLblAll.setCaption(TempStr1 + '00:00:00, ' + TempStr2);
  //set overall progress bar to 100%
  ProgBarAll.Position:= ProgBarAll.Max;
  //progbar label will be saying clone complete, change
  ProgBarLbl.setCaption(rCloneFinStr);
  application.processmessages; //otherwise updated labels will not appear
  //finished enable the buttons
  FinishBtn.Enabled:= true;
  AddLogBtn.Enabled:= true;
  case pfAction of
       acRestore : Self.Caption:= rRestFinishStr;
       acCloneF  : Self.Caption:= rCloneFfinStr;
       acClone   : Self.Caption:= rCloneFinStr;
  end;
  //turn swap back on
  SwapOn(true);
end; //end procedure TProgressFm.runClone;
{-------------------------------------------------------------------------------
PROCEDURE RUNFOXCLONE
This is the main procedure by which other procedures gain access to the progress form and run
partclone. It is not a method of TProgressFm - it is the procedure that creates the progress form.

It sets the backup/restore filename (or device name for a clone) and then runs the appropriate
method to do the backup/restore/clone.

It would be nice to use showmodal, but cannot. Execution would halt at showmodal until the form was
closed, hence backup, etc. could not run.

Parameters

    fAction : acBackup, etc. Defines the action to be taken which is then used via a case statement
    to run the appropriate procedure: backup, restore, etc.

    FileName : the filename for the backup or restore. In the case of a clone this will be the device
    in the form /dev/sdx.

Used by:

V1.0 - 11 Oct 2021, backup done.
V1.1 - 21 Nov 2021, restore done and pfAction set (new so that event handlers for buttons know what
                    action is being taken.
V1.2 - 03 Dec 2021, amended for acClone
V1.3 - 04 Dec 2021, amended for acVerify
V1.4 - 28 Jan 2022, was checking for gparted in TDrive.sfdisk, now moved here and at start so if running
                    foxclone stays at the main window.
-------------------------------------------------------------------------------}
procedure runFoxClone(fAction : TFoxType; FileName : string = '');
var
  ProgForm : TProgressFm;
begin
  if gpartedCheck then exit;
  ProgForm:= TProgressFm.CreateNew(MainFm);
  ProgForm.pfAction:= fAction; //set so available throughout unit
  MainFm.Enabled:= false; //it is enabled when the progress form is closed
  ProgForm.show;
  with ProgForm do case fAction of
       acBackup  :           begin
                               DeviceName:= FileName; //pass backup file name into progress form
                               runBackup;
                             end;
       acRestore, acCloneF : begin
                               DeviceName:= FileName; //pass backup file name into progress form
                               CancelBtn.Enabled:= false;
                               CancelBtn.visible:= false;
                               AddLogBtn.Caption:= rLogBtnStr;
                               runRestore;
                             end;
       acClone:              begin
                               CancelBtn.Enabled:= false;
                               CancelBtn.visible:= false;
                               AddLogBtn.Caption:= rLogBtnStr;
                               runClone;
                             end;
       acVerify            : begin
                               DeviceName:= FileName; //pass backup file name into progress form
                               AddLogBtn.Caption:= rLogBtnStr;
                               runVerify;
                             end;
  end;
end; //end procedure runFoxClone
{-------------------------------------------------------------------------------
PROCEDURE TPROGRESSFM.KILLPARTCLONE
Kills partclone and would be callled if an error is detected when running partclone. The procedure
uses getstr. When this detects an empty string on the output of running the command it repeats it
five times. This does not matter here.

Parameters
    FiieSystem : a string of the form ext4 or fat32 which is used to set what version of partclone is
    running. If the filesystem is unknown, e.g. for MS reserved partitions, then partclone.dd will be
    running.

Used by:

V1.0 - 03 Nov 2021
V1.1 - 05 Dec 2021, modified for acVerify to kill partclone.chkimg
-------------------------------------------------------------------------------}
procedure TProgressFm.KillPartClone(FileSystem : string);
const
  UnknownStr = 'unknown';
  CheckStr = 'chkimg';
  KillStr = 'pkexec killall partclone.';
begin
  if pfAction = acVerify then
     begin
       getstr(KillStr + CheckStr);
       exit;
     end;
  if FileSystem = UnknownStr then getstr(KillStr + 'dd') else getstr(KillStr + FileSystem);
end; //end procedure TProgressFm.KillPartClone
{-------------------------------------------------------------------------------
PROCEDURE TPROGRESSFM.CANCELBTNCK
Event handler for the cancel button. Because partclone will be running in the background, the
procedure sets the cancelflag which is then read by runPartClone. runPartClone then kills partclone.

partclone works on a 3 second loop (can be changed), so the UI will not immediately respond to clicking
on the button, hence providing a visual clue to the user by disabling the button.

Used by:

V1.0 - 07 Nov 2021
-------------------------------------------------------------------------------}
procedure TProgressFm.CancelBtnCK(Sender: TObject);
begin
  CancelFlag:= true;
  CancelBtn.Enabled:= false;
  application.processmessages;
end; //end procedure TProgressFm.CancelBtnCK
{-------------------------------------------------------------------------------
PROCEDURE TPROGRESSFM.FINISHBTNCK
Event handler for the finish button. It is not enabled until partclone has finished running.
Initially ran it with self.free, but this generated an error message to the terminal:

   WARNING: TButton.Destroy with LCLRefCount>0. Hint: Maybe the component is processing an event?

So changed to free and leave the parent object to free the progress form.

Used by:

V1.0 - 07 Nov 2021
-------------------------------------------------------------------------------}
procedure TProgressFm.FinishBtnCK(Sender : TObject);
begin
  Self.close; //leave it to the calling process to free up the progress form
end; //end procedure TProgressFm.FinishBtnCK
{-------------------------------------------------------------------------------
PROCEDURE TPROGRESSFM.ADDLOGBTNCK
Event handler for the add note button. It is not enabled until partclone has finished running.
Creates the note form and loads it with the note if it already exists.

To get the filename for the note passed, DeviceName is a 'global' variable in the TProgressFm
definition. This holds the backup file name. There is a similar global variable in the note form,
NoteFname.

Used by:

V1.0 - 10 Nov 2021
V1.1 - 20 Nov 2021, modified to show the restore log
V1.2 - 24 Nov 2021, modified to add acCloneF to the restore log code
V1.3 - 02 Dec 2021, modified to add acClone to the restore log code and case statement replaced by if.
V1.4 - 05 Dec 2021, modified for acVerify. Back to a case statement.
-------------------------------------------------------------------------------}
procedure TProgressFm.AddLogBtnCK(Sender: TObject);
const
  BackExtn = '.backup';
  NoteExtn = '.note.txt';
var
  BackNote: TNoteForm;
  LogList, LogFiles : TStringList;
  i, dIndex : shortint;
  TitleStr : string;
  BackFolder : string;
begin
  if pfAction = acRestore then TitleStr:= rRestTabStr;
  if pfAction = acCloneF then TitleStr:= rCloneFtitleStr;
  if pfAction = acClone then TitleStr:= rCloneTitleStr;
  if pfAction = acVerify then TitleStr:= rVerifyTabStr;
  Self.Enabled:= false; //disable the progress form
  BackNote:= TNoteForm.Create(Self);
  with BackNote do case pfAction of
       acBackup : begin
                    NoteFName:= replacestr(DeviceName,BackExtn,NoteExtn);
                    if FileExists(NoteFName) then BackMemo.Lines.LoadFromFile(NoteFName);
                    CancelBtn.Enabled:= true;
                    CancelBtn.Visible:= true;
                    OKBtn.Enabled:= true;
                    OKBtn.Visible:= true;
                    BackMemo.ReadOnly:= false;
                    ShowModal;
                    //change note button caption from add note to edit note
                    if FileExists(NoteFName) then AddLogBtn.Caption:= rEditNoteStr;
                  end;
       acVerify : begin
                    Caption:= 'Verify Log';
                    BackMemo.Lines.Add(TitleStr + Space + FormatDateTime('dd mmm yyyy, hh:nn',now));
                    BackMemo.Lines.Add(DeviceName);
                    BackMemo.Lines.Add(separator2);
                    //get the folder from the DeviceName
                    // /media/ultrabay/UEFI-win10-LM192-full/20191110.backup
                    BackFolder:= copy(DeviceName,1,rpos('/',DeviceName));
                    //writeln('verify back folder=',BackFolder);
                    //get the log files
                    LogFiles:= FindAllFiles(BackFolder, '*.chk', false); //false= no sub dirs
                    LogFiles.Sort;
                    LogList:= TStringList.Create;
                    for i:= 0 to LogFiles.Count - 1 do
                        begin
                          LogList.LoadFromFile(LogFiles[i]);
                          BackMemo.Lines.AddStrings(LogList);
                          BackMemo.Lines.Add(separator2);
                        end;
                    LogList.free;
                    LogFiles.free;
                    //setup the noteform
                    BackMemo.Font.Name:= 'Ubuntu Mono'; //seems to be too small so set size
                    BackMemo.Font.Size:= MainFm.Font.Size + 1; //dont think ubuntu good choice
                    OKBtn.Enabled:= true;
                    OKBtn.Visible:= true;
                    OKBtn.Caption:= rClose;
                    ShowModal;
                  end;
       else
         begin
           if pfAction = acRestore then Caption:= 'Restore Log';
           if pfAction = acCloneF then Caption:= 'Clone from File Log';
           if pfAction = acClone then Caption:= 'Clone Log';
           BackMemo.Lines.Add(TitleStr + Space + FormatDateTime('dd mmm yyyy, hh:nn',now));
           BackMemo.Lines.Add(separator2);
           BackMemo.Lines.Add('PARTITION TABLE');
           //find the target drive
           dIndex:= MyDrives.getDrive(acTarget);
           if dIndex = -1 then SystemError(12,'TProgresFm.AddLogBtnCK');
           LogList:= TStringList.Create;
           //get sfdisk
           getText('pkexec sfdisk --dump /dev/' + MyDrives[dIndex].MountPt,LogList);
           BackMemo.Lines.AddStrings(LogList);
           BackMemo.Lines.Add(separator2);
           //get the log files
           LogFiles:= FindAllFiles(DefaultFolder, '*.log', false); //false= no sub dirs
           LogFiles.Sort;
           for i:= 0 to LogFiles.Count - 1 do
               begin
                 LogList.LoadFromFile(LogFiles[i]);
                 BackMemo.Lines.AddStrings(LogList);
                 BackMemo.Lines.Add(separator2);
               end;
           LogList.free;
           LogFiles.free;
           //setup the noteform
           BackMemo.Font.Name:= 'Ubuntu Mono'; //seems to be too small so set size
           BackMemo.Font.Size:= MainFm.Font.Size + 1; //dont think ubuntu good choice
           OKBtn.Enabled:= true;
           OKBtn.Visible:= true;
           OKBtn.Caption:= rClose;
           ShowModal;
         end;
  end;
  BackNote.free;
  Self.Enabled:= true; //enable the progress form
end; //end procedure TProgressFm.AddLogBtnCK
{-------------------------------------------------------------------------------
PROCEDURE TPROGRESSFM.PROGFORMCLOSE
Event handler for terminate button in the title bar of the progress form.

Checks to see if partclone is running. If running the form cannot be closed. CloseAction must be set,
if not set then caFree seems to be the default option.

Clears the tabs before finish, without this, restore, etc. will give a system error - hintstr cannot
find drive or part as the system has been modified.

Enables the main form on completion.

Parameters:

    TCloseAction - caNone     = Do nothing
                   caHide     = The form is hidden
                   caFree     = The form is destroyed
                   caMinimize = The form is minimized

For some reason on clone drive to drive when partclone has run it is not killed and left as a zombie
process:
ps -A | grep partclone
  76322 pts/0    00:00:00 partclone.ntfs <defunct>
  76349 pts/0    00:00:39 partclone.ntfs <defunct>
  77463 pts/0    00:00:10 partclone.ext4 <defunct>
  77695 pts/0    00:00:00 partclone.ext4 <defunct>

Previous method of just testing output of ps -A | grep partclone does not work. Amended to check
for defunct processes.

Used by:

V1.0 - 11 Nov 2021
V1.1 - 22 Nov 2021, modified to reread the system on a restore
V1.2 - 24 Nov 2021, modified for clone from file also re-read system
V1.3 - 02 Dec 2021, see zombie processes
V1.4 - 04 Dec 2021, don't need to reread the system on verify
V1.5 - 06 Dec 2021, clearall on verify tab added
V1.6 - 26 Jan 2022, umountall added to umount all partitions mounted by foxclone
-------------------------------------------------------------------------------}
procedure TProgressFm.ProgFormClose(Sender: TObject; var CloseAction: TCloseAction);
const
  zombie = 'defunct';
var
  StartupFm : TStartupFm;
  PartCloneList : TStringList;
  SafeToClose : boolean = true; //default assume okay
  i : byte;
begin
  PartCloneList:= TStringList.Create;
  getText('ps -A | grep partclone',PartCloneList); //is partclone running?
  //check for zombie processes
  if PartCloneList.Count > 0 then //not okay, need to check for any process not a zombie
     begin
       for i:= 0 to PartCloneList.Count - 1 do if pos(zombie,PartCloneList[i]) = 0 then
           begin
             SafeToClose:= false;
             break;
           end;
     end;
  if SafeToClose then
     begin
       UmountAll; //unmount any partitions mounted by foxclone
       if not ((pfAction = acBackup) or (pfAction = acVerify)) then //need to reread the system
          begin
            Self.Visible:= false;
            StartupFm:= TStartupFm.create(Self);
            StartupFm.Show;
            StartupFm.ReadSystem;
            StartupFm.Close;
          end;
       CloseAction:= caFree;
       BackTab.ClearAll;
       RestoreTab.ClearAll; //reset the restore tab and clear all boxes
       CloneTab.ClearAll;
       VerifyTab.ClearAll;
       MainFm.Enabled:= true;
    end
  else //cannot close if partclone running
    begin
      FoxMsg(rCloseStr,[rPartCloneRunStr,PartCloneList[i]],mtInformation,[mbOK]);
      CloseAction:= caNone;
    end;
  PartCloneList.free;
end; //end procedure TProgressFm.ProgFormClose
{-------------------------------------------------------------------------------
PROCEDURE TPROGRESSFM.UPDATELOG
A simple procedure to update the log created when a partition is verified. All the method does is load
the log file and insert a header with the date/time the verification was done.

Parameters:
    PartRef - a string in the form sda1, or nvme0n1p1, i.e. including the part prefix. Used to build
    the logfile name from the backup file name, logfile is of the form 20191110.sda1.img.chk

Used by:

V1.0 - 05 Dec 2021
-------------------------------------------------------------------------------}
procedure TProgressFm.UpdateLog(PartRef : string);
const
  dotStr = '.';
  BackExtn = '.backup';
  LogExtn = '.img.chk';
var
  LogFname : string;
  LogList : TStringList;
begin
  LogFname:= replacestr(DeviceName,BackExtn,dotStr + PartRef + LogExtn);
  //writeln('update log, backup file=',DeviceName);
  //writeln('update log, log file=',LogFname);
  if FileExists(LogFname) then
     begin
       //writeln('log file exists...');
       LogList:= TStringList.Create;
       LogList.LoadFromFile(LogFname);
       LogList.Insert(0,''); //blank line before the partclone output
       LogList.Insert(0,PartRef + rVerifyLogStr + FormatDateTime('dd mmm yyyy, hh:nn',now));
       LogList.SaveToFile(LogFname);
       LogList.free;
     end;
end; //end procedure TProgressFm.UpdateLog
{-------------------------------------------------------------------------------
PROCEDURE TPROGRESSFM.UMOUNTALL
Foxclone leaves the system as it finds it, so any partitions mounted by foxclone are unmounted when
finished with. Previously this was being done by the individual methods, e.g. for backup, but this
caused problems - partition was being umounted before creating a note = error.

This procedure runs df to find what is mounted and if mounted by foxclone (it will be mounted in
MountFolder = /tmp/foxclone) it umounts it.

Used by: TProgressFm.ProgFormClose - when the progress form is closed

V1.0 - 26 Jan 2022
-------------------------------------------------------------------------------}
procedure TProgressFm.UmountAll;
const
  devStr = '/dev/';
var
  i : byte;
  dIndex, pIndex : shortint;
  dfList : TStringList;
  MountStr : string;
begin
  dfList:= TStringList.Create;
  GetText('df | grep ^' + devStr,dfList);
  if dfList.Count > 0 then
     for i:= 0 to dfList.Count - 1 do
         if pos(MountFolder,dfList[i]) > 0 then
            begin
              MountStr:= replacestr(dfList[i],devStr,'');
              dIndex:= MyDrives.FindName(MountStr); //find the drive
              if dIndex = -1 then break; //not found
              pIndex:= MyDrives[dindex].Partitions.FindName(MountStr);
              if pIndex = -1 then break; //not found
              MyDrives[dindex].Partitions[pIndex].UMount;
            end;
  dfList.free;
end; //end procedure TProgressFm.UmountAll

end.

