{ FoxClone
  Copyright (C) 2021 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}
{-------------------------------------------------------------------------------
UNIT - RESTORE

The objects and methods for restore. The restore panel is a child of basepanel and requires some
additional objects for the restore tab - some labels and a button, defines the methods required for
restore.

V1.0 - 10 Dec 2021
-------------------------------------------------------------------------------}
unit restore;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, CheckLst, StdCtrls, Forms, Controls, dialogs, strutils, DateUtils, Graphics, basepanel,
  GTKlabel, noteform, drives, partitions, settings, utilities, il8nstrings, fdialog, confform;

type
  TRestore = class(TBasePanel)
    protected
      Box3Toggle : boolean; //used so that msgbox only appears once when clicking on part checkboxes
      procedure Box1CK(Sender: TObject);
      procedure PopulateBox2;
      procedure Box2CK(Sender: TObject);
      function GetFile(dIndex, pIndex : shortint) : boolean;
      function ReadBackupFile : boolean;
      procedure ShowDateAndNote(ShowDate : boolean);
      procedure LBoxBold_OS(Control: TWinControl; Index: Integer; ARect: TRect; State: TOwnerDrawState);
      procedure PopulateBox3;
      procedure PopulateBox4;
      procedure Box3CK(Sender: TObject);
      procedure NoteCK(Sender: TObject);
      procedure NextBtnCK(Sender: TObject);
      function CheckPartTable : boolean;
      procedure Box3CheckCK(Sender: TObject);
    public
      RDateLbl, Rdate, RNowLbl, RNow : TLabelGTK; //these were protected, made public for basepanel font
      NoteBtn : TButton;                          //resize
      constructor Create(TheOwner: TComponent);
      procedure ClearAll;
      procedure ReSizeFilename; //resizes box3lbl to fit the width of the box
      function RestoreSelect(Item : byte) : boolean; //returns the checked state of an item in box3
  end;

var
  RestoreTab : TRestore;
  RestFname : string; //restore file name
  BackDate : string;

implementation
uses
  mainform, backup, clone, verify, partclone; //to avoid circular errors, declared here
{-------------------------------------------------------------------------------
CONSTRUCTOR TRESTORE.CREATE
Creates the restore panel and adds in the properties, functions & procedures specific to clone.

restoretab is an amended copy of basepanel.

Used by:

V1.0 - 01 May 2021
V1.1 - 28 Jul 2021 - InfoImg2 free removed
V1.2 - 25 Aug 2021 - note button height made dependent on text height so that consistency when
                     font resized
-------------------------------------------------------------------------------}
constructor TRestore.Create(TheOwner: TComponent);
const
  delim = ':';
begin
  inherited Create(TheOwner);
  Box1.BoxType:= reBox1;
  Box2.BoxType:= reBox2;
  Box3.BoxType:= reBox3; //no box4 its a listbox
  TitleLbl.setCaption(rRestTitleStr);
  ActionImg.Picture.LoadFromFile(RestoreImgFile);
  //box 1 - drive information
  Box1Lbl.setCaption(rRestBox1Str);
  //box 2 - partition information - lbOwnerDrawFixed is set in basepanel
  Box2Lbl.setCaption(rRestBox2Str);
  //box 3 - partition information - populated from RDrive not MyDrives, hence needs different ondrawitem
  Box3Toggle:= false;
  Box3Lbl.setCaption(rRestBox3Str);
  Box3.OnMouseMove:= @PartHint; //to get custom hints, basepanel is DriveHint
  Box3.Style:= lbOwnerDrawFixed;//lbOwnerDrawFixed; what is the difference - the item height
  Box3.OnDrawItem:= @BoldOS; //specific to box3 and RDrive
  //box 4 - partition information in a listbox not a checklistbox
  Box4Lbl.setCaption(rRestBox4Str);
  NextBtn.Caption:= rRestTabStr;
  freeandnil(Box4);
  LBox4:= TFoxListBox.Create(Self); //create a list box
  with LBox4 do begin
       BoxType:= reBox4;
       Parent:= self;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= Box4Lbl;
       BorderSpacing.top:= phBorder;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Box2;
       BorderSpacing.left:= 0;
       ItemHeight:= BoxItemHeight;
       Width:= Box1.Width;
       Height:= Box1.Height;
       Style:= lbOwnerDrawFixed;
       OnDrawItem:= @LBoxBold_OS; //specific to box4 as a list box
       ExtendedSelect:= false; //stops shift plus click
       MultiSelect:= false;//stops ctrl plus click
       ShowHint:= true;
       OnMouseMove:= @LBoxHint;
       Enabled:= false;
  end;
  NextBtn.AnchorSide[akTop].Control:= LBox4;
  //Backup date label
  RDateLbl:= TLabelGTK.Create(Self);
  with RDateLbl do begin
    Parent:= self;
    AnchorSide[akTop].Side:= asrBottom;
    AnchorSide[akTop].Control:= Box3;
    BorderSpacing.top:= phBorder;
    AnchorSide[akLeft].Side:= asrLeft;
    AnchorSide[akLeft].Control:= Box1;
    BorderSpacing.left:= 0;
    Enabled:= false;
    Visible:= false;
    setCaption(rRestDateStr + delim);
  end;
  //Time now date label
  RNowLbl:= TLabelGTK.Create(Self);
  with RNowLbl do begin
    Parent:= self;
    AnchorSide[akTop].Side:= asrBottom;
    AnchorSide[akTop].Control:= RDateLbl;
    BorderSpacing.top:= phBorder div 2;
    AnchorSide[akLeft].Side:= asrLeft;
    AnchorSide[akLeft].Control:= RDateLbl;
    BorderSpacing.left:= 0;
    Enabled:= false;
    Visible:= false;
    setCaption(rTimeNowStr + delim);
  end;
  //set the date and date now labels the same width
  if RDateLbl.Width > RNowLbl.Width then RNowLbl.Width:= RDateLbl.Width
  else RDateLbl.Width:= RNowLbl.Width;
  //Date of backup taken from the backup file
  RDate:= TLabelGTK.Create(Self);
  with RDate do begin
    Parent:= self;
    AnchorSide[akTop].Side:= asrTop;
    AnchorSide[akTop].Control:= RDateLbl;
    BorderSpacing.top:= 0;
    AnchorSide[akLeft].Side:= asrRight;
    AnchorSide[akLeft].Control:= RDateLbl;
    BorderSpacing.left:= pwborder;
    Enabled:= false;
    Visible:= false;
  end;
  //Current date/time
  RNow:= TLabelGTK.Create(Self);
  with RNow do begin
    Parent:= self;
    AnchorSide[akTop].Side:= asrTop;
    AnchorSide[akTop].Control:= RNowLbl;
    BorderSpacing.top:= 0;
    AnchorSide[akLeft].Side:= asrLeft;
    AnchorSide[akLeft].Control:= RDate;
    BorderSpacing.left:= 0;
    Enabled:= false;
    Visible:= false;
  end;
  //Note button
  NoteBtn:= TButton.Create(Self);
  with NoteBtn do begin
       Parent:= self;
       Caption:= rNoteBtnStr;
       Height:= GetTextHeight(rNoteBtnStr,font) + 14;
       Width:= GetTextWidth(rNoteBtnStr,font) + 20; //manual adjust to get about same as quit button
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= Box3;
       BorderSpacing.top:= phBorder * 2; //for consistency with the next button
       AnchorSide[akLeft].Side:= asrRight;
       AnchorSide[akLeft].Control:= Rdate;
       BorderSpacing.left:= 3 * pwborder;
       Enabled:= false;
       Visible:= false;
  end;
  //Existing label - appears below box4
  RExisting:= TLabelGTK.Create(Self);
  with RExisting do begin
    Parent:= self;
    AnchorSide[akTop].Side:= asrBottom;
    AnchorSide[akTop].Control:= LBox4;
    BorderSpacing.top:= 0;
    AnchorSide[akLeft].Side:= asrLeft;
    AnchorSide[akLeft].Control:= LBox4;
    BorderSpacing.left:= 0;
    Enabled:= false;
    Visible:= false;
    setCaption(rExistStr);
  end;
  //Event handlers
  Box1.OnClick:= @Box1CK;
  Box2.OnClick:= @Box2CK;
  Box3.OnClick:= @Box3CK;
  Box3.OnClickCheck:= @Box3CheckCK; //fix for bug in lazarus, not firing onclick on second checkbox click
  NoteBtn.OnClick:= @NoteCK;
  NextBtn.OnClick:= @NextBtnCK;
end; //end constructor TRestore.Create
{-------------------------------------------------------------------------------
PROCEDURE TRESTORE.BOX1CK
Event handler for clicks on box1.

Modified version of TBack.Box1CK. Only modification is changing clearBox(4) to clearBox(40) as
box4 is a ListBox in restore.

Only a single drive may be selected.

On a valid click that selects a drive:
   sets that drive as the source for the backup file
   populates box2 with the partitions on the drive

If the user clicks on blank space - nothing happens.

Note -  clicking on either the check box or the text triggers this procedure, but if clicking on text it
does not change the state of the checkbox, clicking on the checkbox does change its state. Went round
in circles a few times trying to sort out the logic of this. The initial code on saving the checkbox
state and then resetting it works, but not altogether sure how.

Used by:

V1.0 - 29 Mar 2021
V1.1 - 28 May 2021, modified to use checkstate.
V1.2 - 20 Jun 2021, modified to clearall on clone tab.
-------------------------------------------------------------------------------}
procedure TRestore.Box1CK(Sender: TObject);
var
  Source : shortint;
  dindex : shortint;
begin
  //clear all other tabs
  BackTab.ClearAll;
  CloneTab.ClearAll;
  VerifyTab.ClearAll;
  MyDrives.unSetAll(acNone);
  Source:= Box1.ItemIndex;
  if Source < 0 then exit;
  DebugInfo(true,'TRestore.Box1CK',Box1.Items[Source]);
  clearBox(2);  //clear all the other boxes
  clearBox(3);
  clearBox(40); //40 = LBox4
  ShowDateAndNote(false); //make backup date and note button invisible
  NextBtn.Enabled:= false; //reset the save to file button
  Box1.CheckAll(cbUnchecked); //only one can be selected
  Box2Lbl.Enabled:= true;
  Box1.Checked[Source]:= not Box1.CheckState[Source];
  Box1.CheckState[Source]:= Box1.Checked[Source];
  if Box1.Checked[Source] then
     begin
       dindex:= MyDrives.FindName(Box1.Items[Source]);
       if dindex = -1 then SystemError(12,Box1.Items[Source]); //close foxclone on error
       MyDrives[dindex].setAction:= acSource; //set as the drive containing the backup
       PopulateBox2; //populate box2 with partitions in MyDrives[dindex]
     end;
end; //end procedure TRestore.Box1CK
{-------------------------------------------------------------------------------
PROCEDURE TRESTORE.POPULATEBOX2
Populates box2 with the partitions on the drive selected in box1.

Used by:

V1.0 - 29 Mar 2021
V1.1 - 28 May 2021, modified to setup checkstate
V1.2 - 29 May 2021, no point showing EFI, extended or swap parts - they cannot contain a backup
-------------------------------------------------------------------------------}
procedure TRestore.PopulateBox2;
var
  i, dindex : byte;
begin
  dindex:= MyDrives.getDrive(acSource); //get the drive selected containing the backup
  Box2.enabled:= true;
  for i:= 0 to MyDrives[dindex].TotalParts - 1 do
      with MyDrives[dindex].partitions[i] do if UserSelect then Box2.Items.add(ListStr);
  //setup CheckState
  setLength(Box2.CheckState,Box2.Count);
  for i:= 0 to Box2.Count - 1 do Box2.CheckState[i]:= Box2.Checked[i];
  setScrollBarTBox(Box2);
end; //end procedure TRestore.PopulateBox2
{-------------------------------------------------------------------------------
PROCEDURE TRESTORE.POPULATEBOX3
Populates box3 with the partitions found in the backup file, where the contents of the backup file
have been parsed into RDrive.

Note RDrive contains all partitions (it reads sfdisk), so when populating box3 it checks to see if
there is a backup for the partition (selected = true) and sets the checkbox accordingly.

Used by:

V1.0 - 30 Apr 2021
V1.1 - 17 May 2021, modified to set the BoxCheck array
V1.2 - 28 May 2021, boxcheck renamed CheckState
V1.3 - 20 Jun 2021, modified to work with verify
V1.4 - 14 Sep 2021, modded to use chopstr to truncate the backup filename
V1.5 - 25 Sep 2021, use of chopstr moved into a separate procedure ReSizeFilename.
-------------------------------------------------------------------------------}
procedure TRestore.PopulateBox3;
var
  i : byte;
begin
  if Box3.BoxType = reBox3 then with Box3 do
     begin //only applicable to restore, don't show part table on verify
       Items.add(rRestPTable); //will always be item[0]
       Checked[0]:= true;
     end;
  for i:= 0 to RDrive.TotalParts - 1 do
      begin
        Box3.Items.add(RDrive.Partitions[i].ListStr);
        if (Box3.BoxType = reBox3) and (RDrive.Partitions[i].selected) then Box3.Checked[i+1]:= true;
      end;
  //initialise the CheckState array and set values as the CheckList box
  setLength(Box3.CheckState,Box3.Count);
  for i:= 0 to Box3.Count - 1 do Box3.CheckState[i]:= Box3.Checked[i];
  //setScrollBars;
  setScrollBarTBox(Box3);
  Box3.Enabled:= true;
  Box3Lbl.Enabled:= true;
  ReSizeFilename;
end; //end procedure TRestore.PopulateBox3
{-------------------------------------------------------------------------------
PROCEDURE TRESTORE.POPULATEBOX4
Populates box4 with the partitions found in the backup file and makes the label below Box4 visible.

Also adds the drive name and drive size to box 4 caption.

Used by:

V1.0 - 30 Apr 2021
V1.1 - 13 Oct 2021, const space moved to utilities
V1.2 - 27 Oct 2021, blkadjust removed, drivesize is in 1k blocks.
V1.3 - 02 Jan 2022, failing on restore to a blank drive, not checking the number of parts, index out
                    of bounds error. Fixed.
-------------------------------------------------------------------------------}
procedure TRestore.PopulateBox4;
const
  Lbkt = ' (';
  Rbkt = ')';
var
  i : byte;
  dIndex : shortint;
begin
  LBox4.clear;
  LBox4.enabled:= true;
  //find the drive = acTarget
  dIndex:= MyDrives.getDrive(acTarget); //find the drive that will be restored
  if dIndex = -1 then SystemError(18,'TRestore.PopulateBox4'); //close foxclone on error
  //check that there are parts on the destination before adding to box4
  if MyDrives[dIndex].TotalParts > 0 then
     for i:= 0 to MyDrives[dIndex].Partitions.count - 1 do
         LBox4.Items.add(MyDrives[dIndex].Partitions[i].ListStr);
  setScrollBarLBox(LBox4);
  RExisting.Enabled:= true; //the label saying contents of the destination
  RExisting.Visible:= true;
  Box4Lbl.Enabled:= true;
  with MyDrives[dIndex] do with Box4Lbl do
       setCaption(rRestBox4Str + Space + Model + Lbkt + SizeStr(DriveSize) + Rbkt);
end; //end procedure TRestore.PopulateBox4
{-------------------------------------------------------------------------------
PROCEDURE TRESTORE.BOX2CK
Event handler for clicks on box2.

Does a lot.

Only a single partition may be selected.

Selecting a valid partition opens up the file open dialog to get the backup file.

Opens the backup file and parses it (and the associated sfdisk file).
Populates box3 with the contents of the backup file.
Searches MyDrives for the drive using the model name (and serial number if present) in the backup file.
Sets the drive found in MyDrives as acTarget
Populates box4 (which is a list box not checklist box) with the contents of the target drive.

Used by:

V1.0 - 29 Mar 2021
V1.1 - 05 May 2021  check added to see if the target drive has moved, e.g. sda to sdb
V1.2 - 11 May 2021, FoxDialog replaced with FoxMsg
V1.3 - 28 May 2021, modified to use checkstate
V1.4 - 20 Jun 2021, modified to work with verify, it should be able to make it work with clone from
                    file as well
V1.5 - 08 Sep 2021, nextbtn enabled.
V1.6 - 12 Nov 2021, sets selected=false for all parts on the source drive and then sets selected=true
                    for the part containing the backup. This so that the part can be later unmounted
                    after the restore if foxclone mounted it.
V1.7 - 04 Dec 2021, modification so that the nextbtn is only enabled on restore not verify
V1.8 - 24 Jan 2022, on verify if the source drive has moved, e.g. sda to sdb it raises the warning
                    message that the drive to restore to has changed - not wanted. Cancelling generates
                    a sigsegv. If statement on checking mount point changed to check box type as well.
V1.9 - 26 Jan 2022, Was mounting an unknown filesystem, modified so that it does nothing.
-------------------------------------------------------------------------------}
procedure TRestore.Box2CK(Sender: TObject);
const
  OldMount = '[oldmount]';
  NewMount = '[newmount]';
  RestDrive = '[drive]';
  UnknownStr = 'unknown';
var
  Source : shortint;
  dIndex, pIndex : shortint;
  Msg1, Msg2 : string; //used in the message if the mount point has changed
  i : byte;
begin
  ClearBox(3);
  if assigned(LBox4) then ClearBox(40); //40 = tlist box where 4 = tchecklistbox, if needed for verify
  NextBtn.Enabled:= false; //reset
  ShowDateAndNote(false); //make backup date and note button invisible
  Source:= Box2.ItemIndex;
  if Source < 0 then exit;
  DebugInfo(true,'TRestore.Box2CK',Box2.Items[Source]);
  Box2.CheckAll(cbUnchecked); //only one can be selected
  Box2.Checked[Source]:= not Box2.CheckState[Source];
  Box2.CheckState[Source]:= Box2.Checked[Source];
  if Box2.Checked[Source] then
     begin
       dIndex:= MyDrives.getDrive(acSource); //find the drive containing the backup
       if dIndex = -1 then SystemError(14,'TRestore.Box2CK'); //close foxclone on error
       //get the partition
       pindex:= MyDrives[dindex].Partitions.FindName(Box2.Items[Source]);
       if pindex = -1 then SystemError(15,Box2.Items[Source]);
       //check the filesystem is not unknown, if it is do nothing
       if MyDrives[dIndex].Partitions[pIndex].FileSystem = UnknownStr then
         begin
           Box2.Checked[Source]:= false;
           Box2.CheckState[Source]:= false;
           exit;
         end;
       //check the partition is not extended, swap or EFI
       if not MyDrives[dIndex].Partitions[pIndex].UserSelect then
         begin
           Box2.Checked[Source]:= false;
           Box2.CheckState[Source]:= false;
           exit;
         end;
       //get and check for restore filename, populates RestFname
       if not GetFile(dIndex,pIndex) then
          begin
            Box2.Checked[Source]:= false;
            Box2.CheckState[Source]:= false;
            exit;
          end;
       //parse the backup file into RDrive
       if not ReadBackupFile then
          begin
            Box2.Checked[Source]:= false;
            Box2.CheckState[Source]:= false;
            exit;
          end;
       //having checked and got the backup, set the partition selected so that restore can find it later
       //this is so that if foxclone mounted the source partition it can be unmounted.
       for i:= 0 to MyDrives[dIndex].TotalParts - 1 do MyDrives[dIndex].Partitions[i].selected:= false;
       MyDrives[dIndex].Partitions[pIndex].selected:= true;
       //find the drive to be restored to
       dIndex:= MyDrives.FindModel(RDrive.Model,RDrive.Serial);
       if dIndex = -1 then
          begin
            Box2.Checked[Source]:= false;
            Box2.CheckState[Source]:= false;
            exit;
          end;
       MyDrives[dIndex].setAction:= acTarget;
       PopulateBox3;
       ShowDateAndNote(true);
       //modified to work with verify
       if Box3.BoxType = reBox3 then PopulateBox4; //restore only
       if Box3.BoxType = veBox3 then Box4Lbl.Visible:= true; //verify only using it for something else
       //at this point the user can see the contents of the backup and what is on the destination
       //now check if the mountpoint has changed, e.g. from sda to sdd
       //if it has changed, then this is checked and changed again when the restore is done
       //this only applies on restore not on verify - added to if statement.
       if (Box3.BoxType = reBox3) and (RDrive.MountPt <> MyDrives[dIndex].MountPt) then
          begin
            Msg1:= replacestr(rChangeMtStr,OldMount,RDrive.MountPt);
            Msg1:= replacestr(Msg1,NewMount,MyDrives[dIndex].MountPt);
            Msg1:= replacestr(Msg1,RestDrive,RDrive.Model);
            Msg2:= replacestr(rChangeMtStr2,NewMount,MyDrives[dIndex].MountPt);
            if FoxMsg(rChangeMtTitleStr,[Msg1,Msg2],mtConfirmation,[mbYes,mbNo]) = mrNo then
               begin
                 //clear boxes 3 and 4, remove backup dates & note, reset mydrives, remove rdrive
                 ClearBox(3);
                 ClearBox(40);
                 ShowDateAndNote(false);
                 //RDrive is created by readbackupfile, when it is created it checks to see if it
                 //exists and if it does, frees it. So no need to do anything here
                 //setAction is set acTarget a few lines up.
                 for i:= 0 to MyDrives.Count - 1 do if MyDrives[i].setAction = acTarget then
                     MyDrives[i].setAction:= acNone;
               end
            else NextBtn.Enabled:= true; //finally enable the restore button
          end
       else if Box3.BoxType = reBox3 then NextBtn.Enabled:= true;
       //finally enable the restore button, but don't want it enabled for a verify
     end;
end; //end TRestore.Box2CK
{-------------------------------------------------------------------------------
FUNCTION TRESTORE.GETFILE
Populates RestFname with the filename for the backupfile. The function returns true if a valid filename
has been selected, false otherwise.

The function checks to see if the user has tried to select another device from the places pane in the
open file dialog box. If this is the case displays an error message.
          procedure TRestore.ReadBackupFile;

Parameters:
    dIndex - index of the drive selected as containing the backup file
    pIndex - index of the partition selected as containing the backup file

Note - did try disabling the mainform, initially this left box2 with the custom draw for OS = bold
unchanged. Added in style= lbstandard and then setting it back to lbownerdraw when finished. This
resulted in a sigsegv error, so abandoned.

Used by:

V1.0 - 08 Apr 2021
V1.1 - 10 Apr 2021, modified to add RestFname to Box3 title
V1.2 - 13 Oct 2021, const space moved to utilities
V1.3 - 24 Jan 2022, for a restore getfile mounts the partition with the backup file as read-only as it
                    only needs to read it. But getfile is also used in verify where the verify log
                    is written to the same folder as the backup files = error as read-only. Changed
                    mount to read/write.
V1.4 - 19 Feb 2022, debug info added
-------------------------------------------------------------------------------}
function TRestore.GetFile(dIndex, pIndex : shortint) : boolean;
const
  DialogFilterStr = 'Backup files (*.backup)|*.backup';
var
  RestMount : string;
  OpenDialog : TOpenDialog;
begin
  DebugInfo(false,'TRestore.GetFile');
  MainFm.Enabled:= false;
  Box2.Enabled:= false; //have to explicitly disable box2 for the ondraw event to recognise it
  GetFile:= false;
  RestFname:= '';
  //mount the partition if needed and get the mount point
  //mount read/write as write is needed for a verify
  RestMount:= MyDrives[dIndex].Partitions[pIndex].Mount(true);
  //now need to get the open file dialog
  OpenDialog:= TOpenDialog.Create(RestoreTab);
  OpenDialog.Title:= rRestFileTitleStr;
  OpenDialog.InitialDir:= RestMount;
  OpenDialog.Filter := DialogFilterStr;
  if OpenDialog.Execute then
     begin
       RestFname:= OpenDialog.Filename;
       //now check if the file selected is on the partition that has been mounted
       if pos(RestMount,RestFname) = 1 then GetFile:= true
       else FoxMsg(rWrongPartTitle,[rWrongPartMsg1,RestFname,rWrongPartMsg2 + space + RestMount,
                   rWrongPartMsg3],mtInformation,[mbOk]);
     end;
  OpenDialog.free;
  MainFm.Enabled:= true;
  Box2.Enabled:= true;
end; //end function TRestore.GetFile
{-------------------------------------------------------------------------------
FUNCTION TRESTORE.READBACKUPFILE
Reads the backup file and the sfdisk file, parses the contents in RDrive and the date into BackDate.

The function checks for the existence of the backup file and the sfdisk file. It returns false if
either are not found, true otherwise.

It creates RDrive and uses TDrive.ReadsfDisk for drive info and TPart.ParseBackFile & TPart.ParseSFdisk
to parse the partition information in the two files.

Information is parsed as follows:

    TDrive info:
    Device     - e.g. sda populated from sfdisk with TDrive.ReadsfDisk
    Selected   - irrelevant for RDrive
    dBoot      - irrelevant for RDrive
    dModel     - from the backup file
    dLogical   - not populated, maybe for the future?
    dPhysical  - not populated, maybe for the future?
    dFlags     - not populated, maybe for the future? , yet see it populated in any parted output
    pTableType - msdos or gpt, populated from sfdisk with TDrive.ReadsfDisk
    Serial     - from the backup file, not present in backups from some earlier foxclone versions
    PartPrefix - from sfdisk using TDrive.ReadsfDisk
    FirstPart  - from sfdisk using TDrive.ReadsfDisk
    Size       - not available in backup info, add for the future?
    sfLabel    - this is the label-id in sfdisk looks like a UUID, not used in foxclone at the moment
    sfFirstLBA - gpt only, populated from sfdisk, with TDrive.ReadsfDisk
    sfLastLBA  - gpt only, populated from sfdisk, with TDrive.ReadsfDisk
    BlkAdjust  - set from sfdisk checking for unit: sectors, TDrive.ReadsfDisk
    PartCount  - # of TParts calculated from sfdisk
    Parts      - created and then the # of TParts added from sfdisk, TDrive.ReadsfDisk

Used by:

V1.0 - 08 Apr 2021
V1.1 - 10 Apr 2021, modified to add RestFname to Box3 title
V1.2 - 11 May 2021, FoxDialog replaced by FoxMsg
V1.3 - 28 Oct 2021, changes from deleting blkadjust, procedure now sets the sector size for the restore.
                    From V50 on, sector size is defined in the backup file, before that search for the
                    drive and set the sector size from that, if the drive cannot be found then
                    default to a sector size of 512 bytes.
                    Changed to populate RSettings, had not been done.
V1.4 - 19 Feb 2022, debug info added and bugfix, on clone from file where no sector size in the backup
                    file was searching for the drive and not finding it = error message. Fixed.
-------------------------------------------------------------------------------}
function TRestore.ReadBackupFile : boolean;
const //search strings
  CompStr = 'Compression:';
  SplitStr = 'Split files:';
  ModStr = 'Model:';
  SNstr = 'Serial:';
  MountStr = 'Mount point:';
  PartStr = 'Partition:';
  SectorStr = 'Sector size:';
var
  RestData : TStringList;
  sfFname : string;
  i : byte;
  TempStr : string;
  Compress, Split : boolean;
  pIndex : shortint;
  SectorFlag : boolean = false; //used to check if the sector size is parsed out of the backup file
                                //if not then sector size determined from the current drive or default
  dIndex : shortint; //to find the drive for sector size
begin
  DebugInfo(false,'TRestore.ReadBackupFile',RestFname);
  ReadBackupFile:= true;
  //check files exist - backup file
  if not FileExists(RestFname) then
     begin
       FoxMsg(rFileNotFoundStr,[rFileNotFoundStr,RestFname],mtError,[mbOk]);
       ReadBackupFile:= false;
       exit;
     end;
  //check files exist - sfdisk file
  sfFname:= replacestr(RestFname,'.backup','.sfdisk');
  if not FileExists(sfFname) then
     begin
       FoxMsg(rFileNotFoundStr,[rFileNotFoundStr,sfFname],mtError,[mbOk]);
       ReadBackupFile:= false;
       exit;
     end;
  //tidy up
  if assigned(RDrive) then FreeAndNil(RDrive);
  RDrive:= TDrive.Create;
  RDrive.Partitions:= TPartList.create;
  //originally this was after ReadsfDisk, but need to get the sector size before calling ReadsfDisk
  //so load the backup file and parse out the sector size first, bit of a kludge
  RestData:= TStringList.Create;
  RestData.LoadFromFile(RestFname);
  for i:= 1 to RestData.Count - 1 do with RDrive do //addition for V50 - add in the sector size
      begin
        //model was later, needed to find drive if sector size not in backup file
        if pos(ModStr,RestData[i]) = 1 then Model:= trim(replacestr(RestData[i],ModStr,''));
        if pos(SectorStr,RestData[i]) = 1 then
           begin
             DriveSector:= StrToInt(trim(replacestr(RestData[i],SectorStr,'')));
             SectorFlag:= true;
           end;
      end;
  //now test sectorflag, if not set then try and find the drive in the current system
  //on a clone from file the drive will not be in the system, so just set sector size = 512,
  //bug fix as clone from file will generate an error message when FindModel can't find the drive.
  if not SectorFlag then
     begin
       dIndex:= -1;
       RDrive.DriveSector:= 512; //default
       //don't look for the drive if clone from file
       if Box2.BoxType <> clBox2File then dIndex:= MyDrives.FindModel(RDrive.Model,'');
       if dIndex > -1 then RDrive.DriveSector:= MyDrives[dIndex].DriveSector;
     end;
  //load the sfdisk file and read the info
  RDrive.ReadsfDisk(sfFname);
  //get remaining info from the backup file
  //first entry is the date of the backup - set up the labels for backup date and time now
  BackDate:= RestData[0];
  //now read the rest of the backup file
  for i:= 1 to RestData.Count - 1 do with RDrive do
      begin
        if pos(CompStr,RestData[i]) = 1 then Compress:= pos('YES',RestData[i]) > 0;
        if pos(SplitStr,RestData[i]) = 1 then Split:= pos('YES',RestData[i]) > 0;
        if pos(SNstr,RestData[i]) = 1 then Serial:= trim(replacestr(RestData[i],SNstr,''));
        //mount point is also parsed out of the sfdisk file, this overwrites
        if pos(MountStr,RestData[i]) = 1 then MountPt:= trim(replacestr(RestData[i],MountStr,''));
        if pos(PartStr,RestData[i]) = 1 then
           begin
             TempStr:= replacestr(RestData[i],PartStr,''); //get rid of 'Partition:' at the start
             pIndex:= Partitions.FindName(TempStr); ////have to find the partition in restdata[i]
             if pIndex > -1 then Partitions[pIndex].ParseBackFile(RDrive.pTable,RestData[i],RestData[i+1])
             else begin SystemError(17,'TRestore.ReadBackupFile',RestData[i]); break; end;
             //funny one here, systemerror would not close mainfm without finishing for loop hence break
           end;
      end;
  RestData.free;
  //populate RSettings
  //seNone = no compression, no split
  //seComp = compression, no split = default
  //seSplit = no compression, split
  //seBoth  = compression and split
  if Compress then
     if Split then RSettings:= seBoth else RSettings:= seComp
  else
     if Split then RSettings:= seSplit else RSettings:= seNone;
end; //end function TRestore.ReadBackupFile
{-------------------------------------------------------------------------------
FUNCTION TRESTORE.SHOWDATEANDNOTE
Switches on and off the display of the date/time of the backup and current date/time. Also displays the
note button if a note file is present in the backup.

Parameters:
    isVisibile - when true enables the labels and note button, when false disables then and sets
    visible = false

Used by:

V1.0 - 15 Apr 2021
-------------------------------------------------------------------------------}
procedure TRestore.ShowDateAndNote(ShowDate : boolean);
const
  DateTimeStr = 'dd mmm yyyy, hh:nn'; //format string for time now
begin
  if ShowDate then
     begin
       //make the date labels visible
       RDateLbl.Enabled:= true;
       RDateLbl.Visible:= true;
       RDate.setCaption(BackDate);
       RDate.Enabled:= true;
       RDate.Visible:= true;
       RNowLbl.Enabled:= true;
       RNowLbl.Visible:= true;
       RNow.setCaption(FormatDateTime(DateTimeStr,Now));
       RNow.Enabled:= true;
       RNow.Visible:= true;
       //need to check for notefile
       if FileExists(replacestr(RestFname,'.backup','.note.txt')) then
          begin
            NoteBtn.Enabled:= true;
            NoteBtn.Visible:= true;
          end;
     end
  else
     begin
       RDateLbl.Enabled:= false;
       RDateLbl.Visible:= false;
       RDate.Enabled:= false;
       RDate.Visible:= false;
       RNowLbl.Enabled:= false;
       RNowLbl.Visible:= false;
       RNow.Enabled:= false;
       RNow.Visible:= false;
       NoteBtn.Enabled:= false;
       NoteBtn.Visible:= false;
     end;
end; //end procedure TRestore.ShowDateAndNote
{-------------------------------------------------------------------------------
PROCEDURE TRESTORE.LBOLD_OS
The event handler for OnDrawItem for box4 on a restore = a listbox.

It is identical to TBasePanel.BoldOS other than there is no checkbox to check.

See TBasePanel.BoldOS for more details.

Used by:

V1.0 - 05 May 2021
V1.1 - 02 Sep 2021, modified to use textout on when disabled, see TBasePanel.BoldOS
                    Previously was checking to see if the TCheckList box was enabled/disabled, changed to
                    MainFm.Enabled as this avoids having to enable/disable individual boxes.
-------------------------------------------------------------------------------}
procedure TRestore.LBoxBold_OS(Control: TWinControl; Index: Integer; ARect: TRect; State: TOwnerDrawState);
const
  AdjustL = 5;
  AdjustT = 3;
  Offset = 1;
begin
  ARect.Height:= Box1.ItemHeight; //all box heights are the same
  with (Control as TListBox) do
       if MainFm.Enabled then
          begin
            if MyDrives.isOS(Items[Index]) then Canvas.Font.Bold:= true;
            Canvas.TextOut(ARect.Left + AdjustL,ARect.Top + AdjustT,Items[Index]);
          end
       else
          begin
            Canvas.Brush.Style:= bsClear; //set the background colour = transparent
            //with GTK text in a checkbox that is disabled is grey with a white shadow.
            //draw the shadow first
            Canvas.Font.Color:= clBtnHighlight; //Btn highlight color used to achieve 3D effect
            Canvas.TextOut(ARect.Left + AdjustL+Offset,ARect.Top + AdjustT+Offset,Items[Index]);
            Canvas.Font.Color:=clGrayText; //The font color of disabled element
            Canvas.TextOut(ARect.Left + AdjustL,ARect.Top + AdjustT,Items[Index]);
          end;
end; //end procedure TRestore.LBoxBold_OS
{-------------------------------------------------------------------------------
PROCEDURE TRESTORE.BOXCHECK3CK
This is a bug fix for a problem in lazarus. On box3 the first click on the part table brings up the
enable part edit dialogue and resets the checkbox to what it was - because the click on the checkbox
fires onclick. The second click toggles the checkbox off - this is because onclick is not fired as it
should be. Third click and behaviour is back to normal.

Testing showed that on the second click, for some reason, the onclick handler is NOT fired. It should be.
It did show that the onclickcheck handler fired. So for this TCheckListBox an onclickcheck handler is
required to simply set the checkbox back to what it should be. This should not be required.

As of date, not tested other boxes to see if problem exists with them.

Used by:

V1.0 - 28 May 2021
-------------------------------------------------------------------------------}
procedure TRestore.Box3CheckCK(Sender: TObject);
begin
  if Box3.ItemIndex >= 0 then with Box3 do Checked[ItemIndex]:= CheckState[ItemIndex];
end; //end procedure TRestore.Box3CheckCK
{-------------------------------------------------------------------------------
PROCEDURE TRESTORE.BOX3CK
The event handler for OnClick for box3.

Having found that the hooks for onclick events do not behave as expected, see
   https://forum.lazarus.freepascal.org/index.php/topic,54622.msg405980.html#msg405980
TFoxCheckBox has been modified to include CheckState : array of boolean. This is initialised with
PopulateBox3 where it is set true/false to mirror the state of Box3 as populated. It is used here
to determine current status of the item clicked and is updated, dependent on the outcome of the click.

CheckState is needed because clicking on the item does not change the checkbox state, whereas clicking
on the checkbox does change its state.

The rationale followed for the contents of Box3 are:

  The partition table is always the first entry in the checklist box.

  If the partition table has been changed since the backup, it MUST be restored, i.e. set true.

  An extended partition follows the setting for the partition table. It cannot be selected independently.

  If the partition table is to be restored, then an EFI partition must be restored.

  A partition where there is no backup cannot be set true.

  Any changes to the status of items in Box3 must be reflected in Box3.CheckState.

No changes are made to RDrive - RDrive.Partitions[i].Selected defines the status of the backup. When
false, no backup exists for that partition, set true = the partition has a backup.

Note - PopulateBox3 adds partitions from RDrive, as item[0] is always the partition table,
item[i] will always be RDrive.partitions[i-1]

Still having problems with checkboxes!!!

Used by:

V1.0 - 23 May 2021
V1.1 - 28 May 2021, modified to use checkstate to sort problems with clicking in TCheckListBoxes
-------------------------------------------------------------------------------}
procedure TRestore.Box3CK(Sender: TObject);
const
  ExtendStr = 'extended';
  PartStr = '[partition]';
var
  Source, i : byte;
  Msg1 : string;
begin
  if Box3.ItemIndex < 0 then exit; //need to add >= box.count
  Source:= Box3.ItemIndex;
  DebugInfo(true,'TRestore.Box3CK',Box3.Items[Source]);
  //a checkbox click may have flipped the state so reset (observed in testing)
  Box3.Checked[Source]:= Box3.CheckState[Source];
  if not Defaults.Advanced then //partition edit has not been enabled
     begin
       if not Box3Toggle then
          begin
            FoxMsg(rPartEditCapStr,[rPartEditStr],mtInformation,[mbOK]);
            Box3Toggle:= true; //so the message only shows once
          end;
       exit;
     end;
  with Box3 do //a set of if statements each exiting the procedure
     begin
       if Source = 0 then //it's the partition table - if changed it must be restored
          begin
            if CheckPartTable then //then okay to toggle
               begin
                 Checked[Source]:= not CheckState[Source];
                 CheckState[Source]:= Checked[Source]; //save the new state
               end
            else //must restore part table
               begin
                 Checked[Source]:= true; //default assuming part table has changed
                 CheckState[Source]:= true; //must be restored
               end;
            //check to see if there is an extended partition, its state = state of part table
            for i:= 1 to Count - 1 do if pos(ExtendStr,Items[i]) > 0 then
               begin
                 Checked[i]:= Checked[Source];
                 CheckState[i]:= CheckState[Source];
               end;
            exit;
          end;
       //was the partition backed up?
       if not RDrive.Partitions[Source-1].selected then //the partition was not backed up
          begin
            Checked[Source]:= false;
            CheckState[Source]:= false; //these should be no change
            with RDrive.Partitions[Source-1] do Msg1:= replacestr(rNoBackupPartStr,PartStr,Drive + Number);
            FoxMsg(rNoBackupStr,[Msg1],mtInformation,[mbOK]);
            exit;
          end;
       //is it an efi partition?
       if RDrive.Partitions[Source-1].EFIpartition then //if part table true then restore
          if CheckState[0] then
             begin
               Checked[Source]:= true;
               CheckState[Source]:= true;
               with RDrive.Partitions[Source-1] do Msg1:= replacestr(rEFImsg1,PartStr,Drive + Number);
               FoxMsg(rEFItitle,[Msg1,rEFImsg2],mtInformation,[mbOK]);
               exit;
             end;
       //is it an extended partition?
       if pos(ExtendStr,Items[Source]) > 0 then //extended part follows setting of the part table
          begin
            Checked[Source]:= CheckState[0];
            CheckState[Source]:= CheckState[0];
            FoxMsg(rExttitle,[rExtMsg],mtInformation,[mbOK]);
            exit;
          end;
       //a standard partition, state can be changed
       Checked[Source]:= not CheckState[Source];
       CheckState[Source]:= Checked[Source];
     end
end; //end procedure TRestore.Box3CK
{-------------------------------------------------------------------------------
PROCEDURE TRESTORE.NOTECK
The event handler for the view note button in the restore window.

Creates a note form and populates it.

Note the need to disable/re-enable individual boxes. This is required so that the custome ondrawitem
for the boxes can detect the box has been disabled. Disabling the mainform does not set enabled = false
on child objects. With 'standard' Tchecklistboxes/listboxes without custom draw is is only necessary to
disable the main form and all the children appear disabled.

Used by:

V1.0 - 05 May 2021
V1.1 - 07 Jun 2021, amended for clone, not all objects are assigned in clone
V1.2 - 02 Sep 2021, previously was having to enable/disable individual boxes because of the custom
                    ondraw events checking if the box was enabled. The ondraw events now check to see
                    if the main form is enabled/disabled. Saves code here.
V1.3 - 04 Dec 2021, caption on note button changed to rclose. This is used to test in noteform on whether
                    to save the file or not, generates an error on restore and clone. Setting it to
                    rClose fixes the issue.
-------------------------------------------------------------------------------}
procedure TRestore.NoteCK(Sender: TObject);
var
  RestNote : TNoteForm;
begin
  RestNote:= TNoteForm.create(Self);
  MainFm.Enabled:= false;
  with RestNote do
      begin
        OkBtn.Enabled:= true;
        OkBtn.Visible:= true;
        OKBtn.Caption:= rClose;
        BackMemo.Lines.LoadFromFile(replacestr(RestFname,'.backup','.note.txt'));
        ShowModal;
      end;
  RestNote.Free;
  MainFm.Enabled:= true;
end; //end procedure TRestore.NoteCK
{-------------------------------------------------------------------------------
PROCEDURE TRESTORE.CLEARALL
A wrapper to clear all boxes in the restore tab.

Used by: TBasePanel.CreateNew

V1.0 - 30 Mar 2021
V1.1 - 20 Jun 2021, was missing clearing date and note
V1.2 - 21 Jun 2021, modded so can use with verify, check lbox4 assigned.
-------------------------------------------------------------------------------}
procedure TRestore.ClearAll;
begin
  with Self do
      begin
        ClearBox(1);
        ClearBox(2);
        ClearBox(3);
        if assigned(LBox4) then ClearBox(40);
        ShowDateAndNote(false);
      end;
end; //end procedure TRestore.ClearAll
{-------------------------------------------------------------------------------
FUNCTION TRESTORE.CHECKPARTTABLE
On a restore checks to see if the partition table has changed since the backup.

Returns true is the partition table has not changed, false if it has changed.

The check is done by loading the backup sfdisk file into a TStringList and then generating sfdisk
output for the current drive (the drive set acTarget), loading that into another TStringList and
comparing.

First it has to check if the mount point has changed and adjusting the mount point in the backup to
match the current drive.

Two checks are performed:

    A count on the number of lines - if a mismatch, the number of partitions are counted and reported
    as an error message.

    If the # lines are the same, a line by line comparison. If a mismatch, the lines are reported as an
    error message.

Used by:

V1.0 - 14 May 2021
-------------------------------------------------------------------------------}
function TRestore.CheckPartTable : boolean;
const
  BackFileStr = '.backup';
  SFfileStr = '.sfdisk';
  delim = ': ';
  PartStr = '[partitions]';
  DevStr = '/dev/';
var
  BackupPtable, CurrentPtable : TStringList;
  i, dIndex, CurrentParts, BackupParts : shortint;
  Msg1,Msg2 : string;
begin
  CheckPartTable:= true; //default = no change to the part table
  BackupPtable:= TStringList.Create;
  BackupPtable.LoadFromFile(replacestr(RestFname,BackFileStr,SFfileStr));
  CurrentPtable:= TStringList.Create;
  //find the target drive
  dIndex:= MyDrives.getDrive(acTarget);
  if dIndex = -1 then SystemError(18,rSysErrMsg18a);
  GetText('pkexec sfdisk --dump /dev/' + MyDrives[dIndex].MountPt,CurrentPtable);
  //before any checking on differences first need to check if the mount point has changed and amend
  //if necessary - at this point the user has agreed to restore to a different mount point
  if RDrive.MountPt <> MyDrives[dIndex].MountPt then //the mount point has changed
     //amend the backup TStringList to match the current drive - have to modify the 'device: /dev/sda'
     //line and all partition lines, e.g. '/dev/sda1 : start= ...', this line also has the part prefix
     //but a sata drive would never switch to an nvme drive so safe to just replace 'sda'
     for i:= 0 to BackupPtable.Count - 1 do if pos(RDrive.MountPt,BackupPtable[i]) > 0 then
         replacestr(BackupPtable[i],RDrive.MountPt,MyDrives[dIndex].MountPt);
  //check both sfdisk outputs are the same len[/url]gth
  if BackupPtable.Count <> CurrentPtable.Count then
     begin
       //count the number of partitions in the backup and on the current drive
       CurrentParts:= 0;
       for i:= 0 to CurrentPtable.Count - 1 do if pos(DevStr,CurrentPtable[i]) = 1 then CurrentParts+= 1;
       BackupParts:= 0;
       for i:= 0 to BackupPtable.Count - 1 do if pos(DevStr,BackupPtable[i]) = 1 then BackupParts+= 1;
       Msg1:= replacestr(rChkPTbackStr,PartStr,IntToStr(BackupParts));
       Msg2:= replacestr(rChkPTcurrentStr,PartStr,IntToStr(CurrentParts));
       FoxMsg(rPartTableStr,[rChkPTchangeStr + delim,Msg1,Msg2,rChkPTreplaceStr],mtError,[mbOK]);
       CheckPartTable:= false;
       exit;
     end;
  //check the partition tables line by line - both the same length
  for i:= 0 to CurrentPtable.Count - 1 do if CurrentPtable[i] <> BackupPtable[i] then
     begin
       Msg1:= rBackTabStr + delim + BackupPtable[i];
       Msg2:= rChkPTCurrentDriveStr + delim + CurrentPtable[i];
       FoxMsg(rPartTableStr,[rChkPTchangeStr + delim,Msg1,Msg2,rChkPTreplaceStr],mtError,[mbOK]);
       CheckPartTable:= false;
       exit;
     end;
  BackupPtable.Free;
  CurrentPtable.Free;
end; //end function TRestore.CheckPartTable
{-------------------------------------------------------------------------------
PROCEDURE TRESTORE.NEXTBTNCK
Event handler for the next button, if restore is confirmed runs the restore.

Used by:

V1.0 - 12 Nov 2021
-------------------------------------------------------------------------------}
procedure TRestore.NextBtnCK(Sender: TObject);
begin
  if Confirm(acRestore,RestFname) then runFoxClone(acRestore,RestFname);
end; //end procedure TRestore.NextBtnCK
{-------------------------------------------------------------------------------
PROCEDURE TRESTORE.RESIZEFILENAME
Brought out as a separate procedure so that it can be used by the general resize procedure in
basepanel. Uses chopstr to ensure that the restore filename used fits within the width available. This
is governed by the width of box3.

Initially was using box3 width, changed to box1 as there is no box3 in clone, it is a listbox.

Used by:

V1.0 - 25 Sep 2021
V1.1 - 13 Oct 2021, const space moved to utilities
-------------------------------------------------------------------------------}
procedure TRestore.ReSizeFilename;
var
  FileWidth : integer;
begin
  //without the if statement a blank restore tab will display the last restfilename used.
  if Box3Lbl.Enabled then
     begin
       FileWidth:= Box1.Width - getTextWidth(rRestBox3Str,Font);
       Box3Lbl.setCaption(rRestBox3Str + space + ChopStr(RestFname,Font,FileWidth));
     end;
end; //end procedure TRestore.ReSizeFilename
{-------------------------------------------------------------------------------
PROCEDURE TRESTORE.RESTORESELECT
On a restore the status of RDrive.partition[i].selected defines whether a backup exists for the
specified partition. The status of the checkbox in Box 3 on the restore tab defines whether the
partition has been selected for restore.

The function returns the status of the checkbox for the item specified.

Parameters
    Item : the index for the item in Box3 on the restore tab. Note that item=0 is the partition table.

Used by: progress form

V1.0 - 12 Nov 2021
-------------------------------------------------------------------------------}
function TRestore.RestoreSelect(Item : byte) : boolean;
begin
  if Item > Box3.Count then RestoreSelect:= false else RestoreSelect:= Box3.Checked[Item];
end; //end function TRestore.RestoreSelect

end.

