{ FoxClone
  Copyright (C) 2021 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}
{-------------------------------------------------------------------------------
UNIT - SETTINGS

The setting tab in foxclone. Defines the panel containing the settings which is made of of three
panels:

    BackSetPanel - backup settings, turn compression ON/OFF and turn split backup files ON/OFF

    USetPanel - enable partition selection, disabled by default to make it less easy for newbies to
    make mistakes. And change font size - to make the display easier to read for those with poor eyesight.

    DebugSetPanel - turn ON/OFF logging and display the log file name (which is saved to the desktop).

This unit defines the data structures and methods for settings.

V1.0 - 28 Jul 2021
-------------------------------------------------------------------------------}
unit settings;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, StdCtrls, ExtCtrls, Graphics, Dialogs, GTKlabel,
  utilities, il8nstrings, fdialog, process, info;

type TBackSet = (seNone, seComp, seSplit, seBoth);
     //seNone = no compression, no split
     //seComp = compression, no split = default
     //seSplit = no compression, split - split is automatically turned ON if the destination is fat32
     //seBoth  = compression and split

type
  TSettings = class(TPanel)
  const
    phBottom = 68; //space at bottom for Tbuttons, declared here because used in resize
    phBorder = 10; //default border unit for gaps vertically in the panel
    pwBorder = 15; //default border unit for gaps horizontally in the panel
    ImageSize = 100; //previously was using SettingImg.Width but first time round was reporting 90
                    //next time round it was 64, so messing up the layout on resize
  private
    PartEdit : boolean; //when true user can select partitions, default is false
    SettingImg : TImage; //image top right in the panel
    TitleLbl : TLabelGTK; //title for the top of the tab
    BackLbl : TLabelGTK;

    BackSetPanel : TPanel;
    CompCheckBox : TCheckBox;
    SplitCheckBox : TCheckBox;

    USetPanel : TPanel;
    UserLbl : TLabelGTK;
    PartEditBox : TCheckBox;
    FontComboBox : TComboBox;
    FontLbl : TLabelGTK;

    DebugSetPanel : TPanel;
    DebugLbl : TLabelGTK;
    DebugBox : TCheckBox;
    DebugInfo : TLabelGTK;

    procedure EditPartitions(Sender: TObject);
    procedure SettingsFontSize(Sender: TObject);
    procedure CompClick(Sender: TObject);
    procedure SplitClick(Sender: TObject);
    procedure DebugCK(Sender: TObject);
  public
    BackSettings : TBackSet;
    constructor Create(TheOwner: TComponent);
    property Advanced : boolean read PartEdit;
    property bSettings : TBackSet read BackSettings;
    procedure SettingsResize;
    function IsSplit : boolean;
  end;

  procedure DebugInfo(proc : boolean; fname : string;
                   p1 : string = '';
                   p2 : string = '';
                   p3 : string = '';
                   p4 : string = '';
                   p5 : string = '';
                   p6 : string = '');
var
  Defaults : TSettings; //Defaults is used to determine what to use for a backup,
  RSettings : TBackSet; //RSettings is populated from the restore settings.
  Debug : boolean;
  DebugFname : string;

implementation

uses mainform, backup, restore, clone, verify, about, drives; //needed for font resize, drives to run sfdisk
{-------------------------------------------------------------------------------
CONSTRUCTOR TSETTINGS.CREATE
Creates the settings tab

Used by:

V1.0 - 28 Jul 2021 - created before this, but this is when the strings were converted to consts in
                     il8nstrings.
V1.1 - 09 Dec 2021, Problems with visual appearance in the iso, finally fixed by setting the colour
                    of the panel to clForm (after trying many of the other colours). See
                    https://wiki.freepascal.org/Colors#System_colors.
-------------------------------------------------------------------------------}
constructor TSettings.Create(TheOwner: TComponent);
var
  i : byte;
  PanelHeight, PanelWidth : integer;
begin
    inherited Create(TheOwner);
    PartEdit:= false;
    //Image top right
    SettingImg:=TImage.Create(Self);
    Color:= clForm; //required for visual appearance in the iso
    with SettingImg do begin
         Parent:=self;
         AnchorSide[akTop].Side:= asrTop;
         AnchorSide[akTop].Control:= self;
         BorderSpacing.top:= phBorder;
         AnchorSide[akLeft].Side:= asrLeft;
         AnchorSide[akLeft].Control:= self;
         BorderSpacing.left:= wWidth - ImageSize;
         BorderSpacing.right:= 0;
         Picture.LoadFromFile(SettingsImgFile);
         AutoSize:= true;
    end;
    // Title label
    TitleLbl:= TLabelGTK.Create(Self);
    with TitleLbl do begin
         Parent:= self;
         AnchorSide[akTop].Side:= asrTop;
         AnchorSide[akTop].Control:= self;
         BorderSpacing.top:= phBorder * 2;
         AnchorSide[akLeft].Side:= asrLeft;
         AnchorSide[akLeft].Control:= self;
         BorderSpacing.left:= pwBorder;
         Font.Size:= DefaultFontSize + 4;
         setCaption(rSettingTitle);
    end;
    PanelHeight:= (wHeight - wGap - 13 * phBorder - TitleLbl.Height) div 3;
    PanelWidth:= wWidth - 2 * pwborder - ImageSize;
    //border around the backup settings - compress and split
    BackSetPanel:= TPanel.Create(Self);
    with BackSetPanel do begin
         Parent:= self;
         Color:= clForm; //required for visual appearance in the iso
         AnchorSide[akTop].Side:= asrBottom;
         AnchorSide[akTop].Control:= TitleLbl;
         BorderSpacing.top:= phBorder * 2;
         AnchorSide[akLeft].Side:= asrLeft;
         AnchorSide[akLeft].Control:= TitleLbl;
         BorderSpacing.left:= 0;
         BevelInner:=bvLowered;
         Height:= PanelHeight;
         Width:= PanelWidth;
    end;
    BackLbl:= TLabelGTK.Create(Self);
    with BackLbl do begin
         Parent:= BackSetPanel;
         AnchorSide[akTop].Side:= asrTop;
         AnchorSide[akTop].Control:= BackSetPanel;
         BorderSpacing.top:= 0;
         AnchorSide[akLeft].Side:= asrLeft;
         AnchorSide[akLeft].Control:= BackSetPanel;
         BorderSpacing.left:= pwBorder div 2;
         setCaption(rBackupSettings);
    end;
    CompCheckBox:= TCheckBox.Create(Self);
    with CompCheckBox do begin
         Parent:= BackSetPanel;
         AnchorSide[akTop].Side:= asrBottom;
         AnchorSide[akTop].Control:= BackLbl;
         BorderSpacing.top:= phBorder;
         AnchorSide[akLeft].Side:= asrLeft;
         AnchorSide[akLeft].Control:= BackSetPanel;
         BorderSpacing.left:= pwBorder;
         Caption:= rCompFiles;
         Checked:= true;
         OnClick:= @CompClick;
    end;
    SplitCheckBox:= TCheckBox.Create(Self);
    with SplitCheckBox do begin
         Parent:= BackSetPanel;
         AnchorSide[akTop].Side:= asrBottom;
         AnchorSide[akTop].Control:= CompCheckBox;
         BorderSpacing.top:= phBorder;
         AnchorSide[akLeft].Side:= asrLeft;
         AnchorSide[akLeft].Control:= BackSetPanel;
         BorderSpacing.left:= pwBorder;
         Caption:= rSplitFiles;
         OnClick:= @SplitClick;
    end;
    USetPanel:= TPanel.Create(Self);
    with USetPanel do begin
         Parent:= self;
         Color:= clForm; //required for visual appearance in the iso
         AnchorSide[akTop].Side:= asrBottom;
         AnchorSide[akTop].Control:= BackSetPanel;
         BorderSpacing.top:= phBorder * 2;
         AnchorSide[akLeft].Side:= asrLeft;
         AnchorSide[akLeft].Control:= TitleLbl;
         BorderSpacing.left:= 0;
         BevelInner:=bvLowered;
         Height:= PanelHeight;
         Width:= PanelWidth;
    end;
    UserLbl:= TLabelGTK.Create(Self);
    with UserLbl do begin
         Parent:= USetPanel;
         AnchorSide[akTop].Side:= asrTop;
         AnchorSide[akTop].Control:= USetPanel;
         BorderSpacing.top:= 0;
         AnchorSide[akLeft].Side:= asrLeft;
         AnchorSide[akLeft].Control:= USetPanel;
         BorderSpacing.left:= pwBorder div 2;
         setCaption(rUserSettings);
    end;
    PartEditBox:= TCheckBox.Create(Self);
    with PartEditBox do begin
         Parent:= USetPanel;
         AnchorSide[akTop].Side:= asrBottom;
         AnchorSide[akTop].Control:= UserLbl;
         BorderSpacing.top:= phBorder;
         AnchorSide[akLeft].Side:= asrLeft;
         AnchorSide[akLeft].Control:= UserLbl;
         BorderSpacing.left:= 0;
         Caption:= rPartSelect;
         OnClick:= @EditPartitions;
    end;
    FontComboBox:= TComboBox.Create(Self);
    with FontComboBox do
         begin
           Parent:= USetPanel;
           Style:= csDropDownList;
           AnchorSide[akTop].Side:= asrBottom;
           AnchorSide[akTop].Control:= PartEditBox;
           BorderSpacing.top:= phBorder;
           AnchorSide[akLeft].Side:= asrLeft;
           AnchorSide[akLeft].Control:= UserLbl;
           BorderSpacing.left:= 0;
           Width:= GetTextWidth('00',Font) + 30; //  40;
           OnChange := @SettingsFontSize;
           ReadOnly:= true; //deprecated
          end;
    for i:= 1 to 9 do FontComboBox.Items.Add(IntToStr(i+9));
    FontComboBox.ItemIndex:= 0;
    FontLbl:= TLabelGTK.Create(Self);
    with FontLbl do begin
         Parent:= USetPanel;
         AnchorSide[akTop].Side:= asrCenter;
         AnchorSide[akTop].Control:= FontComboBox;
         BorderSpacing.top:= 0;
         AnchorSide[akLeft].Side:= asrRight;
         AnchorSide[akLeft].Control:= FontComboBox;
         BorderSpacing.left:= pwBorder div 2;
         setCaption(rFontSize);
    end;
    DebugSetPanel:= TPanel.Create(Self);
    with DebugSetPanel do begin
         Parent:= self;
         Color:= clForm; //required for visual appearance in the iso
         AnchorSide[akTop].Side:= asrBottom;
         AnchorSide[akTop].Control:= USetPanel;
         BorderSpacing.top:= phBorder * 2;
         AnchorSide[akLeft].Side:= asrLeft;
         AnchorSide[akLeft].Control:= TitleLbl;
         BorderSpacing.left:= 0;
         BevelInner:=bvLowered;
         Height:= PanelHeight;
         Width:= PanelWidth
    end;
    DebugLbl:= TLabelGTK.Create(Self);
    with DebugLbl do begin
         Parent:= DebugSetPanel;
         AnchorSide[akTop].Side:= asrTop;
         AnchorSide[akTop].Control:= DebugSetPanel;
         BorderSpacing.top:= 0;
         AnchorSide[akLeft].Side:= asrLeft;
         AnchorSide[akLeft].Control:= DebugSetPanel;
         BorderSpacing.left:= pwBorder div 2;
         setCaption(rTroubleShoot);
    end;
    DebugBox:= TCheckBox.Create(Self);
    with DebugBox do begin
         Parent:= DebugSetPanel;
         AnchorSide[akTop].Side:= asrBottom;
         AnchorSide[akTop].Control:= DebugLbl;
         BorderSpacing.top:= phBorder;
         AnchorSide[akLeft].Side:= asrLeft;
         AnchorSide[akLeft].Control:= DebugLbl;
         BorderSpacing.left:= 0;
         Caption:= rEnableDebug;
         OnClick:= @DebugCK;
    end;
    DebugInfo:= TLabelGTK.Create(Self);
    with DebugInfo do begin
         Parent:= DebugSetPanel;
         AnchorSide[akTop].Side:= asrBottom;
         AnchorSide[akTop].Control:= DebugBox;
         BorderSpacing.top:= 0;
         AnchorSide[akLeft].Side:= asrLeft;
         AnchorSide[akLeft].Control:= DebugLbl;
         BorderSpacing.left:= 0;
         setCaption('');
    end;
    //set the initial settings
    BackSettings:= seComp;
    //turn off debug
    Debug:= false;
end; //end constructor
{-------------------------------------------------------------------------------
PROCEDURE TSETTINGS.EDITPARTITIONS
Sets the boolean variable PartEdit based on the state of the part edit checkbox. PartEdit is available
globablly through the property 'Advanced' and used to check if selecting partitions is allowed.

Used by:

V1.0 - 28 Jul 2021 - created before this.
-------------------------------------------------------------------------------}
procedure TSettings.EditPartitions(Sender: TObject);
begin
  PartEdit:= PartEditBox.Checked;
end; //end TSettings.EditPartitions
{-------------------------------------------------------------------------------
PROCEDURE TSETTINGS.SETTINGSFONTSIZE
Based on the font size selected, sets font size in all tabs and all labels, etc.

Note - warning/error messages are not resized.

Used by:

V1.0 - 18 Aug 2021
-------------------------------------------------------------------------------}
procedure TSettings.SettingsFontSize(Sender: TObject);
var
  FontSize : byte;
begin
  with FontComboBox do FontSize:= strtoint(Items[ItemIndex]);
  MainFm.Font.Size:= FontSize;

  FontComboBox.Width:= GetTextWidth('00',Font) + 40;

  MainFm.QuitBtn.Width:= GetTextWidth(rQuitStr,MainFm.QuitBtn.Font) + 60;
  MainFm.QuitBtn.Height:= GetTextHeight(rQuitStr,MainFm.QuitBtn.Font) + 14;
  //tab titles
  BackTab.FontResize;
  RestoreTab.FontResize;
  CloneTab.FontResize;
  VerifyTab.FontResize;
  //need to reset caption to reset width of caption on resize
  Defaults.TitleLbl.Font.Size:= MainFm.Font.Size + 4;
  Defaults.TitleLbl.setCaption(Defaults.TitleLbl.Caption);
  AboutTab.TitleLbl.Font.Size:= MainFm.Font.Size + 4;
  AboutTab.TitleLbl.setCaption(AboutTab.TitleLbl.Caption);
  //settings tab
  BackLbl.setCaption(BackLbl.Caption);
  UserLbl.setCaption(UserLbl.Caption);
  FontLbl.setCaption(FontLbl.Caption);
  DebugLbl.setCaption(DebugLbl.Caption);
  DebugInfo.setCaption(DebugInfo.Caption);
end; //end TSettings.SettingsFontSize
{-------------------------------------------------------------------------------
PROCEDURE TSETTINGS.SETTINGSRESIZE
Resizes the settings tab, had some troubles initially with the horizontal resizing
due to the image size of the icon changing between as constructed and resized.

Used by:

V1.0 - 19 Aug 2021
-------------------------------------------------------------------------------}
procedure TSettings.SettingsResize;
var
  PWidth, PHeight : integer;
begin
  SettingImg.BorderSpacing.left:= Parent.ClientWidth - ImageSize;
  PWidth:= Parent.ClientWidth - 2 * pwborder - ImageSize + 5;
  // +23 is the difference between Parent.ClientHeight and wHeight - wGap
  PHeight:= (Parent.ClientHeight - 13 * phBorder - TitleLbl.Height + 23) div 3;
  BackSetPanel.Width:= PWidth;
  BackSetPanel.Height:= PHeight;
  USetPanel.Width:= PWidth;
  USetPanel.Height:= PHeight;
  DebugSetPanel.Width:= PWidth;
  DebugSetPanel.Height:= PHeight;
end; //end TSettings.SettingsResize
{-------------------------------------------------------------------------------
PROCEDURE TSETTINGS.COMPCLICK
Event handler for clicking on the compression checkbox.

Sets BackSettings dependent on the state of the checkbox and the state of the split checkbox.

seNone = no compression, no split
seComp = compression, no split = default
seSplit = no compression, split - split is automatically turned ON if the destination is fat32
seBoth  = compression and split

Used by:

V1.0 - 22 Oct 2021
-------------------------------------------------------------------------------}
procedure TSettings.CompClick(Sender: TObject);
begin
  if CompCheckBox.Checked then
     if SplitCheckBox.Checked then BackSettings:= seBoth else BackSettings:= seComp
  else
     if SplitCheckBox.Checked then BackSettings:= seSplit else BackSettings:= seNone;
end; //end procedure TSettings.CompClick
{-------------------------------------------------------------------------------
PROCEDURE TSETTINGS.SPLITCLICK
Event handler for clicking on the split checkbox.

Sets BackSettings dependent on the state of the checkbox and the state of the compression checkbox.

Used by:

V1.0 - 22 Oct 2021
-------------------------------------------------------------------------------}
procedure TSettings.SplitClick(Sender: TObject);
begin
  if SplitCheckBox.Checked then
     if CompCheckBox.Checked then BackSettings:= seBoth else BackSettings:= seSplit
  else
     if CompCheckBox.Checked then BackSettings:= seComp else BackSettings:= seNone;
end; //end procedure TSettings.SplitClick
{-------------------------------------------------------------------------------
FUNCTION TSETTINGS.ISSPLIT
Returns the setting of the splitcheckbox.

Used by: used to test if the destination for a backup is fat32 and if so then must split files.

V1.0 - 23 Oct 2021
-------------------------------------------------------------------------------}
function TSettings.IsSplit : boolean;
begin
  IsSplit:= SplitCheckBox.Checked;
end; //end function TSettings.IsSplit
{-------------------------------------------------------------------------------
PROCEDURE TSETTINGS.DEBUGCK
Event handler for clicking on the debug checkbox.

Toggles the debug flag on/off. When on creates the debug file and dumps parted output into it and then
the sfdisk output for each drive in the system.

The debug file is dumped into the first user found in /home, on the iso this will be ubuntu, when
installed from deb into an existing ubuntu install it will the the first user discovered.

It will not capture the initial reading of the system as this is done before debug can be initiated.

Used by:

V1.0 - 26 Oct 2021
V1.1 - 28 Oct 2021, added in the print methods for drives and partitions
V1.2 - 12 Dec 2021, bug fix, index out of bounds on a blank drive or drive with loop part table.
V1.3 - 13 Dec 2021, Version and build date added to debug header. Debug file saved to desktop folder
                    if it exists.
V1.4 - 28 Dec 2021, modified to take advantage of sysinfo - output from parted and sfdisk already
                    available, much quicker to generate the log file.
V1.5 - 15 Jan 2022, log file name changed.
-------------------------------------------------------------------------------}
procedure TSettings.DebugCK(Sender: TObject);
const
  FindHomeStr = 'find /home -maxdepth 1 -type d | grep /home/';
  FoxcloneFnameStr = '-foxclone.log';
  TitleStr = ' Foxclone log';
  emailStr = 'email this file to help@foxclone.com';
  DesktopFolder = '/Desktop';
  LogStartStr = 'START LOG';
  FoxAnalysisStr = 'Foxclone system analysis';
var
  TextList : TStringList;
  i,j : integer;
  DebugF : TextFile;
begin
  if DebugBox.Checked then
     begin
       //find user's home folder
       TextList:= TStringList.Create;
       gettext(FindHomeStr,TextList);
       if TextList.Count > 0 then DebugFname:= TextList[0] else
          begin
            Debug:= false;
            FoxMsg(rDebugTitleStr,[rDebugErrStr,FindHomeStr,rDebugErrStr2],mtInformation,[mbOK]);
          end;
       //check for desktop folder - if it exists save debug file on desktop
       if DirectoryExists(DebugFname + DesktopFolder) then
          DebugFname+= DesktopFolder + '/' + FormatDateTime('yyyymmdd',now) + FoxcloneFnameStr
       else DebugFname+='/' + FormatDateTime('yyyymmdd',now) + FoxcloneFnameStr;
       DebugInfo.setCaption(rDebugInfo + DebugFname);
       //helps with the caption, changed state of the checkbox takes longer
       application.processmessages; //can take a few seconds to run parted, sfdisk, etc.
       //write header info, [time] header and email to ...
       TextList.Clear;
       TextList.Add(FormatDateTime('yyyymmdd',now)+ TitleStr);
       TextList.Add('Version: ' + inttostr(Version) + space + rVerDateStr);
       TextList.Add(emailStr);
       TextList.Add(Separator2);
       //get parted output
       SysInfo.CopyPartedLM(TextList); //the machine readable version of parted
       TextList.Add(Separator2);
       SysInfo.CopyParted(TextList); //the standard version of parted
       TextList.Add(Separator2);
       //get sfdisk output for each drive
       SysInfo.CopyPtables(TextList); //the standard version of parted
       TextList.SaveToFile(DebugFname);
       TextList.free;
       //now using the old fashioned way print foxclones view of drives and partitions.
       AssignFile(DebugF,DebugFname);
       append(DebugF);
       writeln(DebugF,FoxAnalysisStr);
       for i:=0 to MyDrives.Count - 1 do
           begin
             MyDrives[i].Print(DebugF);
             //getting index out of bounds error so test
             with MyDrives[i] do if TotalParts > 0 then
                  for j:= 0 to TotalParts - 1 do Partitions[j].Print(DebugF);
           end;
       writeln(debugF,Separator2);
       writeln(debugF,LogStartStr);
       CloseFile(DebugF);

       Debug:= true; //put at end so initial termcmd not dumped in debug file
     end
  else
     begin
       Debug:= false;
       DebugInfo.setCaption('');
     end;
end; //end procedure TSettings.DebugCK
{-------------------------------------------------------------------------------
PROCEDURE DEBUGINFO

The old fashioned way of text file handling is used as use of tstringlist does not support append.
It also appends to the debug file and closes it, so hopefully, in the event of a problem info will
been recorded prior to foxclone hanging/crashing.

Is used by any procedure/function where it is useful to record debug info.

Typical use is to pass the procedure/function name and then any parameters or useful information.
It adds a line to the existing debug file of the form:

     [16:34:31] procedure termCmd : lsblk -nio mountpoint /dev/sdb2

This might be better implement with the strings p1-p6 passed as an array of strings? This would mean
that there was no limit on the number of strings to pass.

Used by:

V1.0 - 26 Oct 2021
-------------------------------------------------------------------------------}
procedure DebugInfo(proc : boolean; fname : string;
                   p1 : string = '';
                   p2 : string = '';
                   p3 : string = '';
                   p4 : string = '';
                   p5 : string = '';
                   p6 : string = '');
const
  delim = ', ';
  separator = ' : ';
  functionStr = 'function ';
  procedureStr = 'procedure ';
var
  DebugStr : string;
  DebugF : TextFile;
begin
  if not Debug then exit;
  DebugStr:= '[' + FormatDateTime('hh:nn:ss',now) + '] ';
  if proc then DebugStr+= procedureStr + fname else DebugStr+= functionStr + fname;
  if p1 <> '' then DebugStr+= separator + p1;
  if p2 <> '' then DebugStr+= delim + p2;
  if p3 <> '' then DebugStr+= delim + p3;
  if p4 <> '' then DebugStr+= delim + p4;
  if p5 <> '' then DebugStr+= delim + p5;
  if p6 <> '' then DebugStr+= delim + p6;
  AssignFile(DebugF,DebugFname);
  append(DebugF);
  writeln(DebugF,DebugStr);
  CloseFile(DebugF);
end; //procedure DebugInfo

end.

