{ FoxClone
  Copyright (C) 2021 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}
{-------------------------------------------------------------------------------
UNIT - UTILITIES

Various procedures, functions and constants used across foxclone.

V1.0 - 21 Feb 2021
V1.2 - 05 Jan 2022 - gpartedCheck added
-------------------------------------------------------------------------------}
{TESTING and CHANGES

need to check a restoring where the target has moved, e.g. from sda to sdb

need to check that info is displayed correctly when a backup is not full and the box shows a partition
that has not been backed up.

does mkdir have an option to create the upper folder if not present - to prevent the need to create
/mnt/foxclone?     yes the -p option. DONE and TESTED

default and cancel = true on other buttons

testing - need to check a restore where the mount point has changed - to test the checkpartable function

testing - need to check restore box3 when restoring uefi and part table hasn't changed - can you toggle
selecting the efi partition.

}
unit utilities;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Graphics, process, Dialogs, strutils;

const
  Version = 51;
  LF = LineEnding; //shorter
  Space = ' ';
  EscSpace = '\ '; //for escaping any spaces in terminal commands

  wHeight = 600;  //main window initial height
  wWidth = 800;   //main window initial width
  wGap = 60;      //main window margin at bottom for version label was 40 try 50

  DefaultFontSize = 10;

  DefaultFolder = '/tmp/foxclone/';
  MountFolder = '/mnt/foxclone/';
  ImageFolder = '/usr/share/pixmaps/foxclone/';

  BackupImgFile = ImageFolder + 'backup.png';
  BackupImgWidth = 198; //it is not the same as reported in Xviewer, done by trial & error

  RestoreImgFile = ImageFolder + 'restore.png';
  RestoreImgWidth = 198; //not used BackupImgWidth is working

  CloneImgFile = ImageFolder + 'clone.png';
  CloneFileImgFile = ImageFolder + 'clonefile.png';
  CloneImgWidth = 215; //it is not the same as reported in Xviewer, done by trial & error
  CloneFileImgWidth = 205;

  VerifyImgFile = ImageFolder + 'verify.png';
  VerifyFileImgWidth =200;

  SettingsImgFile = ImageFolder + 'settings64.png';

  AboutImgFile = ImageFolder + 'fox-icon96.png';

  DriveImgFile = ImageFolder + 'drive80.png';
  PartImgFile = ImageFolder + 'partition64.png';
  FileImgFile = ImageFolder + 'file2-64.png';
  ArrowImgFile = ImageFolder + 'arrow.png';
  DnArrowImgFile = ImageFolder + 'downarrow.png';
  FoxImgFile =  ImageFolder + 'fox-icon96.png';
  CheckImgFile = ImageFolder + 'checkbox.png';
  InfoImgFile =  ImageFolder + 'information.png';
  InfoImgWidth = 16;
  //used in terminal dumps of the drives & partitions
  separator2 = '--------------------------------------------------------------------------------';
  separator =  '================================================================================';

type
  //used across foxclone to specify the action to be taken
  TFoxType = (acBackup, acRestore, acClone, acCloneF, acVerify);

  function GetTextWidth(AText: String; AFont: TFont): Integer;
  function GetTextHeight(aText: String; AFont: TFont): Integer;
  function SizeStr(blocks : int64) : string;
  procedure termCmd(CmdLine: string; var Process : TProcess);
  procedure getText(cmdstr : string; AList : TStringList);
  function getstr(cmdstr : string) : string;
  function mkdir(MountFolder : string) : integer;
  function rmdir(partition : string) : integer;
  procedure SystemError(Reference : byte; CallFunction : string; aMsg : string = '');
  function ChopStr(TextStr : string; Font : TFont; Width : integer) : string;
  function SectorToBlock(Sectors : int64; SectorSize : integer) : int64;
  function BlockToSector(Blocks : int64; SectorSize : integer) : int64;
  procedure SwapOn(OnOff : boolean);
  function gpartedCheck : boolean;
  function FormatMB(Blocks : int64) : string;
  function FindVentoy : string;

var
  debug : boolean = false;
  advanced : boolean = false; //put here now will be part of settings


implementation

uses
  mainform,il8nstrings,fdialog, settings; //fdialog uses imagefolder which is defined in utilities
{-------------------------------------------------------------------------------
FUNCTION GETTEXTWIDTH
Return the width of a text string in pixels.
Used in checklistboxes to determine item width and hence turn on/off horizontal scroll bars
Used in some labels to determine alignment as lazarus.

It pastes the text into a bitmap to determine width. Did look at other alternatives but based on
units either very old not straightforward.

Parameters:
  aText - the string to calculate the width of
  aFont - the font being used.

Used by:

V1.0 - 21 Feb 2021
-------------------------------------------------------------------------------}
function GetTextWidth(aText: String; AFont: TFont): Integer;
var
  bmp: TBitmap;
begin
  Result := 0;
  bmp := TBitmap.Create;
  try
    bmp.Canvas.Font.Assign(AFont);
    Result := bmp.Canvas.TextWidth(AText);
  finally
    bmp.Free;
  end;
end; //function GetTextWidth
{-------------------------------------------------------------------------------
FUNCTION GETTEXTHEIGHT
Return the height of a text string in pixels.

It pastes the text into a bitmap to determine width. Did look at other alternatives but based on
units either very old not straightforward.

Parameters:
  aText - the string to calculate the width of
  aFont - the font being used.

Used by:

V1.0 - 21 Feb 2021
-------------------------------------------------------------------------------}
function GetTextHeight(aText: String; AFont: TFont): Integer;
var
  bmp: TBitmap;
begin
  Result := 0;
  bmp := TBitmap.Create;
  try
    bmp.Canvas.Font.Assign(AFont);
    Result := bmp.Canvas.TextHeight(AText);
  finally
    bmp.Free;
  end;
end; //function GetTextHeight
{-------------------------------------------------------------------------------
FUNCTION SIZESTR
Returns a string formatted in MB, GB or TB and used to display drive and partition sizes.
Note - assumes that the parameter passed is in 1k blocks and this is declared as a constant just in
case a change is made to sectors at some point in the future.

Parameters:
  blocks - number of blocks to be converted

Used by:

V1.0 - 21 Feb 2021
-------------------------------------------------------------------------------}
function SizeStr(blocks : int64) : string;
const
  blocksize = 1024; //bytes assuming logical sector size
var
  size : real;
begin
  size:= blocks * blocksize / 1000000; //size in MB
  if size < 1000 then SizeStr:= FormatFloat('#,##0',size)+ 'MB'
  else
    begin
      size:= size / 1000;
      if size < 1000 then SizeStr:= FormatFloat('#,##0',size)+ 'GB'
      else
        begin
          size:= size / 1000;
          SizeStr:= FormatFloat('#,##0.0',size)+ 'TB';
        end;
    end;
end; //end function SizeStr
{-------------------------------------------------------------------------------
PROCEDURE TERMCMD
Executes a terminal command. To allow for piping all commands are created and
run from bash.

Modified version of original used in earlier foxclone. It is now the responsibility of the calling
process to create Process (to avoid warning messages in the compiler

Parameters:
  CmdLine - a string containing the command line to be executed
  Process - a TProcess to be used by calling process to read output either from
            stdout or stderr

Used by:

V1.0 - 27 May 2019
V1.1 - 30 Dec 2019 - debug code
V1.2 - 21 Feb 2021 - modified for new foxclone, creation of process removed
V1.3 - 26 Oct 2021 - debug added
-------------------------------------------------------------------------------}
procedure termCmd(CmdLine: string; var Process : TProcess);
const
  execCmd = '/bin/bash';
begin
  DebugInfo(true,'termCmd',CmdLine);
  with Process do
       begin
         Executable:= execCmd;
         Parameters.Add('-c');
         Parameters.Add(CmdLine);
         Options:= [poUsePipes]; //to get output
         Execute; //run
       end;
end; //end procedure TermCmd
{-------------------------------------------------------------------------------
PROCEDURE GETTEXT
Runs a command and returns the result as a TStringList. It is the responsibility of the calling
process to create the TStringList.

Could a string be used instead of an array of char?

Parameters:
  cmdStr - a string containing the command line to be executed
  aList - the TStringList to be populated with the output from the command

Used by:

V1.0 - 21 Feb 2021
V1.1 - 26 Oct 2021, bufsize increased from 1024 to 4096
-------------------------------------------------------------------------------}
procedure getText(cmdstr : string; aList : TStringList);
const
  bufsize = 4096; // Buffer size for reading output
var
  aProcess : TProcess;
  aStream : TMemoryStream;
  Buffer    : array[1..bufsize] of char;
  BytesRead, i : integer;
begin
  aProcess:= TProcess.create(nil);
  aStream:= TMemoryStream.Create;
  termCmd(cmdstr,aProcess);
  for i:= 1 to bufsize do Buffer[i]:= chr(0); //to get rid of compiler warning
  repeat
    BytesRead:= aProcess.Output.Read(Buffer, bufsize);
    aStream.Write(Buffer, BytesRead);
  until BytesRead = 0;  // Stop if no more data is available
  aProcess.Free; //finished with it
  aStream.Position:= 0; //make sure we start at the beginning of the stream
  aList.LoadFromStream(aStream);
  aStream.Free;
end; //end procedure getText
{-------------------------------------------------------------------------------
PROCEDURE GETSTR
Runs a command and returns the result as a string. Generally used to run commands to get drive and
partition information when populating TDrives and TPartitions.

Early testing showed that occasionally some commands would return empty, hence a short delay has been
put in and a loop to retry a limited number of times. So on any command where an empty string is an
okay output it will take a little longer.

To think about - reading from stdout and stderr??

Parameters:
  cmdStr - a string containing the command line to be executed

Used by:

V1.0 - 21 Feb 2021
-------------------------------------------------------------------------------}
function getstr(cmdstr : string) : string;
const
  bufsize = 128; // Buffer size for reading output
  delay = 50;    //millisecs
  count = 5;
var
  strProcess : TProcess;
  buffer     : string = '';
  BytesRead    : integer;
  loop : integer = 0;
begin
  repeat
    sleep(delay); //wait a bit
    strProcess:= TProcess.create(nil);
    setlength(buffer,bufsize);
    termCmd(cmdstr,strProcess);
    BytesRead:= strProcess.Output.Read(buffer[1],length(buffer));
    setlength(buffer,BytesRead);
    strProcess.free;
    getstr:= buffer;
    //if getstr = '' then writeln('getstr no output on: ',CmdStr,' trying again...');
    loop+= 1;
  until (loop = count) or (getstr <> '');
  //if (getstr <> '') and (loop > 1) then writeln('success, ',cmdstr); //only if it has looped at least once.
  getstr:= trim(getstr); //get rid of any leading or trailing spaces
end; //end function getstr
{-------------------------------------------------------------------------------
FUNCTION MKDIR
Makes a directory in the default location with the name passed as partition.
Partition will normally be of the form sda1 or sdb3, etc., e.g. it will create /tmp/foxclone/sdd1

The function returns:
  0 = success, the folder was created
  1 = the folder exists but is empty and can be used as a mount point
 -1 = the folder exists and is NOT empty, not safe to use as a mount point

V1.4 - checks to see if the folder exists before trying to create it.
       If the folder exists it checks to see if it is empty, if empty it is safe to use as a mount
       point. If NOT empty it cannot be used as a mount point.

       Initially did try the findnext function - over complicated and even on an empty folder would
       find one file - the folder itself. Much simpler test:
            ls /mnt/foxclone/sdd1 | wc -l
       returns '0' if no files and the number (as a string) of files if not empty.

Parameters:
  partition - a string containing the partition in the form sda1, sdc3, etc.

Used by:

V1.0 - 27 May 2019
V1.1 - 30 Dec 2019 - debug code
V1.2 - 28 Feb 2020 - changed to pkexec mkdir to allow for creating foxclone folder in /mnt
V1.3 - 21 Feb 2021 - added resourcestrings to message dialog
V1.4 - 26 Feb 2021
V1.5 - 02 Apr 2021 - was not returning -1 if error when creating with mkdir = bytesread
V1.6 - 03 May 2021 - added -p to the mkdir command = make the parent if not present, e.g. make
                     /mnt/foxclone if it doesn't exist. Done and tested.
-------------------------------------------------------------------------------}
function mkdir(MountFolder : string) : integer;
const
  bufsize = 64; // Buffer size for reading output
var
  mkdirProcess : TProcess;
  Buffer       : string = '';
  BytesRead    : integer;
begin
  mkdir:= -1; //set to fail
  if not DirectoryExists(MountFolder) then
     begin
       setlength(buffer,bufsize);
       mkdirProcess:= TProcess.create(nil);
       termCmd('pkexec mkdir -p ' + MountFolder,mkdirProcess);
       //output from the command is on stderr
       BytesRead:= mkdirProcess.stderr.Read(buffer[1],length(buffer));
       if BytesRead > 0 then
          begin
            setlength(buffer,BytesRead);
            MessageDlg(rErrorStr,rMkDirFailStr + ' ' + MountFolder
                       + LF + buffer,mtError,[mbOK],0) ;
            mkdir:= -1;
          end
       else mkdir:= 0;
       mkdirProcess.free;
     end
  else if getStr('ls ' + MountFolder + ' | wc -l') = '0' then mkdir:= 1;
end; //end function mkdir
{-------------------------------------------------------------------------------
FUNCTION RMDIR
Removes a folder in the default location with the name passed as partition.
The function returns:
  0 = success.
 >0 = failure, which will normally be because the folder doesn't exist or that
      the folder is not empty.
rmdir is used to delete a folder that has been created to mount a partition.

Parameters:
  partition - a string containing the partition in the form sda1, sdc3, etc.

Used by:

V1.0 - 27 May 2019
V1.1 - 30 Dec 19 - debug code
V1.2 - 28 Feb 2020 - changed to pkexec rmdir as mount folder is now in /mnt and owned by root
V1.3 - 21 Feb 2021 - added resourcestrings to message dialog
-------------------------------------------------------------------------------}
function rmdir(partition : string) : integer;
const
  bufsize = 64; // Buffer size for reading output
var
  rmdirProcess : TProcess;
  Buffer       : string = '';
  BytesRead    : integer;
begin
  setlength(buffer,bufsize);
  rmdirProcess:= TProcess.create(nil);
  termCmd('pkexec rmdir ' + MountFolder + partition,rmdirProcess);
  //output from the command is on stderr
  BytesRead:= rmdirProcess.stderr.Read(buffer[1],length(buffer));
  if BytesRead > 0 then
     begin
       setlength(buffer,BytesRead);
       MessageDlg(rErrorStr,rRmDirFailStr + ' ' + MountFolder + partition
                  + LF + buffer,mtError,[mbOK],0)
     end;
  rmdir:= BytesRead;
  rmdirProcess.free;
end; //end function rmdir
{-------------------------------------------------------------------------------
PROCEDURE SYSTEMERROR
This is a procedure that should never be run. It is invoked as a result of an internal error, such
as clicking on a drive in a checklistbox and then failing to find that drive.

It displays a message dialog and then closes foxclone by calling MainFm.Close.

Originally I was trying to FreeAndNil MyDrives before closing but ran into SIGSEGV errors which I
raised at
https://forum.lazarus.freepascal.org/index.php?topic=53827.new;topicseen#new

The procedure now just calls MainFm.Close and relies on the the main foxcloneXX.lpi file to FreeAndNil
MyDrives. This works.

Parameters:
    Reference - byte used in a case statement to pick the relevant one line message to display
    CallFunction - string, the calling procedure or function
    aMsg - a string, optional, to display additional info in the message dialog. Typically this will
    be a command string on the failed command.

Used by:

V1.0 - 23 Mar 2021
V1.1 - 11 May 2021, FoxDialog replaced with FoxMsg and the error reference added to the title
-------------------------------------------------------------------------------}
procedure SystemError(Reference : byte; CallFunction : string; aMsg : string = '');
var
  ErrorMsg : string;
begin
  case Reference of
       1 : ErrorMsg:= rSysErrMsg1;
       2 : ErrorMsg:= rSysErrMsg2;
       3 : ErrorMsg:= rSysErrMsg3;
       4 : ErrorMsg:= rSysErrMsg4;
       5 : ErrorMsg:= rSysErrMsg5;
       6 : ErrorMsg:= rSysErrMsg6;
       7 : ErrorMsg:= rSysErrMsg7;
       8 : ErrorMsg:= rSysErrMsg8;
       9 : ErrorMsg:= rSysErrMsg9;
      10 : ErrorMsg:= rSysErrMsg10;
      11 : ErrorMsg:= rSysErrMsg11;
      12 : ErrorMsg:= rSysErrMsg12;
      13 : ErrorMsg:= rSysErrMsg13;
      14 : ErrorMsg:= rSysErrMsg14;
      15 : ErrorMsg:= rSysErrMsg15;
      16 : ErrorMsg:= rSysErrMsg16;
      17 : ErrorMsg:= rSysErrMsg17;
      18 : ErrorMsg:= rSysErrMsg18;
      19 : ErrorMsg:= rSysErrMsg19;
      20 : ErrorMsg:= rSysErrMsg20;
      21 : ErrorMsg:= rSysErrMsg21;
      22 : ErrorMsg:= rSysErrMsg22;
      23 : ErrorMsg:= rSysErrMsg23;
      24 : ErrorMsg:= rSysErrMsg24;
      25 : ErrorMsg:= rSysErrMsg25;
      26 : ErrorMsg:= rSysErrMsg26;
  end;
  if aMsg = '' then
     FoxMsg(rSysErrorCaption+' ['+IntToStr(Reference)+']',[CallFunction,ErrorMsg,rCloseMsgStr],mtCustom,[mbOk])
  else
     FoxMsg(rSysErrorCaption+' ['+IntToStr(Reference)+']',[CallFunction,ErrorMsg,aMsg,rCloseMsgStr],mtCustom,[mbOk]);
  MainFm.Close;
end; //end procedure SystemError
{-------------------------------------------------------------------------------
FUNCTION CHOPSTR
Chops a long pathname for a file down to a specified length or less by stripping out parts of the path
starting at the left, e.g. /123/abc/something/else.backup could become .../something/else.backup.

Used for display of filenames where the width available may not be wide enough to show the full pathname.

Parameters:
    TextStr - the string to be chopped.
    Font - the font being used, it is used to calculate the width of the string.
    Width - the maximum width that the string must fit within.

The function returns the shortened filename.

ChopStr cannot shorten the filename if there are no "/" in the pathname. If this is the case the
function returns the original string. If after chopping the string contains no "/" and is still too long,
it returns the amended string.

Used by:

V1.0 - 12 Sep 2021
-------------------------------------------------------------------------------}
function ChopStr(TextStr : string; Font : TFont; Width : integer) : string;
const
  ellipsis = '…';
  slash = '/';
var
  Awidth : integer;
begin
  if GetTextWidth(TextStr,Font) <= Width then //nothing to do
     begin
       ChopStr:= TextStr;
       exit;
     end;
  if pos(slash,TextStr) = 0 then //no sensible way of shortening the string
     begin
       ChopStr:= TextStr;
       exit;
     end;
  //need to remove the path starting at the top. First allow for .../ that will be added
  Awidth:= Width - GetTextWidth(ellipsis + slash,Font);
  while GetTextWidth(TextStr,Font) > Awidth do
        begin
          TextStr:= copy(TextStr,pos(slash,TextStr)+1,99);
          if pos(slash,TextStr) = 0 then break;
        end;
  ChopStr:= ellipsis + slash + TextStr;
end; //end function ChopStr
{-------------------------------------------------------------------------------
FUNCTION SECTORTOBLOCK
Converts sectors into 1K blocks. Need a function as doing integer division and multiplication.
Direct calc involves either division or multiplication and x0.5 = real or float.

Majority of drives are 512 byte sectors, but 4096 bytes are common on large drives. Note it is the
logical sector size that is used, not the physical sector size as reported by parted. Don't know
where the physical sector size is used.

Used by:

V1.0 - 24 Oct 2021
-------------------------------------------------------------------------------}
function SectorToBlock(Sectors : int64; SectorSize : integer) : int64;
begin
  case SectorSize of
       512  : SectorToBlock:= Sectors div 2;
       1024 : SectorToBlock:= Sectors;
       2048 : SectorToBlock:= Sectors * 2;
       4096 : SectorToBlock:= Sectors * 4;
       8192 : SectorToBlock:= Sectors * 8;
  end;
end; //end function SectorToBlock
{-------------------------------------------------------------------------------
FUNCTION BLOCKTOSECTOR
Converts 1K blocks into sectors. Need a function as doing integer division and multiplication.
Direct calc involves either division or multiplication and x0.5 = real or float.

Majority of drives are 512 byte sectors, but 4096 bytes are common on large drives. Note it is the
logical sector size that is used, not the physical sector size as reported by parted. Don't know
where the physical sector size is used.

Used by:

V1.0 - 24 Oct 2021
-------------------------------------------------------------------------------}
function BlockToSector(Blocks : int64; SectorSize : integer) : int64;
begin
  case SectorSize of
       512  : BlockToSector:= Blocks * 2;
       1024 : BlockToSector:= Blocks;
       2048 : BlockToSector:= Blocks div 2;
       4096 : BlockToSector:= Blocks div 4;
       8192 : BlockToSector:= Blocks div 8;
  end;
end; //end function BlockToSector
{-------------------------------------------------------------------------------
PROCEDURE SWAPON
Introduced in V42, either turns swap on or swap off where the action is determined by OnOff. Previously
swap was turned off in either the iso or the deb. This procedure brings it within foxclone. This is
required because on boot systemd will use any swap partition it finds. If this is on a partition that
is either being restored or cloned to, sfdisk will be unable to complete correctly - the kernel will not
be able to re-read the partition table because it is in use. The net result is that the restore/clone
will complete but it won't work. This is needed more for the deb than the iso.

Note - needs a polkit for swapon and swapoff.

Parameters:
  OnOff = true then turn swap on, if false turn swap off.

Used by: foxcloneXX

V1.0 - 03 Oct 2020, original version in foxclone < 50
V1.1 - 13 Nov 2021, minor mods, termcmd does not create the process in foxclone50.
-------------------------------------------------------------------------------}
procedure SwapOn(OnOff : boolean);
const
  bufsize = 64; // Buffer size for reading output
var
  swapProcess   : TProcess;
  buffer,action : string;
  BytesRead     : integer;
begin
  swapProcess:= TProcess.Create(nil);
  setlength(buffer,bufsize);
  if OnOff then action:= 'pkexec swapon -a' else action:= 'pkexec swapoff -a';
  termCmd(action,swapProcess);
  //assume output in one hit
  BytesRead:= swapProcess.Output.Read(buffer[1],length(buffer));
  swapProcess.free;
end; //end procedure SwapOn
{-------------------------------------------------------------------------------
FUNCTION GPARTEDCHECK
In testing found that if gparted was running when sfdisk restored a partition table it would give
the output:

    The partition table has been altered.
    Calling ioctl() to re-read partition table.
    The kernel still uses the old table. The new table will be used at the next reboot or after
    you run partprobe(8) or kpartx(8).

This is in spite of sfdisk reporting "Checking that no-one is using this disk right now ... OK"

Not convinced that partprobe will fix the problem, so gpartedcheck looks to see if gparted is running
and generates an error message if it is.

The function returns true if gparted is running, false otherwise.

Used by:

V1.0 - 05 Jan 2022
-------------------------------------------------------------------------------}
function gpartedCheck : boolean;
const
  gpartedstr = 'ps -A | grep gparted';
var
  Gparted : string;
begin
  Gparted:= GetStr(gpartedstr);
  if length(GParted) > 0 then
     begin
       gpartedCheck:= true;
       foxmsg(rGpartedTitle,[rGpartedError1,rGpartedError2],mtError,mbOK);
     end
  else gpartedCheck:= false;
end;
{-------------------------------------------------------------------------------
FUNCTION FORMATMB
Returns a string showing the size in MB.

Parameters:
    Blocks : the size to convert into MB, is assumed to 1024 byte blocks.

Was using blocks to display detail in hint strings and error messages. Considered to be too detailed
by user so changed to displaying size in MB. This should give sufficient granularity when foxclone
reports that a destination is too small for a backup or clone.

Used by:

V1.0 - 05 Jan 2022
-------------------------------------------------------------------------------}
function FormatMB(Blocks : int64) : string;
begin
  FormatMB:= FormatFloat('#,##0MB',Blocks * 1024/1e6);
end;
//runs vtoydump64, returns boot device, e.g. sdc
{-------------------------------------------------------------------------------
FUNCTION FINDVENTOY
Uses vtoydump to find the drive that is booting ventoy and hence foxclone

Running ventoy generates this output (if foxclone was booted from ventoy):

     ubuntu@ubuntu ~ $ sudo vtoydump64
     === ventoy runtime data ===
     disk name : /dev/sdc
     disk size : 30972837888
     disk part : 1
     filesystem: exfat
     image size: 930086912
     image path: /foxclone50-07-22.iso

This is loaded into a TStringList and searched for the line begining "disk name". The drive is then
parsed out of the line and returned by the function.

If not booting ventoy and this function is called, an empty string is returned.

Used by:

V1.0 - 02 Feb 2022
-------------------------------------------------------------------------------}
function FindVentoy : string;
const
  VenStr = 'disk name : /dev/';
var
  VentoyOut : TStringList;
  i : byte;
begin
  FindVentoy:= ''; //nil output
  VentoyOut:= TStringList.Create;
  GetText('pkexec vtoydump64',VentoyOut);
  if VentoyOut.Count > 0 then
     for i:= 0 to VentoyOut.Count - 1 do
         if pos(VenStr,VentoyOut[i]) > 0 then FindVentoy:= trim(replacestr(VentoyOut[i],VenStr,''));
  VentoyOut.Free;
end;

end.

