{ FoxClone
  Copyright (C) 2021 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}
{-------------------------------------------------------------------------------
UNIT - VERIFY

The objects and methods for verify. The verify panel is a child of the restore panel. No additional
objects are required so the unit defines the methods required for verify - essentially selecting which
backup files should be checked and running partclone verify on them.

V1.0 - 22 Jun 2021
-------------------------------------------------------------------------------}
unit verify;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, CheckLst, StdCtrls, Forms, Controls, dialogs, strutils, DateUtils, Graphics,
  restore, basepanel, backup, clone, GTKlabel, drives, partitions, utilities,
  il8nstrings, fdialog;

type
  TVerify = class(TRestore)
    private
      procedure Box1CK(Sender: TObject);
      procedure Box3CK(Sender: TObject);
      procedure NextBtnCK(Sender: TObject);
    public
      constructor Create(TheOwner: TComponent);
  end;

var
  VerifyTab : TVerify;

implementation
uses partclone;
{-------------------------------------------------------------------------------
CONSTRUCTOR TVERIFY.CREATE
Creates the verify panel and adds in the properties, functions & procedures specific to clone.

verifytab is an amended copy of restoretab.

Used by:

V1.0 - 22 Jun 2021
-------------------------------------------------------------------------------}
constructor TVerify.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  Box1.BoxType:= veBox1;
  Box2.BoxType:= veBox2;
  Box3.BoxType:= veBox3;
  TitleLbl.setCaption(rVerifyTitleStr);
  ActionImg.Picture.LoadFromFile(VerifyImgFile);
  //re-using box4 label as a prompt to select backup files to verify
  with Box4Lbl do begin
       setCaption(rVerifyBox4Str);
       AnchorSide[akTop].Side:= asrCenter;
       AnchorSide[akTop].Control := Box3;
       Enabled:= true;
       Visible:= false;
  end;
  freeandnil(LBox4); //not needed in verify
  with NextBtn do begin
       Caption:= rVerifyTabStr;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control := Box3;
  end;
  //event handlers
  Box1.OnClick:= @Box1CK;
  Box3.OnClick:= @Box3CK;
  Box3.OnClickCheck:= @Box3CheckCK; //fix for bug in lazarus, not firing onclick on second checkbox click
  NoteBtn.OnClick:= @NoteCK;
  NextBtn.OnClick:= @NextBtnCK;
end; //end constructor TVerify.Create
{-------------------------------------------------------------------------------
PROCEDURE TVERIFY.BOX1CK
Event handler for clicks on box1.

Modified version of TRestore.Box1CK.

Modifications are:
   resets restore tab
   no listbox 4.

Only a single drive may be selected.

On a valid click that selects a drive:
   sets that drive as the source for the backup file
   populates box2 with the partitions on the drive

If the user clicks on blank space - nothing happens.

Note -  clicking on either the check box or the text triggers this procedure, but if clicking on text it
does not change the state of the checkbox, clicking on the checkbox does change its state. Went round
in circles a few times trying to sort out the logic of this. The initial code on saving the checkbox
state and then resetting it works, but not altogether sure how.

Used by:

V1.0 - 20 Jun 2021
-------------------------------------------------------------------------------}
procedure TVerify.Box1CK(Sender: TObject);
var
  Source : shortint;
  dindex : shortint;
begin
  //clear all other tabs
  BackTab.ClearAll;
  RestoreTab.ClearAll;
  CloneTab.ClearAll;
  MyDrives.unSetAll(acNone);
  Source:= Box1.ItemIndex;
  if Source < 0 then exit;
  clearBox(2);       //clear all the other boxes
  clearBox(3);
  ShowDateAndNote(false); //make backup date and note button invisible
  NextBtn.Enabled:= false; //reset the save to file button
  Box1.CheckAll(cbUnchecked); //only one can be selected
  Box2Lbl.Enabled:= true;
  Box1.Checked[Source]:= not Box1.CheckState[Source];
  Box1.CheckState[Source]:= Box1.Checked[Source];
  if Box1.Checked[Source] then
     begin
       dindex:= MyDrives.FindName(Box1.Items[Source]);
       if dindex = -1 then SystemError(12,Box1.Items[Source]); //close foxclone on error
       MyDrives[dindex].setAction:= acSource; //set as the drive containing the backup
       PopulateBox2; //populate box2 with partitions in MyDrives[dindex]
     end;
end; //end procedure TVerify.Box1CK
{-------------------------------------------------------------------------------
PROCEDURE TVERIFY.BOX3CK
The event handler for OnClick for box3.

Multiple partitions may be selected, clicking invokes three checks:

     does the partition exist in the backup set - if not backed up cannot be selected.

     is the partition an extended partition - cannot be selected.

     is the partition a swap partition - cannot be selected.

Used by:

V1.0 - 22 Jun 2021
V1.1 - 13 Oct 2021, const space moved to utilities
V1.2 - 04 Dec 2021, box3ck allows you to select an unknown partition type, fixed.
-------------------------------------------------------------------------------}
procedure TVerify.Box3CK(Sender: TObject);
const
  ExtendStr = 'extended';
  SwapStr = 'swap';
  PartStr = '[partition]';
var
  Source, i : byte;
  pIndex : shortint;
  Msg1 : string;
begin
  //writeln('box3 clicked on item ',Box3.ItemIndex);
  if Box3.ItemIndex < 0 then exit; //need to add >= box.count
  Source:= Box3.ItemIndex;
  //a checkbox click may have flipped the state so reset (observed in testing)
  Box3.Checked[Source]:= Box3.CheckState[Source];
  with Box3 do //a set of if statements each exiting the procedure
     begin
       //find the partition - it should match Source, but bolt and braces
       pIndex:= RDrive.Partitions.FindName(Items[Source]);
       if pIndex = -1 then SystemError(23,Items[Source]);
       //was the partition backed up?
       if not RDrive.Partitions[pIndex].selected then //the partition was not backed up
          begin
            Checked[Source]:= false;
            CheckState[Source]:= false; //these should be no change
            with RDrive.Partitions[pIndex] do Msg1:= replacestr(rNoVerifyBackupStr,PartStr,Drive + Number);
            FoxMsg(rNoBackupStr,[Msg1 + Space + rNoVerifyStr],mtInformation,[mbOK]);
            exit;
          end;
       //is it an extended partition?
       if pos(ExtendStr,Items[Source]) > 0 then //extended part follows setting of the part table
          begin
            Checked[Source]:= false;
            CheckState[Source]:= false;
            FoxMsg(rExttitle,[rExtMsg + Space + rNoVerifyStr],mtInformation,[mbOK]);
            exit;
          end;
       //is it a swap partition?
       if pos(SwapStr,Items[Source]) > 0 then //extended part follows setting of the part table
          begin
            Checked[Source]:= false;
            CheckState[Source]:= false;
            FoxMsg(rSwapStr,[rSwapMsgStr + Space + rNoVerifyStr],mtInformation,[mbOK]);
            exit;
          end;
       //is it an unknown filesystem
       if RDrive.Partitions[pIndex].FileSystem = 'unknown' then
          begin
            Checked[Source]:= false;
            CheckState[Source]:= false;
            with RDrive.Partitions[pIndex] do
                 FoxMsg(rSwapStr,[replacestr(rUnknownStr,PartStr,Drive + Number),rNoVerifyStr],mtInformation,[mbOK]);
            exit;
          end;
       //a standard partition, state can be changed
       Checked[Source]:= not CheckState[Source];
       CheckState[Source]:= Checked[Source];
       //reset the verify button
       NextBtn.Enabled:= false;
       //and set the verify button
       for i:= 0 to Count - 1 do if Checked[i] then
          begin
            NextBtn.Enabled:= true;
            exit;
          end
     end
end; //end procedure TVerify.Box3CK
{-------------------------------------------------------------------------------
PROCEDURE TVERIFY.BOX3CK
The event handler for clicking on the next button, just calls runfoxclone.

Used by:

V1.0 - 22 Jun 2021
-------------------------------------------------------------------------------}
procedure TVerify.NextBtnCK(Sender: TObject);
begin
  runFoxClone(acVerify,RestFname);
end;

end.

